# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220518

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # This class provides a wrapper around {OCI::AccessGovernanceCp::AccessGovernanceCPClient} and offers convenience methods
  # for operations that would otherwise need to be chained together. For example, instead of performing an action
  # on a resource (e.g. launching an instance, creating a load balancer) and then using a waiter to wait for the resource
  # to enter a given state, you can call a single method in this class to accomplish the same functionality
  class AccessGovernanceCp::AccessGovernanceCPClientCompositeOperations
    # The {OCI::AccessGovernanceCp::AccessGovernanceCPClient} used to communicate with the service_client
    #
    # @return [OCI::AccessGovernanceCp::AccessGovernanceCPClient]
    attr_reader :service_client

    # Initializes a new AccessGovernanceCPClientCompositeOperations
    #
    # @param [OCI::AccessGovernanceCp::AccessGovernanceCPClient] service_client The client used to communicate with the service.
    #   Defaults to a new service client created via {OCI::AccessGovernanceCp::AccessGovernanceCPClient#initialize} with no arguments
    def initialize(service_client = OCI::AccessGovernanceCp::AccessGovernanceCPClient.new)
      @service_client = service_client
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::AccessGovernanceCp::AccessGovernanceCPClient#create_governance_instance} and then waits for the {OCI::AccessGovernanceCp::Models::GovernanceInstance} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::AccessGovernanceCp::Models::CreateGovernanceInstanceDetails] create_governance_instance_details The details of a new GovernanceInstance.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::AccessGovernanceCp::Models::GovernanceInstance#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::AccessGovernanceCp::AccessGovernanceCPClient#create_governance_instance}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::AccessGovernanceCp::Models::GovernanceInstance}
    def create_governance_instance_and_wait_for_state(create_governance_instance_details, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.create_governance_instance(create_governance_instance_details, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_governance_instance(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::AccessGovernanceCp::AccessGovernanceCPClient#delete_governance_instance} and then waits for the {OCI::AccessGovernanceCp::Models::GovernanceInstance} acted upon
    # to enter the given state(s).
    #
    # @param [String] governance_instance_id The OCID of the GovernanceInstance
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::AccessGovernanceCp::Models::GovernanceInstance#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::AccessGovernanceCp::AccessGovernanceCPClient#delete_governance_instance}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type nil
    def delete_governance_instance_and_wait_for_state(governance_instance_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      initial_get_result = @service_client.get_governance_instance(governance_instance_id)
      operation_result = @service_client.delete_governance_instance(governance_instance_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)

      begin
        waiter_result = initial_get_result.wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200,
          succeed_on_not_found: true
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::AccessGovernanceCp::AccessGovernanceCPClient#update_governance_instance} and then waits for the {OCI::AccessGovernanceCp::Models::GovernanceInstance} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::AccessGovernanceCp::Models::UpdateGovernanceInstanceDetails] update_governance_instance_details The details of the GovernanceInstance to be updated.
    # @param [String] governance_instance_id The OCID of the GovernanceInstance
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::AccessGovernanceCp::Models::GovernanceInstance#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::AccessGovernanceCp::AccessGovernanceCPClient#update_governance_instance}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::AccessGovernanceCp::Models::GovernanceInstance}
    def update_governance_instance_and_wait_for_state(update_governance_instance_details, governance_instance_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.update_governance_instance(update_governance_instance_details, governance_instance_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_governance_instance(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
