# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220421
require 'date'
require 'logger'
require_relative 'scm_configuration'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # An external SCM configuration extends a SCM Configuration with necessary data to reach and use the Source Code Management tool/platform used by a Remediation Recipe.
  # An external SCM in ADM refers to GitHub, or GitLab.
  #
  class Adm::Models::ExternalScmConfiguration < Adm::Models::ScmConfiguration
    EXTERNAL_SCM_TYPE_ENUM = [
      EXTERNAL_SCM_TYPE_GITHUB = 'GITHUB'.freeze,
      EXTERNAL_SCM_TYPE_GITLAB = 'GITLAB'.freeze,
      EXTERNAL_SCM_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The type of External Source Code Management.
    # @return [String]
    attr_reader :external_scm_type

    # **[Required]** The repository URL for the SCM.
    # For Non-Enterprise GitHub the expected format is https://github.com/[owner]/[repoName]
    # For Enterprise GitHub the expected format is http(s)://[hostname]/api/v3/repos/[owner]/[repoName]
    # For GitLab the expected format is https://gitlab.com/[groupName]/[repoName]
    #
    # @return [String]
    attr_accessor :repository_url

    # The username for the SCM (to perform operations such as cloning or pushing via HTTP).
    # @return [String]
    attr_accessor :username

    # **[Required]** The Oracle Cloud Identifier ([OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm)) of the Private Access Token (PAT) Secret.
    # The secret provides the credentials necessary to authenticate against the SCM.
    #
    # @return [String]
    attr_accessor :pat_secret_id

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'scm_type': :'scmType',
        'branch': :'branch',
        'build_file_location': :'buildFileLocation',
        'is_automerge_enabled': :'isAutomergeEnabled',
        'external_scm_type': :'externalScmType',
        'repository_url': :'repositoryUrl',
        'username': :'username',
        'pat_secret_id': :'patSecretId'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'scm_type': :'String',
        'branch': :'String',
        'build_file_location': :'String',
        'is_automerge_enabled': :'BOOLEAN',
        'external_scm_type': :'String',
        'repository_url': :'String',
        'username': :'String',
        'pat_secret_id': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :branch The value to assign to the {OCI::Adm::Models::ScmConfiguration#branch #branch} proprety
    # @option attributes [String] :build_file_location The value to assign to the {OCI::Adm::Models::ScmConfiguration#build_file_location #build_file_location} proprety
    # @option attributes [BOOLEAN] :is_automerge_enabled The value to assign to the {OCI::Adm::Models::ScmConfiguration#is_automerge_enabled #is_automerge_enabled} proprety
    # @option attributes [String] :external_scm_type The value to assign to the {#external_scm_type} property
    # @option attributes [String] :repository_url The value to assign to the {#repository_url} property
    # @option attributes [String] :username The value to assign to the {#username} property
    # @option attributes [String] :pat_secret_id The value to assign to the {#pat_secret_id} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['scmType'] = 'EXTERNAL_SCM'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.external_scm_type = attributes[:'externalScmType'] if attributes[:'externalScmType']

      raise 'You cannot provide both :externalScmType and :external_scm_type' if attributes.key?(:'externalScmType') && attributes.key?(:'external_scm_type')

      self.external_scm_type = attributes[:'external_scm_type'] if attributes[:'external_scm_type']

      self.repository_url = attributes[:'repositoryUrl'] if attributes[:'repositoryUrl']

      raise 'You cannot provide both :repositoryUrl and :repository_url' if attributes.key?(:'repositoryUrl') && attributes.key?(:'repository_url')

      self.repository_url = attributes[:'repository_url'] if attributes[:'repository_url']

      self.username = attributes[:'username'] if attributes[:'username']

      self.pat_secret_id = attributes[:'patSecretId'] if attributes[:'patSecretId']

      raise 'You cannot provide both :patSecretId and :pat_secret_id' if attributes.key?(:'patSecretId') && attributes.key?(:'pat_secret_id')

      self.pat_secret_id = attributes[:'pat_secret_id'] if attributes[:'pat_secret_id']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] external_scm_type Object to be assigned
    def external_scm_type=(external_scm_type)
      # rubocop:disable Style/ConditionalAssignment
      if external_scm_type && !EXTERNAL_SCM_TYPE_ENUM.include?(external_scm_type)
        OCI.logger.debug("Unknown value for 'external_scm_type' [" + external_scm_type + "]. Mapping to 'EXTERNAL_SCM_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @external_scm_type = EXTERNAL_SCM_TYPE_UNKNOWN_ENUM_VALUE
      else
        @external_scm_type = external_scm_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        scm_type == other.scm_type &&
        branch == other.branch &&
        build_file_location == other.build_file_location &&
        is_automerge_enabled == other.is_automerge_enabled &&
        external_scm_type == other.external_scm_type &&
        repository_url == other.repository_url &&
        username == other.username &&
        pat_secret_id == other.pat_secret_id
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [scm_type, branch, build_file_location, is_automerge_enabled, external_scm_type, repository_url, username, pat_secret_id].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
