# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20231107

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # This class provides a wrapper around {OCI::CapacityManagement::CapacityManagementClient} and offers convenience methods
  # for operations that would otherwise need to be chained together. For example, instead of performing an action
  # on a resource (e.g. launching an instance, creating a load balancer) and then using a waiter to wait for the resource
  # to enter a given state, you can call a single method in this class to accomplish the same functionality
  class CapacityManagement::CapacityManagementClientCompositeOperations
    # The {OCI::CapacityManagement::CapacityManagementClient} used to communicate with the service_client
    #
    # @return [OCI::CapacityManagement::CapacityManagementClient]
    attr_reader :service_client

    # Initializes a new CapacityManagementClientCompositeOperations
    #
    # @param [OCI::CapacityManagement::CapacityManagementClient] service_client The client used to communicate with the service.
    #   Defaults to a new service client created via {OCI::CapacityManagement::CapacityManagementClient#initialize} with no arguments
    def initialize(service_client = OCI::CapacityManagement::CapacityManagementClient.new)
      @service_client = service_client
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::CapacityManagement::CapacityManagementClient#create_occ_availability_catalog} and then waits for the {OCI::CapacityManagement::Models::OccAvailabilityCatalog} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::CapacityManagement::Models::CreateOccAvailabilityCatalogDetails] create_occ_availability_catalog_details The request details for creating an availability catalog.
    #
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::CapacityManagement::Models::OccAvailabilityCatalog#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::CapacityManagement::CapacityManagementClient#create_occ_availability_catalog}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::CapacityManagement::Models::OccAvailabilityCatalog}
    def create_occ_availability_catalog_and_wait_for_state(create_occ_availability_catalog_details, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.create_occ_availability_catalog(create_occ_availability_catalog_details, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_occ_availability_catalog(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::CapacityManagement::CapacityManagementClient#create_occ_capacity_request} and then waits for the {OCI::CapacityManagement::Models::OccCapacityRequest} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::CapacityManagement::Models::CreateOccCapacityRequestDetails] create_occ_capacity_request_details The request details for creating a capacity request.
    #
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::CapacityManagement::Models::OccCapacityRequest#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::CapacityManagement::CapacityManagementClient#create_occ_capacity_request}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::CapacityManagement::Models::OccCapacityRequest}
    def create_occ_capacity_request_and_wait_for_state(create_occ_capacity_request_details, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.create_occ_capacity_request(create_occ_capacity_request_details, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_occ_capacity_request(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::CapacityManagement::CapacityManagementClient#delete_occ_availability_catalog} and then waits for the {OCI::CapacityManagement::Models::OccAvailabilityCatalog} acted upon
    # to enter the given state(s).
    #
    # @param [String] occ_availability_catalog_id The OCID of the availability catalog.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::CapacityManagement::Models::OccAvailabilityCatalog#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::CapacityManagement::CapacityManagementClient#delete_occ_availability_catalog}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type nil
    def delete_occ_availability_catalog_and_wait_for_state(occ_availability_catalog_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      initial_get_result = @service_client.get_occ_availability_catalog(occ_availability_catalog_id)
      operation_result = @service_client.delete_occ_availability_catalog(occ_availability_catalog_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)

      begin
        waiter_result = initial_get_result.wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200,
          succeed_on_not_found: true
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::CapacityManagement::CapacityManagementClient#delete_occ_capacity_request} and then waits for the {OCI::CapacityManagement::Models::OccCapacityRequest} acted upon
    # to enter the given state(s).
    #
    # @param [String] occ_capacity_request_id The OCID of the capacity request.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::CapacityManagement::Models::OccCapacityRequest#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::CapacityManagement::CapacityManagementClient#delete_occ_capacity_request}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type nil
    def delete_occ_capacity_request_and_wait_for_state(occ_capacity_request_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      initial_get_result = @service_client.get_occ_capacity_request(occ_capacity_request_id)
      operation_result = @service_client.delete_occ_capacity_request(occ_capacity_request_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)

      begin
        waiter_result = initial_get_result.wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200,
          succeed_on_not_found: true
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::CapacityManagement::CapacityManagementClient#update_occ_availability_catalog} and then waits for the {OCI::CapacityManagement::Models::OccAvailabilityCatalog} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::CapacityManagement::Models::UpdateOccAvailabilityCatalogDetails] update_occ_availability_catalog_details Request to update the properties of the availability catalog.
    # @param [String] occ_availability_catalog_id The OCID of the availability catalog.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::CapacityManagement::Models::OccAvailabilityCatalog#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::CapacityManagement::CapacityManagementClient#update_occ_availability_catalog}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::CapacityManagement::Models::OccAvailabilityCatalog}
    def update_occ_availability_catalog_and_wait_for_state(update_occ_availability_catalog_details, occ_availability_catalog_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.update_occ_availability_catalog(update_occ_availability_catalog_details, occ_availability_catalog_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_occ_availability_catalog(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::CapacityManagement::CapacityManagementClient#update_occ_capacity_request} and then waits for the {OCI::CapacityManagement::Models::OccCapacityRequest} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::CapacityManagement::Models::UpdateOccCapacityRequestDetails] update_occ_capacity_request_details Request to update the properties of the capacity request.
    # @param [String] occ_capacity_request_id The OCID of the capacity request.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::CapacityManagement::Models::OccCapacityRequest#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::CapacityManagement::CapacityManagementClient#update_occ_capacity_request}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::CapacityManagement::Models::OccCapacityRequest}
    def update_occ_capacity_request_and_wait_for_state(update_occ_capacity_request_details, occ_capacity_request_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.update_occ_capacity_request(update_occ_capacity_request_details, occ_capacity_request_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_occ_capacity_request(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
