# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200131
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details of cloud guard resource
  class CloudGuard::Models::Resource
    RISK_LEVEL_ENUM = [
      RISK_LEVEL_CRITICAL = 'CRITICAL'.freeze,
      RISK_LEVEL_HIGH = 'HIGH'.freeze,
      RISK_LEVEL_MEDIUM = 'MEDIUM'.freeze,
      RISK_LEVEL_LOW = 'LOW'.freeze,
      RISK_LEVEL_MINOR = 'MINOR'.freeze,
      RISK_LEVEL_NONE = 'NONE'.freeze,
      RISK_LEVEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Ocid for CG resource
    # @return [String]
    attr_accessor :id

    # Name for the CG resource
    # @return [String]
    attr_accessor :resource_name

    # **[Required]** resource type of the CG resource
    # @return [String]
    attr_accessor :resource_type

    # **[Required]** CompartmentId of CG Resource
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** TargetId of CG Resource
    # @return [String]
    attr_accessor :target_id

    # Target name for the CG Resource
    # @return [String]
    attr_accessor :target_name

    # **[Required]** region of CG Resource
    # @return [String]
    attr_accessor :region

    # The Risk Level
    # @return [String]
    attr_reader :risk_level

    # First monitored time
    # @return [DateTime]
    attr_accessor :time_first_monitored

    # Last monitored time
    # @return [DateTime]
    attr_accessor :time_last_monitored

    # Count of existing problems for a resource
    # @return [Integer]
    attr_accessor :problem_count

    # Count of existing number of vulnerabilities in the resource
    # @return [Integer]
    attr_accessor :vulnerability_count

    # Number of open ports in a resource
    # @return [Integer]
    attr_accessor :open_ports_count

    # @return [OCI::CloudGuard::Models::ResourceAdditionalDetails]
    attr_accessor :additional_details

    # Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
    # Example: `{\"bar-key\": \"value\"}`
    #
    # Avoid entering confidential information.
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # System tags for this resource. Each key is predefined and scoped to a namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # System tags can be viewed by users, but can only be created by the system.
    #
    # Example: `{\"orcl-cloud\": {\"free-tier-retained\": \"true\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :system_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'resource_name': :'resourceName',
        'resource_type': :'resourceType',
        'compartment_id': :'compartmentId',
        'target_id': :'targetId',
        'target_name': :'targetName',
        'region': :'region',
        'risk_level': :'riskLevel',
        'time_first_monitored': :'timeFirstMonitored',
        'time_last_monitored': :'timeLastMonitored',
        'problem_count': :'problemCount',
        'vulnerability_count': :'vulnerabilityCount',
        'open_ports_count': :'openPortsCount',
        'additional_details': :'additionalDetails',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'system_tags': :'systemTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'resource_name': :'String',
        'resource_type': :'String',
        'compartment_id': :'String',
        'target_id': :'String',
        'target_name': :'String',
        'region': :'String',
        'risk_level': :'String',
        'time_first_monitored': :'DateTime',
        'time_last_monitored': :'DateTime',
        'problem_count': :'Integer',
        'vulnerability_count': :'Integer',
        'open_ports_count': :'Integer',
        'additional_details': :'OCI::CloudGuard::Models::ResourceAdditionalDetails',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'system_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :resource_name The value to assign to the {#resource_name} property
    # @option attributes [String] :resource_type The value to assign to the {#resource_type} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :target_id The value to assign to the {#target_id} property
    # @option attributes [String] :target_name The value to assign to the {#target_name} property
    # @option attributes [String] :region The value to assign to the {#region} property
    # @option attributes [String] :risk_level The value to assign to the {#risk_level} property
    # @option attributes [DateTime] :time_first_monitored The value to assign to the {#time_first_monitored} property
    # @option attributes [DateTime] :time_last_monitored The value to assign to the {#time_last_monitored} property
    # @option attributes [Integer] :problem_count The value to assign to the {#problem_count} property
    # @option attributes [Integer] :vulnerability_count The value to assign to the {#vulnerability_count} property
    # @option attributes [Integer] :open_ports_count The value to assign to the {#open_ports_count} property
    # @option attributes [OCI::CloudGuard::Models::ResourceAdditionalDetails] :additional_details The value to assign to the {#additional_details} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :system_tags The value to assign to the {#system_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.resource_name = attributes[:'resourceName'] if attributes[:'resourceName']

      raise 'You cannot provide both :resourceName and :resource_name' if attributes.key?(:'resourceName') && attributes.key?(:'resource_name')

      self.resource_name = attributes[:'resource_name'] if attributes[:'resource_name']

      self.resource_type = attributes[:'resourceType'] if attributes[:'resourceType']

      raise 'You cannot provide both :resourceType and :resource_type' if attributes.key?(:'resourceType') && attributes.key?(:'resource_type')

      self.resource_type = attributes[:'resource_type'] if attributes[:'resource_type']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.target_id = attributes[:'targetId'] if attributes[:'targetId']

      raise 'You cannot provide both :targetId and :target_id' if attributes.key?(:'targetId') && attributes.key?(:'target_id')

      self.target_id = attributes[:'target_id'] if attributes[:'target_id']

      self.target_name = attributes[:'targetName'] if attributes[:'targetName']

      raise 'You cannot provide both :targetName and :target_name' if attributes.key?(:'targetName') && attributes.key?(:'target_name')

      self.target_name = attributes[:'target_name'] if attributes[:'target_name']

      self.region = attributes[:'region'] if attributes[:'region']

      self.risk_level = attributes[:'riskLevel'] if attributes[:'riskLevel']

      raise 'You cannot provide both :riskLevel and :risk_level' if attributes.key?(:'riskLevel') && attributes.key?(:'risk_level')

      self.risk_level = attributes[:'risk_level'] if attributes[:'risk_level']

      self.time_first_monitored = attributes[:'timeFirstMonitored'] if attributes[:'timeFirstMonitored']

      raise 'You cannot provide both :timeFirstMonitored and :time_first_monitored' if attributes.key?(:'timeFirstMonitored') && attributes.key?(:'time_first_monitored')

      self.time_first_monitored = attributes[:'time_first_monitored'] if attributes[:'time_first_monitored']

      self.time_last_monitored = attributes[:'timeLastMonitored'] if attributes[:'timeLastMonitored']

      raise 'You cannot provide both :timeLastMonitored and :time_last_monitored' if attributes.key?(:'timeLastMonitored') && attributes.key?(:'time_last_monitored')

      self.time_last_monitored = attributes[:'time_last_monitored'] if attributes[:'time_last_monitored']

      self.problem_count = attributes[:'problemCount'] if attributes[:'problemCount']

      raise 'You cannot provide both :problemCount and :problem_count' if attributes.key?(:'problemCount') && attributes.key?(:'problem_count')

      self.problem_count = attributes[:'problem_count'] if attributes[:'problem_count']

      self.vulnerability_count = attributes[:'vulnerabilityCount'] if attributes[:'vulnerabilityCount']

      raise 'You cannot provide both :vulnerabilityCount and :vulnerability_count' if attributes.key?(:'vulnerabilityCount') && attributes.key?(:'vulnerability_count')

      self.vulnerability_count = attributes[:'vulnerability_count'] if attributes[:'vulnerability_count']

      self.open_ports_count = attributes[:'openPortsCount'] if attributes[:'openPortsCount']

      raise 'You cannot provide both :openPortsCount and :open_ports_count' if attributes.key?(:'openPortsCount') && attributes.key?(:'open_ports_count')

      self.open_ports_count = attributes[:'open_ports_count'] if attributes[:'open_ports_count']

      self.additional_details = attributes[:'additionalDetails'] if attributes[:'additionalDetails']

      raise 'You cannot provide both :additionalDetails and :additional_details' if attributes.key?(:'additionalDetails') && attributes.key?(:'additional_details')

      self.additional_details = attributes[:'additional_details'] if attributes[:'additional_details']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.system_tags = attributes[:'systemTags'] if attributes[:'systemTags']

      raise 'You cannot provide both :systemTags and :system_tags' if attributes.key?(:'systemTags') && attributes.key?(:'system_tags')

      self.system_tags = attributes[:'system_tags'] if attributes[:'system_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] risk_level Object to be assigned
    def risk_level=(risk_level)
      # rubocop:disable Style/ConditionalAssignment
      if risk_level && !RISK_LEVEL_ENUM.include?(risk_level)
        OCI.logger.debug("Unknown value for 'risk_level' [" + risk_level + "]. Mapping to 'RISK_LEVEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @risk_level = RISK_LEVEL_UNKNOWN_ENUM_VALUE
      else
        @risk_level = risk_level
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        resource_name == other.resource_name &&
        resource_type == other.resource_type &&
        compartment_id == other.compartment_id &&
        target_id == other.target_id &&
        target_name == other.target_name &&
        region == other.region &&
        risk_level == other.risk_level &&
        time_first_monitored == other.time_first_monitored &&
        time_last_monitored == other.time_last_monitored &&
        problem_count == other.problem_count &&
        vulnerability_count == other.vulnerability_count &&
        open_ports_count == other.open_ports_count &&
        additional_details == other.additional_details &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        system_tags == other.system_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, resource_name, resource_type, compartment_id, target_id, target_name, region, risk_level, time_first_monitored, time_last_monitored, problem_count, vulnerability_count, open_ports_count, additional_details, freeform_tags, defined_tags, system_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
