# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200131
require 'date'
require_relative 'target_details'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Target details for a security zone target.
  class CloudGuard::Models::SecurityZoneTargetDetails < CloudGuard::Models::TargetDetails
    # The OCID of the security zone to associate with this compartment
    # @return [String]
    attr_accessor :security_zone_id

    # The name of the security zone to associate with this compartment.
    # @return [String]
    attr_accessor :security_zone_display_name

    # The list of security zone recipes to associate with this compartment
    # @return [Array<OCI::CloudGuard::Models::SecurityRecipe>]
    attr_accessor :target_security_zone_recipes

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'target_resource_type': :'targetResourceType',
        'security_zone_id': :'securityZoneId',
        'security_zone_display_name': :'securityZoneDisplayName',
        'target_security_zone_recipes': :'targetSecurityZoneRecipes'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'target_resource_type': :'String',
        'security_zone_id': :'String',
        'security_zone_display_name': :'String',
        'target_security_zone_recipes': :'Array<OCI::CloudGuard::Models::SecurityRecipe>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :security_zone_id The value to assign to the {#security_zone_id} property
    # @option attributes [String] :security_zone_display_name The value to assign to the {#security_zone_display_name} property
    # @option attributes [Array<OCI::CloudGuard::Models::SecurityRecipe>] :target_security_zone_recipes The value to assign to the {#target_security_zone_recipes} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['targetResourceType'] = 'SECURITY_ZONE'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.security_zone_id = attributes[:'securityZoneId'] if attributes[:'securityZoneId']

      raise 'You cannot provide both :securityZoneId and :security_zone_id' if attributes.key?(:'securityZoneId') && attributes.key?(:'security_zone_id')

      self.security_zone_id = attributes[:'security_zone_id'] if attributes[:'security_zone_id']

      self.security_zone_display_name = attributes[:'securityZoneDisplayName'] if attributes[:'securityZoneDisplayName']

      raise 'You cannot provide both :securityZoneDisplayName and :security_zone_display_name' if attributes.key?(:'securityZoneDisplayName') && attributes.key?(:'security_zone_display_name')

      self.security_zone_display_name = attributes[:'security_zone_display_name'] if attributes[:'security_zone_display_name']

      self.target_security_zone_recipes = attributes[:'targetSecurityZoneRecipes'] if attributes[:'targetSecurityZoneRecipes']

      raise 'You cannot provide both :targetSecurityZoneRecipes and :target_security_zone_recipes' if attributes.key?(:'targetSecurityZoneRecipes') && attributes.key?(:'target_security_zone_recipes')

      self.target_security_zone_recipes = attributes[:'target_security_zone_recipes'] if attributes[:'target_security_zone_recipes']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        target_resource_type == other.target_resource_type &&
        security_zone_id == other.security_zone_id &&
        security_zone_display_name == other.security_zone_display_name &&
        target_security_zone_recipes == other.target_security_zone_recipes
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [target_resource_type, security_zone_id, security_zone_display_name, target_security_zone_recipes].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
