# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200430
require 'date'
require_relative 'connection_summary'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The connection summary details for the HDFS data asset.
  class DataIntegration::Models::ConnectionSummaryFromHdfs < DataIntegration::Models::ConnectionSummary
    # **[Required]** The HDFS principal.
    # @return [String]
    attr_accessor :hdfs_principal

    # **[Required]** The HDFS Data Node principal.
    # @return [String]
    attr_accessor :data_node_principal

    # **[Required]** The HDFS Name Node principal.
    # @return [String]
    attr_accessor :name_node_principal

    # HDFS Realm name.
    # @return [String]
    attr_accessor :realm

    # The HDFS Key Distribution Center.
    # @return [String]
    attr_accessor :key_distribution_center

    # @return [OCI::DataIntegration::Models::SensitiveAttribute]
    attr_accessor :key_tab_content

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'model_type': :'modelType',
        'key': :'key',
        'model_version': :'modelVersion',
        'parent_ref': :'parentRef',
        'name': :'name',
        'description': :'description',
        'object_version': :'objectVersion',
        'object_status': :'objectStatus',
        'identifier': :'identifier',
        'primary_schema': :'primarySchema',
        'connection_properties': :'connectionProperties',
        'is_default': :'isDefault',
        'metadata': :'metadata',
        'key_map': :'keyMap',
        'hdfs_principal': :'hdfsPrincipal',
        'data_node_principal': :'dataNodePrincipal',
        'name_node_principal': :'nameNodePrincipal',
        'realm': :'realm',
        'key_distribution_center': :'keyDistributionCenter',
        'key_tab_content': :'keyTabContent'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'model_type': :'String',
        'key': :'String',
        'model_version': :'String',
        'parent_ref': :'OCI::DataIntegration::Models::ParentReference',
        'name': :'String',
        'description': :'String',
        'object_version': :'Integer',
        'object_status': :'Integer',
        'identifier': :'String',
        'primary_schema': :'OCI::DataIntegration::Models::Schema',
        'connection_properties': :'Array<OCI::DataIntegration::Models::ConnectionProperty>',
        'is_default': :'BOOLEAN',
        'metadata': :'OCI::DataIntegration::Models::ObjectMetadata',
        'key_map': :'Hash<String, String>',
        'hdfs_principal': :'String',
        'data_node_principal': :'String',
        'name_node_principal': :'String',
        'realm': :'String',
        'key_distribution_center': :'String',
        'key_tab_content': :'OCI::DataIntegration::Models::SensitiveAttribute'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :key The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#key #key} proprety
    # @option attributes [String] :model_version The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#model_version #model_version} proprety
    # @option attributes [OCI::DataIntegration::Models::ParentReference] :parent_ref The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#parent_ref #parent_ref} proprety
    # @option attributes [String] :name The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#name #name} proprety
    # @option attributes [String] :description The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#description #description} proprety
    # @option attributes [Integer] :object_version The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#object_version #object_version} proprety
    # @option attributes [Integer] :object_status The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#object_status #object_status} proprety
    # @option attributes [String] :identifier The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#identifier #identifier} proprety
    # @option attributes [OCI::DataIntegration::Models::Schema] :primary_schema The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#primary_schema #primary_schema} proprety
    # @option attributes [Array<OCI::DataIntegration::Models::ConnectionProperty>] :connection_properties The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#connection_properties #connection_properties} proprety
    # @option attributes [BOOLEAN] :is_default The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#is_default #is_default} proprety
    # @option attributes [OCI::DataIntegration::Models::ObjectMetadata] :metadata The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#metadata #metadata} proprety
    # @option attributes [Hash<String, String>] :key_map The value to assign to the {OCI::DataIntegration::Models::ConnectionSummary#key_map #key_map} proprety
    # @option attributes [String] :hdfs_principal The value to assign to the {#hdfs_principal} property
    # @option attributes [String] :data_node_principal The value to assign to the {#data_node_principal} property
    # @option attributes [String] :name_node_principal The value to assign to the {#name_node_principal} property
    # @option attributes [String] :realm The value to assign to the {#realm} property
    # @option attributes [String] :key_distribution_center The value to assign to the {#key_distribution_center} property
    # @option attributes [OCI::DataIntegration::Models::SensitiveAttribute] :key_tab_content The value to assign to the {#key_tab_content} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['modelType'] = 'HDFS_CONNECTION'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.hdfs_principal = attributes[:'hdfsPrincipal'] if attributes[:'hdfsPrincipal']

      raise 'You cannot provide both :hdfsPrincipal and :hdfs_principal' if attributes.key?(:'hdfsPrincipal') && attributes.key?(:'hdfs_principal')

      self.hdfs_principal = attributes[:'hdfs_principal'] if attributes[:'hdfs_principal']

      self.data_node_principal = attributes[:'dataNodePrincipal'] if attributes[:'dataNodePrincipal']

      raise 'You cannot provide both :dataNodePrincipal and :data_node_principal' if attributes.key?(:'dataNodePrincipal') && attributes.key?(:'data_node_principal')

      self.data_node_principal = attributes[:'data_node_principal'] if attributes[:'data_node_principal']

      self.name_node_principal = attributes[:'nameNodePrincipal'] if attributes[:'nameNodePrincipal']

      raise 'You cannot provide both :nameNodePrincipal and :name_node_principal' if attributes.key?(:'nameNodePrincipal') && attributes.key?(:'name_node_principal')

      self.name_node_principal = attributes[:'name_node_principal'] if attributes[:'name_node_principal']

      self.realm = attributes[:'realm'] if attributes[:'realm']

      self.key_distribution_center = attributes[:'keyDistributionCenter'] if attributes[:'keyDistributionCenter']

      raise 'You cannot provide both :keyDistributionCenter and :key_distribution_center' if attributes.key?(:'keyDistributionCenter') && attributes.key?(:'key_distribution_center')

      self.key_distribution_center = attributes[:'key_distribution_center'] if attributes[:'key_distribution_center']

      self.key_tab_content = attributes[:'keyTabContent'] if attributes[:'keyTabContent']

      raise 'You cannot provide both :keyTabContent and :key_tab_content' if attributes.key?(:'keyTabContent') && attributes.key?(:'key_tab_content')

      self.key_tab_content = attributes[:'key_tab_content'] if attributes[:'key_tab_content']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        model_type == other.model_type &&
        key == other.key &&
        model_version == other.model_version &&
        parent_ref == other.parent_ref &&
        name == other.name &&
        description == other.description &&
        object_version == other.object_version &&
        object_status == other.object_status &&
        identifier == other.identifier &&
        primary_schema == other.primary_schema &&
        connection_properties == other.connection_properties &&
        is_default == other.is_default &&
        metadata == other.metadata &&
        key_map == other.key_map &&
        hdfs_principal == other.hdfs_principal &&
        data_node_principal == other.data_node_principal &&
        name_node_principal == other.name_node_principal &&
        realm == other.realm &&
        key_distribution_center == other.key_distribution_center &&
        key_tab_content == other.key_tab_content
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [model_type, key, model_version, parent_ref, name, description, object_version, object_status, identifier, primary_schema, connection_properties, is_default, metadata, key_map, hdfs_principal, data_node_principal, name_node_principal, realm, key_distribution_center, key_tab_content].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
