# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200430
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details of copy object.
  class DataIntegration::Models::CreateCopyObjectRequestDetails
    # **[Required]** The workspace id of the source from where we need to copy object.
    # @return [String]
    attr_accessor :source_workspace_id

    # **[Required]** The list of the objects to be copied.
    # @return [Array<String>]
    attr_accessor :object_keys

    # This attribute is required.
    # @return [OCI::DataIntegration::Models::CopyConflictResolution]
    attr_accessor :copy_conflict_resolution

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'source_workspace_id': :'sourceWorkspaceId',
        'object_keys': :'objectKeys',
        'copy_conflict_resolution': :'copyConflictResolution'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'source_workspace_id': :'String',
        'object_keys': :'Array<String>',
        'copy_conflict_resolution': :'OCI::DataIntegration::Models::CopyConflictResolution'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :source_workspace_id The value to assign to the {#source_workspace_id} property
    # @option attributes [Array<String>] :object_keys The value to assign to the {#object_keys} property
    # @option attributes [OCI::DataIntegration::Models::CopyConflictResolution] :copy_conflict_resolution The value to assign to the {#copy_conflict_resolution} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.source_workspace_id = attributes[:'sourceWorkspaceId'] if attributes[:'sourceWorkspaceId']

      raise 'You cannot provide both :sourceWorkspaceId and :source_workspace_id' if attributes.key?(:'sourceWorkspaceId') && attributes.key?(:'source_workspace_id')

      self.source_workspace_id = attributes[:'source_workspace_id'] if attributes[:'source_workspace_id']

      self.object_keys = attributes[:'objectKeys'] if attributes[:'objectKeys']

      raise 'You cannot provide both :objectKeys and :object_keys' if attributes.key?(:'objectKeys') && attributes.key?(:'object_keys')

      self.object_keys = attributes[:'object_keys'] if attributes[:'object_keys']

      self.copy_conflict_resolution = attributes[:'copyConflictResolution'] if attributes[:'copyConflictResolution']

      raise 'You cannot provide both :copyConflictResolution and :copy_conflict_resolution' if attributes.key?(:'copyConflictResolution') && attributes.key?(:'copy_conflict_resolution')

      self.copy_conflict_resolution = attributes[:'copy_conflict_resolution'] if attributes[:'copy_conflict_resolution']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        source_workspace_id == other.source_workspace_id &&
        object_keys == other.object_keys &&
        copy_conflict_resolution == other.copy_conflict_resolution
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [source_workspace_id, object_keys, copy_conflict_resolution].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
