# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181201
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The details of the aggregation dimensions used for summarizing SQL violations.
  class DataSafe::Models::SqlFirewallViolationAggregationDimensions
    SQL_LEVEL_ENUM = [
      SQL_LEVEL_USER_ISSUED_SQL = 'USER_ISSUED_SQL'.freeze,
      SQL_LEVEL_ALL_SQL = 'ALL_SQL'.freeze,
      SQL_LEVEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # The time of the SQL violation occurrence in the target database.
    # @return [Array<DateTime>]
    attr_accessor :operation_time

    # The name of the database user.
    # @return [Array<String>]
    attr_accessor :db_user_name

    # The OCID of the target database.
    # @return [Array<String>]
    attr_accessor :target_id

    # The name of the target database.
    # @return [Array<String>]
    attr_accessor :target_name

    # The application from which the SQL violation was generated. Examples SQL Plus or SQL Developer.
    # @return [Array<String>]
    attr_accessor :client_program

    # The name of the action executed by the user on the target database, for example, ALTER, CREATE, DROP.
    # @return [Array<String>]
    attr_accessor :operation

    # The name of the operating system user for the database session.
    # @return [Array<String>]
    attr_accessor :client_os_user_name

    # Indicates whether SQL or context violation.
    # @return [Array<String>]
    attr_accessor :violation_cause

    # The IP address of the host from which the session was spawned.
    # @return [Array<String>]
    attr_accessor :client_ip

    # The action taken for this SQL violation.
    # @return [Array<String>]
    attr_accessor :violation_action

    # Specifies the level of SQL included for this SQL Firewall policy.
    # USER_ISSUED_SQL - User issued SQL statements only.
    # ALL_SQL - Includes all SQL statements including SQL statement issued inside PL/SQL units.
    #
    # @return [Array<String>]
    attr_reader :sql_level

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'operation_time': :'operationTime',
        'db_user_name': :'dbUserName',
        'target_id': :'targetId',
        'target_name': :'targetName',
        'client_program': :'clientProgram',
        'operation': :'operation',
        'client_os_user_name': :'clientOsUserName',
        'violation_cause': :'violationCause',
        'client_ip': :'clientIp',
        'violation_action': :'violationAction',
        'sql_level': :'sqlLevel'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'operation_time': :'Array<DateTime>',
        'db_user_name': :'Array<String>',
        'target_id': :'Array<String>',
        'target_name': :'Array<String>',
        'client_program': :'Array<String>',
        'operation': :'Array<String>',
        'client_os_user_name': :'Array<String>',
        'violation_cause': :'Array<String>',
        'client_ip': :'Array<String>',
        'violation_action': :'Array<String>',
        'sql_level': :'Array<String>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [Array<DateTime>] :operation_time The value to assign to the {#operation_time} property
    # @option attributes [Array<String>] :db_user_name The value to assign to the {#db_user_name} property
    # @option attributes [Array<String>] :target_id The value to assign to the {#target_id} property
    # @option attributes [Array<String>] :target_name The value to assign to the {#target_name} property
    # @option attributes [Array<String>] :client_program The value to assign to the {#client_program} property
    # @option attributes [Array<String>] :operation The value to assign to the {#operation} property
    # @option attributes [Array<String>] :client_os_user_name The value to assign to the {#client_os_user_name} property
    # @option attributes [Array<String>] :violation_cause The value to assign to the {#violation_cause} property
    # @option attributes [Array<String>] :client_ip The value to assign to the {#client_ip} property
    # @option attributes [Array<String>] :violation_action The value to assign to the {#violation_action} property
    # @option attributes [Array<String>] :sql_level The value to assign to the {#sql_level} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.operation_time = attributes[:'operationTime'] if attributes[:'operationTime']

      raise 'You cannot provide both :operationTime and :operation_time' if attributes.key?(:'operationTime') && attributes.key?(:'operation_time')

      self.operation_time = attributes[:'operation_time'] if attributes[:'operation_time']

      self.db_user_name = attributes[:'dbUserName'] if attributes[:'dbUserName']

      raise 'You cannot provide both :dbUserName and :db_user_name' if attributes.key?(:'dbUserName') && attributes.key?(:'db_user_name')

      self.db_user_name = attributes[:'db_user_name'] if attributes[:'db_user_name']

      self.target_id = attributes[:'targetId'] if attributes[:'targetId']

      raise 'You cannot provide both :targetId and :target_id' if attributes.key?(:'targetId') && attributes.key?(:'target_id')

      self.target_id = attributes[:'target_id'] if attributes[:'target_id']

      self.target_name = attributes[:'targetName'] if attributes[:'targetName']

      raise 'You cannot provide both :targetName and :target_name' if attributes.key?(:'targetName') && attributes.key?(:'target_name')

      self.target_name = attributes[:'target_name'] if attributes[:'target_name']

      self.client_program = attributes[:'clientProgram'] if attributes[:'clientProgram']

      raise 'You cannot provide both :clientProgram and :client_program' if attributes.key?(:'clientProgram') && attributes.key?(:'client_program')

      self.client_program = attributes[:'client_program'] if attributes[:'client_program']

      self.operation = attributes[:'operation'] if attributes[:'operation']

      self.client_os_user_name = attributes[:'clientOsUserName'] if attributes[:'clientOsUserName']

      raise 'You cannot provide both :clientOsUserName and :client_os_user_name' if attributes.key?(:'clientOsUserName') && attributes.key?(:'client_os_user_name')

      self.client_os_user_name = attributes[:'client_os_user_name'] if attributes[:'client_os_user_name']

      self.violation_cause = attributes[:'violationCause'] if attributes[:'violationCause']

      raise 'You cannot provide both :violationCause and :violation_cause' if attributes.key?(:'violationCause') && attributes.key?(:'violation_cause')

      self.violation_cause = attributes[:'violation_cause'] if attributes[:'violation_cause']

      self.client_ip = attributes[:'clientIp'] if attributes[:'clientIp']

      raise 'You cannot provide both :clientIp and :client_ip' if attributes.key?(:'clientIp') && attributes.key?(:'client_ip')

      self.client_ip = attributes[:'client_ip'] if attributes[:'client_ip']

      self.violation_action = attributes[:'violationAction'] if attributes[:'violationAction']

      raise 'You cannot provide both :violationAction and :violation_action' if attributes.key?(:'violationAction') && attributes.key?(:'violation_action')

      self.violation_action = attributes[:'violation_action'] if attributes[:'violation_action']

      self.sql_level = attributes[:'sqlLevel'] if attributes[:'sqlLevel']

      raise 'You cannot provide both :sqlLevel and :sql_level' if attributes.key?(:'sqlLevel') && attributes.key?(:'sql_level')

      self.sql_level = attributes[:'sql_level'] if attributes[:'sql_level']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] sql_level Object to be assigned
    def sql_level=(sql_level)
      # rubocop:disable Style/ConditionalAssignment
      if sql_level.nil?
        @sql_level = nil
      else
        @sql_level =
          sql_level.collect do |item|
            if SQL_LEVEL_ENUM.include?(item)
              item
            else
              OCI.logger.debug("Unknown value for 'sql_level' [#{item}]. Mapping to 'SQL_LEVEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
              SQL_LEVEL_UNKNOWN_ENUM_VALUE
            end
          end
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        operation_time == other.operation_time &&
        db_user_name == other.db_user_name &&
        target_id == other.target_id &&
        target_name == other.target_name &&
        client_program == other.client_program &&
        operation == other.operation &&
        client_os_user_name == other.client_os_user_name &&
        violation_cause == other.violation_cause &&
        client_ip == other.client_ip &&
        violation_action == other.violation_action &&
        sql_level == other.sql_level
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [operation_time, db_user_name, target_id, target_name, client_program, operation, client_os_user_name, violation_cause, client_ip, violation_action, sql_level].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
