# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190101
require 'date'
require_relative 'artifact_export_details'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Model artifact source details for exporting artifact to service bucket
  class DataScience::Models::ArtifactExportDetailsObjectStorage < DataScience::Models::ArtifactExportDetails
    # The Object Storage namespace used for the request.
    # @return [String]
    attr_accessor :namespace

    # The name of the bucket. Avoid entering confidential information.
    # @return [String]
    attr_accessor :source_bucket

    # The name of the object resulting from the copy operation.
    # @return [String]
    attr_accessor :source_object_name

    # Region in which OSS bucket is present
    # @return [String]
    attr_accessor :source_region

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'artifact_source_type': :'artifactSourceType',
        'namespace': :'namespace',
        'source_bucket': :'sourceBucket',
        'source_object_name': :'sourceObjectName',
        'source_region': :'sourceRegion'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'artifact_source_type': :'String',
        'namespace': :'String',
        'source_bucket': :'String',
        'source_object_name': :'String',
        'source_region': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :namespace The value to assign to the {#namespace} property
    # @option attributes [String] :source_bucket The value to assign to the {#source_bucket} property
    # @option attributes [String] :source_object_name The value to assign to the {#source_object_name} property
    # @option attributes [String] :source_region The value to assign to the {#source_region} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['artifactSourceType'] = 'ORACLE_OBJECT_STORAGE'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.namespace = attributes[:'namespace'] if attributes[:'namespace']

      self.source_bucket = attributes[:'sourceBucket'] if attributes[:'sourceBucket']

      raise 'You cannot provide both :sourceBucket and :source_bucket' if attributes.key?(:'sourceBucket') && attributes.key?(:'source_bucket')

      self.source_bucket = attributes[:'source_bucket'] if attributes[:'source_bucket']

      self.source_object_name = attributes[:'sourceObjectName'] if attributes[:'sourceObjectName']

      raise 'You cannot provide both :sourceObjectName and :source_object_name' if attributes.key?(:'sourceObjectName') && attributes.key?(:'source_object_name')

      self.source_object_name = attributes[:'source_object_name'] if attributes[:'source_object_name']

      self.source_region = attributes[:'sourceRegion'] if attributes[:'sourceRegion']

      raise 'You cannot provide both :sourceRegion and :source_region' if attributes.key?(:'sourceRegion') && attributes.key?(:'source_region')

      self.source_region = attributes[:'source_region'] if attributes[:'source_region']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        artifact_source_type == other.artifact_source_type &&
        namespace == other.namespace &&
        source_bucket == other.source_bucket &&
        source_object_name == other.source_object_name &&
        source_region == other.source_region
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [artifact_source_type, namespace, source_bucket, source_object_name, source_region].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
