# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190101
require 'date'
require 'logger'
require_relative 'metric_expression_rule'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # An expression built using CPU or Memory metrics for triggering an autoscaling action on the model deployment.
  #
  class DataScience::Models::PredefinedMetricExpressionRule < DataScience::Models::MetricExpressionRule
    METRIC_TYPE_ENUM = [
      METRIC_TYPE_CPU_UTILIZATION = 'CPU_UTILIZATION'.freeze,
      METRIC_TYPE_MEMORY_UTILIZATION = 'MEMORY_UTILIZATION'.freeze,
      METRIC_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Metric type
    #
    # @return [String]
    attr_reader :metric_type

    # This attribute is required.
    # @return [OCI::DataScience::Models::PredefinedExpressionThresholdScalingConfiguration]
    attr_accessor :scale_in_configuration

    # This attribute is required.
    # @return [OCI::DataScience::Models::PredefinedExpressionThresholdScalingConfiguration]
    attr_accessor :scale_out_configuration

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'metric_expression_rule_type': :'metricExpressionRuleType',
        'metric_type': :'metricType',
        'scale_in_configuration': :'scaleInConfiguration',
        'scale_out_configuration': :'scaleOutConfiguration'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'metric_expression_rule_type': :'String',
        'metric_type': :'String',
        'scale_in_configuration': :'OCI::DataScience::Models::PredefinedExpressionThresholdScalingConfiguration',
        'scale_out_configuration': :'OCI::DataScience::Models::PredefinedExpressionThresholdScalingConfiguration'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :metric_type The value to assign to the {#metric_type} property
    # @option attributes [OCI::DataScience::Models::PredefinedExpressionThresholdScalingConfiguration] :scale_in_configuration The value to assign to the {#scale_in_configuration} property
    # @option attributes [OCI::DataScience::Models::PredefinedExpressionThresholdScalingConfiguration] :scale_out_configuration The value to assign to the {#scale_out_configuration} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['metricExpressionRuleType'] = 'PREDEFINED_EXPRESSION'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.metric_type = attributes[:'metricType'] if attributes[:'metricType']

      raise 'You cannot provide both :metricType and :metric_type' if attributes.key?(:'metricType') && attributes.key?(:'metric_type')

      self.metric_type = attributes[:'metric_type'] if attributes[:'metric_type']

      self.scale_in_configuration = attributes[:'scaleInConfiguration'] if attributes[:'scaleInConfiguration']

      raise 'You cannot provide both :scaleInConfiguration and :scale_in_configuration' if attributes.key?(:'scaleInConfiguration') && attributes.key?(:'scale_in_configuration')

      self.scale_in_configuration = attributes[:'scale_in_configuration'] if attributes[:'scale_in_configuration']

      self.scale_out_configuration = attributes[:'scaleOutConfiguration'] if attributes[:'scaleOutConfiguration']

      raise 'You cannot provide both :scaleOutConfiguration and :scale_out_configuration' if attributes.key?(:'scaleOutConfiguration') && attributes.key?(:'scale_out_configuration')

      self.scale_out_configuration = attributes[:'scale_out_configuration'] if attributes[:'scale_out_configuration']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] metric_type Object to be assigned
    def metric_type=(metric_type)
      # rubocop:disable Style/ConditionalAssignment
      if metric_type && !METRIC_TYPE_ENUM.include?(metric_type)
        OCI.logger.debug("Unknown value for 'metric_type' [" + metric_type + "]. Mapping to 'METRIC_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @metric_type = METRIC_TYPE_UNKNOWN_ENUM_VALUE
      else
        @metric_type = metric_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        metric_expression_rule_type == other.metric_expression_rule_type &&
        metric_type == other.metric_type &&
        scale_in_configuration == other.scale_in_configuration &&
        scale_out_configuration == other.scale_out_configuration
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [metric_expression_rule_type, metric_type, scale_in_configuration, scale_out_configuration].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
