# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190101
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Scaling configuration for the metric expression rule.
  #
  # This class has direct subclasses. If you are using this class as input to a service operations then you should favor using a subclass over the base class
  class DataScience::Models::ScalingConfiguration
    SCALING_CONFIGURATION_TYPE_ENUM = [
      SCALING_CONFIGURATION_TYPE_THRESHOLD = 'THRESHOLD'.freeze,
      SCALING_CONFIGURATION_TYPE_QUERY = 'QUERY'.freeze
    ].freeze

    # **[Required]** The type of scaling configuration.
    #
    # @return [String]
    attr_reader :scaling_configuration_type

    # The period of time that the condition defined in the alarm must persist before the alarm state
    # changes from \"OK\" to \"FIRING\" or vice versa. For example, a value of 5 minutes means that the
    # alarm must persist in breaching the condition for five minutes before the alarm updates its
    # state to \"FIRING\"; likewise, the alarm must persist in not breaching the condition for five
    # minutes before the alarm updates its state to \"OK.\"
    #
    # The duration is specified as a string in ISO 8601 format (`PT10M` for ten minutes or `PT1H`
    # for one hour). Minimum: PT3M. Maximum: PT1H. Default: PT3M.
    #
    # @return [String]
    attr_accessor :pending_duration

    # The value is used for adjusting the count of instances by.
    #
    # @return [Integer]
    attr_accessor :instance_count_adjustment

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'scaling_configuration_type': :'scalingConfigurationType',
        'pending_duration': :'pendingDuration',
        'instance_count_adjustment': :'instanceCountAdjustment'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'scaling_configuration_type': :'String',
        'pending_duration': :'String',
        'instance_count_adjustment': :'Integer'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize


    # Given the hash representation of a subtype of this class,
    # use the info in the hash to return the class of the subtype.
    def self.get_subtype(object_hash)
      type = object_hash[:'scalingConfigurationType'] # rubocop:disable Style/SymbolLiteral

      return 'OCI::DataScience::Models::CustomExpressionQueryScalingConfiguration' if type == 'QUERY'
      return 'OCI::DataScience::Models::PredefinedExpressionThresholdScalingConfiguration' if type == 'THRESHOLD'

      # TODO: Log a warning when the subtype is not found.
      'OCI::DataScience::Models::ScalingConfiguration'
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :scaling_configuration_type The value to assign to the {#scaling_configuration_type} property
    # @option attributes [String] :pending_duration The value to assign to the {#pending_duration} property
    # @option attributes [Integer] :instance_count_adjustment The value to assign to the {#instance_count_adjustment} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.scaling_configuration_type = attributes[:'scalingConfigurationType'] if attributes[:'scalingConfigurationType']

      raise 'You cannot provide both :scalingConfigurationType and :scaling_configuration_type' if attributes.key?(:'scalingConfigurationType') && attributes.key?(:'scaling_configuration_type')

      self.scaling_configuration_type = attributes[:'scaling_configuration_type'] if attributes[:'scaling_configuration_type']

      self.pending_duration = attributes[:'pendingDuration'] if attributes[:'pendingDuration']
      self.pending_duration = "PT3M" if pending_duration.nil? && !attributes.key?(:'pendingDuration') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :pendingDuration and :pending_duration' if attributes.key?(:'pendingDuration') && attributes.key?(:'pending_duration')

      self.pending_duration = attributes[:'pending_duration'] if attributes[:'pending_duration']
      self.pending_duration = "PT3M" if pending_duration.nil? && !attributes.key?(:'pendingDuration') && !attributes.key?(:'pending_duration') # rubocop:disable Style/StringLiterals

      self.instance_count_adjustment = attributes[:'instanceCountAdjustment'] if attributes[:'instanceCountAdjustment']

      raise 'You cannot provide both :instanceCountAdjustment and :instance_count_adjustment' if attributes.key?(:'instanceCountAdjustment') && attributes.key?(:'instance_count_adjustment')

      self.instance_count_adjustment = attributes[:'instance_count_adjustment'] if attributes[:'instance_count_adjustment']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] scaling_configuration_type Object to be assigned
    def scaling_configuration_type=(scaling_configuration_type)
      raise "Invalid value for 'scaling_configuration_type': this must be one of the values in SCALING_CONFIGURATION_TYPE_ENUM." if scaling_configuration_type && !SCALING_CONFIGURATION_TYPE_ENUM.include?(scaling_configuration_type)

      @scaling_configuration_type = scaling_configuration_type
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        scaling_configuration_type == other.scaling_configuration_type &&
        pending_duration == other.pending_duration &&
        instance_count_adjustment == other.instance_count_adjustment
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [scaling_configuration_type, pending_duration, instance_count_adjustment].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
