# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20160918
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # An Autonomous Container Database is a container database service that enables the customer to host one or more databases within the container database. A basic container database runs on a single Autonomous Exadata Infrastructure from an availability domain without the Extreme Availability features enabled.
  #
  class Database::Models::AutonomousContainerDatabaseSummary
    SERVICE_LEVEL_AGREEMENT_TYPE_ENUM = [
      SERVICE_LEVEL_AGREEMENT_TYPE_STANDARD = 'STANDARD'.freeze,
      SERVICE_LEVEL_AGREEMENT_TYPE_MISSION_CRITICAL = 'MISSION_CRITICAL'.freeze,
      SERVICE_LEVEL_AGREEMENT_TYPE_AUTONOMOUS_DATAGUARD = 'AUTONOMOUS_DATAGUARD'.freeze,
      SERVICE_LEVEL_AGREEMENT_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    INFRASTRUCTURE_TYPE_ENUM = [
      INFRASTRUCTURE_TYPE_CLOUD = 'CLOUD'.freeze,
      INFRASTRUCTURE_TYPE_CLOUD_AT_CUSTOMER = 'CLOUD_AT_CUSTOMER'.freeze,
      INFRASTRUCTURE_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_PROVISIONING = 'PROVISIONING'.freeze,
      LIFECYCLE_STATE_AVAILABLE = 'AVAILABLE'.freeze,
      LIFECYCLE_STATE_UPDATING = 'UPDATING'.freeze,
      LIFECYCLE_STATE_TERMINATING = 'TERMINATING'.freeze,
      LIFECYCLE_STATE_TERMINATED = 'TERMINATED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_BACKUP_IN_PROGRESS = 'BACKUP_IN_PROGRESS'.freeze,
      LIFECYCLE_STATE_RESTORING = 'RESTORING'.freeze,
      LIFECYCLE_STATE_RESTORE_FAILED = 'RESTORE_FAILED'.freeze,
      LIFECYCLE_STATE_RESTARTING = 'RESTARTING'.freeze,
      LIFECYCLE_STATE_MAINTENANCE_IN_PROGRESS = 'MAINTENANCE_IN_PROGRESS'.freeze,
      LIFECYCLE_STATE_ROLE_CHANGE_IN_PROGRESS = 'ROLE_CHANGE_IN_PROGRESS'.freeze,
      LIFECYCLE_STATE_ENABLING_AUTONOMOUS_DATA_GUARD = 'ENABLING_AUTONOMOUS_DATA_GUARD'.freeze,
      LIFECYCLE_STATE_UNAVAILABLE = 'UNAVAILABLE'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    PATCH_MODEL_ENUM = [
      PATCH_MODEL_RELEASE_UPDATES = 'RELEASE_UPDATES'.freeze,
      PATCH_MODEL_RELEASE_UPDATE_REVISIONS = 'RELEASE_UPDATE_REVISIONS'.freeze,
      PATCH_MODEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    VERSION_PREFERENCE_ENUM = [
      VERSION_PREFERENCE_NEXT_RELEASE_UPDATE = 'NEXT_RELEASE_UPDATE'.freeze,
      VERSION_PREFERENCE_LATEST_RELEASE_UPDATE = 'LATEST_RELEASE_UPDATE'.freeze,
      VERSION_PREFERENCE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    ROLE_ENUM = [
      ROLE_PRIMARY = 'PRIMARY'.freeze,
      ROLE_STANDBY = 'STANDBY'.freeze,
      ROLE_DISABLED_STANDBY = 'DISABLED_STANDBY'.freeze,
      ROLE_BACKUP_COPY = 'BACKUP_COPY'.freeze,
      ROLE_SNAPSHOT_STANDBY = 'SNAPSHOT_STANDBY'.freeze,
      ROLE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    COMPUTE_MODEL_ENUM = [
      COMPUTE_MODEL_ECPU = 'ECPU'.freeze,
      COMPUTE_MODEL_OCPU = 'OCPU'.freeze,
      COMPUTE_MODEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    DISTRIBUTION_AFFINITY_ENUM = [
      DISTRIBUTION_AFFINITY_MINIMUM_DISTRIBUTION = 'MINIMUM_DISTRIBUTION'.freeze,
      DISTRIBUTION_AFFINITY_MAXIMUM_DISTRIBUTION = 'MAXIMUM_DISTRIBUTION'.freeze,
      DISTRIBUTION_AFFINITY_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    NET_SERVICES_ARCHITECTURE_ENUM = [
      NET_SERVICES_ARCHITECTURE_DEDICATED = 'DEDICATED'.freeze,
      NET_SERVICES_ARCHITECTURE_SHARED = 'SHARED'.freeze,
      NET_SERVICES_ARCHITECTURE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The OCID of the Autonomous Container Database.
    # @return [String]
    attr_accessor :id

    # **[Required]** The OCID of the compartment.
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** The user-provided name for the Autonomous Container Database.
    # @return [String]
    attr_accessor :display_name

    # **Deprecated.** The `DB_UNIQUE_NAME` value is set by Oracle Cloud Infrastructure.  Do not specify a value for this parameter. Specifying a value for this field will cause Terraform operations to fail.
    #
    # @return [String]
    attr_accessor :db_unique_name

    # The Database name for the Autonomous Container Database. The name must be unique within the Cloud Autonomous VM Cluster, starting with an alphabetic character, followed by 1 to 7 alphanumeric characters.
    # @return [String]
    attr_accessor :db_name

    # **[Required]** The service level agreement type of the container database. The default is STANDARD.
    # @return [String]
    attr_reader :service_level_agreement_type

    # **No longer used.** For Autonomous Database on dedicated Exadata infrastructure, the container database is created within a specified `cloudAutonomousVmCluster`.
    #
    # @return [String]
    attr_accessor :autonomous_exadata_infrastructure_id

    # The OCID of the Autonomous VM Cluster.
    # @return [String]
    attr_accessor :autonomous_vm_cluster_id

    # The infrastructure type this resource belongs to.
    # @return [String]
    attr_reader :infrastructure_type

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the cloud Autonomous Exadata VM Cluster.
    # @return [String]
    attr_accessor :cloud_autonomous_vm_cluster_id

    # The OCID of the key container that is used as the master encryption key in database transparent data encryption (TDE) operations.
    # @return [String]
    attr_accessor :kms_key_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Oracle Cloud Infrastructure [vault](https://docs.cloud.oracle.com/Content/KeyManagement/Concepts/keyoverview.htm#concepts). This parameter and `secretId` are required for Customer Managed Keys.
    # @return [String]
    attr_accessor :vault_id

    # The OCID of the key container version that is used in database transparent data encryption (TDE) operations KMS Key can have multiple key versions. If none is specified, the current key version (latest) of the Key Id is used for the operation. Autonomous Database Serverless does not use key versions, hence is not applicable for Autonomous Database Serverless instances.
    #
    # @return [String]
    attr_accessor :kms_key_version_id

    # Key History Entry.
    # @return [Array<OCI::Database::Models::AutonomousDatabaseKeyHistoryEntry>]
    attr_accessor :key_history_entry

    # **[Required]** The current state of the Autonomous Container Database.
    # @return [String]
    attr_reader :lifecycle_state

    # Additional information about the current lifecycle state.
    # @return [String]
    attr_accessor :lifecycle_details

    # The date and time the Autonomous Container Database was created.
    # @return [DateTime]
    attr_accessor :time_created

    # The date and time the Autonomous Container Database will be reverted to Standby from Snapshot Standby.
    # @return [DateTime]
    attr_accessor :time_snapshot_standby_revert

    # **[Required]** Database patch model preference.
    # @return [String]
    attr_reader :patch_model

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the last patch applied on the system.
    # @return [String]
    attr_accessor :patch_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the last maintenance run.
    # @return [String]
    attr_accessor :last_maintenance_run_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the next maintenance run.
    # @return [String]
    attr_accessor :next_maintenance_run_id

    # @return [OCI::Database::Models::MaintenanceWindow]
    attr_accessor :maintenance_window

    # The scheduling detail for the quarterly maintenance window of the standby Autonomous Container Database.
    # This value represents the number of days before scheduled maintenance of the primary database.
    #
    # @return [Integer]
    attr_accessor :standby_maintenance_buffer_in_days

    # The next maintenance version preference.
    #
    # @return [String]
    attr_reader :version_preference

    # Indicates if an automatic DST Time Zone file update is enabled for the Autonomous Container Database. If enabled along with Release Update, patching will be done in a Non-Rolling manner.
    # @return [BOOLEAN]
    attr_accessor :is_dst_file_update_enabled

    # DST Time-zone File version of the Autonomous Container Database.
    # @return [String]
    attr_accessor :dst_file_version

    # Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # The Data Guard role of the Autonomous Container Database or Autonomous Database, if Autonomous Data Guard is enabled.
    #
    # @return [String]
    attr_reader :role

    # The availability domain of the Autonomous Container Database.
    # @return [String]
    attr_accessor :availability_domain

    # Oracle Database version of the Autonomous Container Database.
    # @return [String]
    attr_accessor :db_version

    # @return [OCI::Database::Models::AutonomousContainerDatabaseBackupConfig]
    attr_accessor :backup_config

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the key store of Oracle Vault.
    # @return [String]
    attr_accessor :key_store_id

    # The wallet name for Oracle Key Vault.
    # @return [String]
    attr_accessor :key_store_wallet_name

    # The amount of memory (in GBs) enabled per ECPU or OCPU in the Autonomous VM Cluster.
    # @return [Integer]
    attr_accessor :memory_per_oracle_compute_unit_in_gbs

    # Sum of CPUs available on the Autonomous VM Cluster + Sum of reclaimable CPUs available in the Autonomous Container Database.
    #
    # @return [Float]
    attr_accessor :available_cpus

    # The number of CPUs allocated to the Autonomous VM cluster.
    #
    # @return [Integer]
    attr_accessor :total_cpus

    # CPUs that continue to be included in the count of CPUs available to the Autonomous Container Database even after one of its Autonomous Database is terminated or scaled down. You can release them to the available CPUs at its parent Autonomous VM Cluster level by restarting the Autonomous Container Database.
    #
    # @return [Float]
    attr_accessor :reclaimable_cpus

    # An array of CPU values that can be used to successfully provision a single Autonomous Database.
    #
    # @return [Array<Float>]
    attr_accessor :provisionable_cpus

    # List of One-Off patches that has been successfully applied to Autonomous Container Database
    # @return [Array<String>]
    attr_accessor :list_one_off_patches

    # The compute model of the Autonomous Container Database. For Autonomous Database on Dedicated Exadata Infrastructure, the CPU type (ECPUs or OCPUs) is determined by the parent Autonomous Exadata VM Cluster's compute model. ECPU compute model is the recommended model and OCPU compute model is legacy. See [Compute Models in Autonomous Database on Dedicated Exadata Infrastructure](https://docs.oracle.com/en/cloud/paas/autonomous-database/dedicated/adbak) for more details.
    #
    # @return [String]
    attr_reader :compute_model

    # The number of CPUs provisioned in an Autonomous Container Database.
    # @return [Float]
    attr_accessor :provisioned_cpus

    # The number of CPUs reserved in an Autonomous Container Database.
    # @return [Float]
    attr_accessor :reserved_cpus

    # The largest Autonomous Database (CPU) that can be created in a new Autonomous Container Database.
    # @return [Float]
    attr_accessor :largest_provisionable_autonomous_database_in_cpus

    # The timestamp of last successful backup. Here NULL value represents either there are no successful backups or backups are not configured for this Autonomous Container Database.
    # @return [DateTime]
    attr_accessor :time_of_last_backup

    # The CPU value beyond which an Autonomous Database will be opened across multiple nodes. The default value of this attribute is 16 for OCPUs and 64 for ECPUs.
    # @return [Integer]
    attr_accessor :db_split_threshold

    # The percentage of CPUs reserved across nodes to support node failover. Allowed values are 0%, 25%, and 50%, with 50% being the default option.
    # @return [Integer]
    attr_accessor :vm_failover_reservation

    # Determines whether an Autonomous Database must be opened across the maximum number of nodes or the least number of nodes. By default, Minimum nodes is selected.
    # @return [String]
    attr_reader :distribution_affinity

    # Enabling SHARED server architecture enables a database server to allow many client processes to share very few server processes, thereby increasing the number of supported users.
    # @return [String]
    attr_reader :net_services_architecture

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'compartment_id': :'compartmentId',
        'display_name': :'displayName',
        'db_unique_name': :'dbUniqueName',
        'db_name': :'dbName',
        'service_level_agreement_type': :'serviceLevelAgreementType',
        'autonomous_exadata_infrastructure_id': :'autonomousExadataInfrastructureId',
        'autonomous_vm_cluster_id': :'autonomousVmClusterId',
        'infrastructure_type': :'infrastructureType',
        'cloud_autonomous_vm_cluster_id': :'cloudAutonomousVmClusterId',
        'kms_key_id': :'kmsKeyId',
        'vault_id': :'vaultId',
        'kms_key_version_id': :'kmsKeyVersionId',
        'key_history_entry': :'keyHistoryEntry',
        'lifecycle_state': :'lifecycleState',
        'lifecycle_details': :'lifecycleDetails',
        'time_created': :'timeCreated',
        'time_snapshot_standby_revert': :'timeSnapshotStandbyRevert',
        'patch_model': :'patchModel',
        'patch_id': :'patchId',
        'last_maintenance_run_id': :'lastMaintenanceRunId',
        'next_maintenance_run_id': :'nextMaintenanceRunId',
        'maintenance_window': :'maintenanceWindow',
        'standby_maintenance_buffer_in_days': :'standbyMaintenanceBufferInDays',
        'version_preference': :'versionPreference',
        'is_dst_file_update_enabled': :'isDstFileUpdateEnabled',
        'dst_file_version': :'dstFileVersion',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'role': :'role',
        'availability_domain': :'availabilityDomain',
        'db_version': :'dbVersion',
        'backup_config': :'backupConfig',
        'key_store_id': :'keyStoreId',
        'key_store_wallet_name': :'keyStoreWalletName',
        'memory_per_oracle_compute_unit_in_gbs': :'memoryPerOracleComputeUnitInGBs',
        'available_cpus': :'availableCpus',
        'total_cpus': :'totalCpus',
        'reclaimable_cpus': :'reclaimableCpus',
        'provisionable_cpus': :'provisionableCpus',
        'list_one_off_patches': :'listOneOffPatches',
        'compute_model': :'computeModel',
        'provisioned_cpus': :'provisionedCpus',
        'reserved_cpus': :'reservedCpus',
        'largest_provisionable_autonomous_database_in_cpus': :'largestProvisionableAutonomousDatabaseInCpus',
        'time_of_last_backup': :'timeOfLastBackup',
        'db_split_threshold': :'dbSplitThreshold',
        'vm_failover_reservation': :'vmFailoverReservation',
        'distribution_affinity': :'distributionAffinity',
        'net_services_architecture': :'netServicesArchitecture'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'compartment_id': :'String',
        'display_name': :'String',
        'db_unique_name': :'String',
        'db_name': :'String',
        'service_level_agreement_type': :'String',
        'autonomous_exadata_infrastructure_id': :'String',
        'autonomous_vm_cluster_id': :'String',
        'infrastructure_type': :'String',
        'cloud_autonomous_vm_cluster_id': :'String',
        'kms_key_id': :'String',
        'vault_id': :'String',
        'kms_key_version_id': :'String',
        'key_history_entry': :'Array<OCI::Database::Models::AutonomousDatabaseKeyHistoryEntry>',
        'lifecycle_state': :'String',
        'lifecycle_details': :'String',
        'time_created': :'DateTime',
        'time_snapshot_standby_revert': :'DateTime',
        'patch_model': :'String',
        'patch_id': :'String',
        'last_maintenance_run_id': :'String',
        'next_maintenance_run_id': :'String',
        'maintenance_window': :'OCI::Database::Models::MaintenanceWindow',
        'standby_maintenance_buffer_in_days': :'Integer',
        'version_preference': :'String',
        'is_dst_file_update_enabled': :'BOOLEAN',
        'dst_file_version': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'role': :'String',
        'availability_domain': :'String',
        'db_version': :'String',
        'backup_config': :'OCI::Database::Models::AutonomousContainerDatabaseBackupConfig',
        'key_store_id': :'String',
        'key_store_wallet_name': :'String',
        'memory_per_oracle_compute_unit_in_gbs': :'Integer',
        'available_cpus': :'Float',
        'total_cpus': :'Integer',
        'reclaimable_cpus': :'Float',
        'provisionable_cpus': :'Array<Float>',
        'list_one_off_patches': :'Array<String>',
        'compute_model': :'String',
        'provisioned_cpus': :'Float',
        'reserved_cpus': :'Float',
        'largest_provisionable_autonomous_database_in_cpus': :'Float',
        'time_of_last_backup': :'DateTime',
        'db_split_threshold': :'Integer',
        'vm_failover_reservation': :'Integer',
        'distribution_affinity': :'String',
        'net_services_architecture': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :db_unique_name The value to assign to the {#db_unique_name} property
    # @option attributes [String] :db_name The value to assign to the {#db_name} property
    # @option attributes [String] :service_level_agreement_type The value to assign to the {#service_level_agreement_type} property
    # @option attributes [String] :autonomous_exadata_infrastructure_id The value to assign to the {#autonomous_exadata_infrastructure_id} property
    # @option attributes [String] :autonomous_vm_cluster_id The value to assign to the {#autonomous_vm_cluster_id} property
    # @option attributes [String] :infrastructure_type The value to assign to the {#infrastructure_type} property
    # @option attributes [String] :cloud_autonomous_vm_cluster_id The value to assign to the {#cloud_autonomous_vm_cluster_id} property
    # @option attributes [String] :kms_key_id The value to assign to the {#kms_key_id} property
    # @option attributes [String] :vault_id The value to assign to the {#vault_id} property
    # @option attributes [String] :kms_key_version_id The value to assign to the {#kms_key_version_id} property
    # @option attributes [Array<OCI::Database::Models::AutonomousDatabaseKeyHistoryEntry>] :key_history_entry The value to assign to the {#key_history_entry} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :lifecycle_details The value to assign to the {#lifecycle_details} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_snapshot_standby_revert The value to assign to the {#time_snapshot_standby_revert} property
    # @option attributes [String] :patch_model The value to assign to the {#patch_model} property
    # @option attributes [String] :patch_id The value to assign to the {#patch_id} property
    # @option attributes [String] :last_maintenance_run_id The value to assign to the {#last_maintenance_run_id} property
    # @option attributes [String] :next_maintenance_run_id The value to assign to the {#next_maintenance_run_id} property
    # @option attributes [OCI::Database::Models::MaintenanceWindow] :maintenance_window The value to assign to the {#maintenance_window} property
    # @option attributes [Integer] :standby_maintenance_buffer_in_days The value to assign to the {#standby_maintenance_buffer_in_days} property
    # @option attributes [String] :version_preference The value to assign to the {#version_preference} property
    # @option attributes [BOOLEAN] :is_dst_file_update_enabled The value to assign to the {#is_dst_file_update_enabled} property
    # @option attributes [String] :dst_file_version The value to assign to the {#dst_file_version} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [String] :role The value to assign to the {#role} property
    # @option attributes [String] :availability_domain The value to assign to the {#availability_domain} property
    # @option attributes [String] :db_version The value to assign to the {#db_version} property
    # @option attributes [OCI::Database::Models::AutonomousContainerDatabaseBackupConfig] :backup_config The value to assign to the {#backup_config} property
    # @option attributes [String] :key_store_id The value to assign to the {#key_store_id} property
    # @option attributes [String] :key_store_wallet_name The value to assign to the {#key_store_wallet_name} property
    # @option attributes [Integer] :memory_per_oracle_compute_unit_in_gbs The value to assign to the {#memory_per_oracle_compute_unit_in_gbs} property
    # @option attributes [Float] :available_cpus The value to assign to the {#available_cpus} property
    # @option attributes [Integer] :total_cpus The value to assign to the {#total_cpus} property
    # @option attributes [Float] :reclaimable_cpus The value to assign to the {#reclaimable_cpus} property
    # @option attributes [Array<Float>] :provisionable_cpus The value to assign to the {#provisionable_cpus} property
    # @option attributes [Array<String>] :list_one_off_patches The value to assign to the {#list_one_off_patches} property
    # @option attributes [String] :compute_model The value to assign to the {#compute_model} property
    # @option attributes [Float] :provisioned_cpus The value to assign to the {#provisioned_cpus} property
    # @option attributes [Float] :reserved_cpus The value to assign to the {#reserved_cpus} property
    # @option attributes [Float] :largest_provisionable_autonomous_database_in_cpus The value to assign to the {#largest_provisionable_autonomous_database_in_cpus} property
    # @option attributes [DateTime] :time_of_last_backup The value to assign to the {#time_of_last_backup} property
    # @option attributes [Integer] :db_split_threshold The value to assign to the {#db_split_threshold} property
    # @option attributes [Integer] :vm_failover_reservation The value to assign to the {#vm_failover_reservation} property
    # @option attributes [String] :distribution_affinity The value to assign to the {#distribution_affinity} property
    # @option attributes [String] :net_services_architecture The value to assign to the {#net_services_architecture} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.db_unique_name = attributes[:'dbUniqueName'] if attributes[:'dbUniqueName']

      raise 'You cannot provide both :dbUniqueName and :db_unique_name' if attributes.key?(:'dbUniqueName') && attributes.key?(:'db_unique_name')

      self.db_unique_name = attributes[:'db_unique_name'] if attributes[:'db_unique_name']

      self.db_name = attributes[:'dbName'] if attributes[:'dbName']

      raise 'You cannot provide both :dbName and :db_name' if attributes.key?(:'dbName') && attributes.key?(:'db_name')

      self.db_name = attributes[:'db_name'] if attributes[:'db_name']

      self.service_level_agreement_type = attributes[:'serviceLevelAgreementType'] if attributes[:'serviceLevelAgreementType']

      raise 'You cannot provide both :serviceLevelAgreementType and :service_level_agreement_type' if attributes.key?(:'serviceLevelAgreementType') && attributes.key?(:'service_level_agreement_type')

      self.service_level_agreement_type = attributes[:'service_level_agreement_type'] if attributes[:'service_level_agreement_type']

      self.autonomous_exadata_infrastructure_id = attributes[:'autonomousExadataInfrastructureId'] if attributes[:'autonomousExadataInfrastructureId']

      raise 'You cannot provide both :autonomousExadataInfrastructureId and :autonomous_exadata_infrastructure_id' if attributes.key?(:'autonomousExadataInfrastructureId') && attributes.key?(:'autonomous_exadata_infrastructure_id')

      self.autonomous_exadata_infrastructure_id = attributes[:'autonomous_exadata_infrastructure_id'] if attributes[:'autonomous_exadata_infrastructure_id']

      self.autonomous_vm_cluster_id = attributes[:'autonomousVmClusterId'] if attributes[:'autonomousVmClusterId']

      raise 'You cannot provide both :autonomousVmClusterId and :autonomous_vm_cluster_id' if attributes.key?(:'autonomousVmClusterId') && attributes.key?(:'autonomous_vm_cluster_id')

      self.autonomous_vm_cluster_id = attributes[:'autonomous_vm_cluster_id'] if attributes[:'autonomous_vm_cluster_id']

      self.infrastructure_type = attributes[:'infrastructureType'] if attributes[:'infrastructureType']

      raise 'You cannot provide both :infrastructureType and :infrastructure_type' if attributes.key?(:'infrastructureType') && attributes.key?(:'infrastructure_type')

      self.infrastructure_type = attributes[:'infrastructure_type'] if attributes[:'infrastructure_type']

      self.cloud_autonomous_vm_cluster_id = attributes[:'cloudAutonomousVmClusterId'] if attributes[:'cloudAutonomousVmClusterId']

      raise 'You cannot provide both :cloudAutonomousVmClusterId and :cloud_autonomous_vm_cluster_id' if attributes.key?(:'cloudAutonomousVmClusterId') && attributes.key?(:'cloud_autonomous_vm_cluster_id')

      self.cloud_autonomous_vm_cluster_id = attributes[:'cloud_autonomous_vm_cluster_id'] if attributes[:'cloud_autonomous_vm_cluster_id']

      self.kms_key_id = attributes[:'kmsKeyId'] if attributes[:'kmsKeyId']

      raise 'You cannot provide both :kmsKeyId and :kms_key_id' if attributes.key?(:'kmsKeyId') && attributes.key?(:'kms_key_id')

      self.kms_key_id = attributes[:'kms_key_id'] if attributes[:'kms_key_id']

      self.vault_id = attributes[:'vaultId'] if attributes[:'vaultId']

      raise 'You cannot provide both :vaultId and :vault_id' if attributes.key?(:'vaultId') && attributes.key?(:'vault_id')

      self.vault_id = attributes[:'vault_id'] if attributes[:'vault_id']

      self.kms_key_version_id = attributes[:'kmsKeyVersionId'] if attributes[:'kmsKeyVersionId']

      raise 'You cannot provide both :kmsKeyVersionId and :kms_key_version_id' if attributes.key?(:'kmsKeyVersionId') && attributes.key?(:'kms_key_version_id')

      self.kms_key_version_id = attributes[:'kms_key_version_id'] if attributes[:'kms_key_version_id']

      self.key_history_entry = attributes[:'keyHistoryEntry'] if attributes[:'keyHistoryEntry']

      raise 'You cannot provide both :keyHistoryEntry and :key_history_entry' if attributes.key?(:'keyHistoryEntry') && attributes.key?(:'key_history_entry')

      self.key_history_entry = attributes[:'key_history_entry'] if attributes[:'key_history_entry']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.lifecycle_details = attributes[:'lifecycleDetails'] if attributes[:'lifecycleDetails']

      raise 'You cannot provide both :lifecycleDetails and :lifecycle_details' if attributes.key?(:'lifecycleDetails') && attributes.key?(:'lifecycle_details')

      self.lifecycle_details = attributes[:'lifecycle_details'] if attributes[:'lifecycle_details']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_snapshot_standby_revert = attributes[:'timeSnapshotStandbyRevert'] if attributes[:'timeSnapshotStandbyRevert']

      raise 'You cannot provide both :timeSnapshotStandbyRevert and :time_snapshot_standby_revert' if attributes.key?(:'timeSnapshotStandbyRevert') && attributes.key?(:'time_snapshot_standby_revert')

      self.time_snapshot_standby_revert = attributes[:'time_snapshot_standby_revert'] if attributes[:'time_snapshot_standby_revert']

      self.patch_model = attributes[:'patchModel'] if attributes[:'patchModel']

      raise 'You cannot provide both :patchModel and :patch_model' if attributes.key?(:'patchModel') && attributes.key?(:'patch_model')

      self.patch_model = attributes[:'patch_model'] if attributes[:'patch_model']

      self.patch_id = attributes[:'patchId'] if attributes[:'patchId']

      raise 'You cannot provide both :patchId and :patch_id' if attributes.key?(:'patchId') && attributes.key?(:'patch_id')

      self.patch_id = attributes[:'patch_id'] if attributes[:'patch_id']

      self.last_maintenance_run_id = attributes[:'lastMaintenanceRunId'] if attributes[:'lastMaintenanceRunId']

      raise 'You cannot provide both :lastMaintenanceRunId and :last_maintenance_run_id' if attributes.key?(:'lastMaintenanceRunId') && attributes.key?(:'last_maintenance_run_id')

      self.last_maintenance_run_id = attributes[:'last_maintenance_run_id'] if attributes[:'last_maintenance_run_id']

      self.next_maintenance_run_id = attributes[:'nextMaintenanceRunId'] if attributes[:'nextMaintenanceRunId']

      raise 'You cannot provide both :nextMaintenanceRunId and :next_maintenance_run_id' if attributes.key?(:'nextMaintenanceRunId') && attributes.key?(:'next_maintenance_run_id')

      self.next_maintenance_run_id = attributes[:'next_maintenance_run_id'] if attributes[:'next_maintenance_run_id']

      self.maintenance_window = attributes[:'maintenanceWindow'] if attributes[:'maintenanceWindow']

      raise 'You cannot provide both :maintenanceWindow and :maintenance_window' if attributes.key?(:'maintenanceWindow') && attributes.key?(:'maintenance_window')

      self.maintenance_window = attributes[:'maintenance_window'] if attributes[:'maintenance_window']

      self.standby_maintenance_buffer_in_days = attributes[:'standbyMaintenanceBufferInDays'] if attributes[:'standbyMaintenanceBufferInDays']

      raise 'You cannot provide both :standbyMaintenanceBufferInDays and :standby_maintenance_buffer_in_days' if attributes.key?(:'standbyMaintenanceBufferInDays') && attributes.key?(:'standby_maintenance_buffer_in_days')

      self.standby_maintenance_buffer_in_days = attributes[:'standby_maintenance_buffer_in_days'] if attributes[:'standby_maintenance_buffer_in_days']

      self.version_preference = attributes[:'versionPreference'] if attributes[:'versionPreference']
      self.version_preference = "NEXT_RELEASE_UPDATE" if version_preference.nil? && !attributes.key?(:'versionPreference') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :versionPreference and :version_preference' if attributes.key?(:'versionPreference') && attributes.key?(:'version_preference')

      self.version_preference = attributes[:'version_preference'] if attributes[:'version_preference']
      self.version_preference = "NEXT_RELEASE_UPDATE" if version_preference.nil? && !attributes.key?(:'versionPreference') && !attributes.key?(:'version_preference') # rubocop:disable Style/StringLiterals

      self.is_dst_file_update_enabled = attributes[:'isDstFileUpdateEnabled'] unless attributes[:'isDstFileUpdateEnabled'].nil?

      raise 'You cannot provide both :isDstFileUpdateEnabled and :is_dst_file_update_enabled' if attributes.key?(:'isDstFileUpdateEnabled') && attributes.key?(:'is_dst_file_update_enabled')

      self.is_dst_file_update_enabled = attributes[:'is_dst_file_update_enabled'] unless attributes[:'is_dst_file_update_enabled'].nil?

      self.dst_file_version = attributes[:'dstFileVersion'] if attributes[:'dstFileVersion']

      raise 'You cannot provide both :dstFileVersion and :dst_file_version' if attributes.key?(:'dstFileVersion') && attributes.key?(:'dst_file_version')

      self.dst_file_version = attributes[:'dst_file_version'] if attributes[:'dst_file_version']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.role = attributes[:'role'] if attributes[:'role']

      self.availability_domain = attributes[:'availabilityDomain'] if attributes[:'availabilityDomain']

      raise 'You cannot provide both :availabilityDomain and :availability_domain' if attributes.key?(:'availabilityDomain') && attributes.key?(:'availability_domain')

      self.availability_domain = attributes[:'availability_domain'] if attributes[:'availability_domain']

      self.db_version = attributes[:'dbVersion'] if attributes[:'dbVersion']

      raise 'You cannot provide both :dbVersion and :db_version' if attributes.key?(:'dbVersion') && attributes.key?(:'db_version')

      self.db_version = attributes[:'db_version'] if attributes[:'db_version']

      self.backup_config = attributes[:'backupConfig'] if attributes[:'backupConfig']

      raise 'You cannot provide both :backupConfig and :backup_config' if attributes.key?(:'backupConfig') && attributes.key?(:'backup_config')

      self.backup_config = attributes[:'backup_config'] if attributes[:'backup_config']

      self.key_store_id = attributes[:'keyStoreId'] if attributes[:'keyStoreId']

      raise 'You cannot provide both :keyStoreId and :key_store_id' if attributes.key?(:'keyStoreId') && attributes.key?(:'key_store_id')

      self.key_store_id = attributes[:'key_store_id'] if attributes[:'key_store_id']

      self.key_store_wallet_name = attributes[:'keyStoreWalletName'] if attributes[:'keyStoreWalletName']

      raise 'You cannot provide both :keyStoreWalletName and :key_store_wallet_name' if attributes.key?(:'keyStoreWalletName') && attributes.key?(:'key_store_wallet_name')

      self.key_store_wallet_name = attributes[:'key_store_wallet_name'] if attributes[:'key_store_wallet_name']

      self.memory_per_oracle_compute_unit_in_gbs = attributes[:'memoryPerOracleComputeUnitInGBs'] if attributes[:'memoryPerOracleComputeUnitInGBs']

      raise 'You cannot provide both :memoryPerOracleComputeUnitInGBs and :memory_per_oracle_compute_unit_in_gbs' if attributes.key?(:'memoryPerOracleComputeUnitInGBs') && attributes.key?(:'memory_per_oracle_compute_unit_in_gbs')

      self.memory_per_oracle_compute_unit_in_gbs = attributes[:'memory_per_oracle_compute_unit_in_gbs'] if attributes[:'memory_per_oracle_compute_unit_in_gbs']

      self.available_cpus = attributes[:'availableCpus'] if attributes[:'availableCpus']

      raise 'You cannot provide both :availableCpus and :available_cpus' if attributes.key?(:'availableCpus') && attributes.key?(:'available_cpus')

      self.available_cpus = attributes[:'available_cpus'] if attributes[:'available_cpus']

      self.total_cpus = attributes[:'totalCpus'] if attributes[:'totalCpus']

      raise 'You cannot provide both :totalCpus and :total_cpus' if attributes.key?(:'totalCpus') && attributes.key?(:'total_cpus')

      self.total_cpus = attributes[:'total_cpus'] if attributes[:'total_cpus']

      self.reclaimable_cpus = attributes[:'reclaimableCpus'] if attributes[:'reclaimableCpus']

      raise 'You cannot provide both :reclaimableCpus and :reclaimable_cpus' if attributes.key?(:'reclaimableCpus') && attributes.key?(:'reclaimable_cpus')

      self.reclaimable_cpus = attributes[:'reclaimable_cpus'] if attributes[:'reclaimable_cpus']

      self.provisionable_cpus = attributes[:'provisionableCpus'] if attributes[:'provisionableCpus']

      raise 'You cannot provide both :provisionableCpus and :provisionable_cpus' if attributes.key?(:'provisionableCpus') && attributes.key?(:'provisionable_cpus')

      self.provisionable_cpus = attributes[:'provisionable_cpus'] if attributes[:'provisionable_cpus']

      self.list_one_off_patches = attributes[:'listOneOffPatches'] if attributes[:'listOneOffPatches']

      raise 'You cannot provide both :listOneOffPatches and :list_one_off_patches' if attributes.key?(:'listOneOffPatches') && attributes.key?(:'list_one_off_patches')

      self.list_one_off_patches = attributes[:'list_one_off_patches'] if attributes[:'list_one_off_patches']

      self.compute_model = attributes[:'computeModel'] if attributes[:'computeModel']

      raise 'You cannot provide both :computeModel and :compute_model' if attributes.key?(:'computeModel') && attributes.key?(:'compute_model')

      self.compute_model = attributes[:'compute_model'] if attributes[:'compute_model']

      self.provisioned_cpus = attributes[:'provisionedCpus'] if attributes[:'provisionedCpus']

      raise 'You cannot provide both :provisionedCpus and :provisioned_cpus' if attributes.key?(:'provisionedCpus') && attributes.key?(:'provisioned_cpus')

      self.provisioned_cpus = attributes[:'provisioned_cpus'] if attributes[:'provisioned_cpus']

      self.reserved_cpus = attributes[:'reservedCpus'] if attributes[:'reservedCpus']

      raise 'You cannot provide both :reservedCpus and :reserved_cpus' if attributes.key?(:'reservedCpus') && attributes.key?(:'reserved_cpus')

      self.reserved_cpus = attributes[:'reserved_cpus'] if attributes[:'reserved_cpus']

      self.largest_provisionable_autonomous_database_in_cpus = attributes[:'largestProvisionableAutonomousDatabaseInCpus'] if attributes[:'largestProvisionableAutonomousDatabaseInCpus']

      raise 'You cannot provide both :largestProvisionableAutonomousDatabaseInCpus and :largest_provisionable_autonomous_database_in_cpus' if attributes.key?(:'largestProvisionableAutonomousDatabaseInCpus') && attributes.key?(:'largest_provisionable_autonomous_database_in_cpus')

      self.largest_provisionable_autonomous_database_in_cpus = attributes[:'largest_provisionable_autonomous_database_in_cpus'] if attributes[:'largest_provisionable_autonomous_database_in_cpus']

      self.time_of_last_backup = attributes[:'timeOfLastBackup'] if attributes[:'timeOfLastBackup']

      raise 'You cannot provide both :timeOfLastBackup and :time_of_last_backup' if attributes.key?(:'timeOfLastBackup') && attributes.key?(:'time_of_last_backup')

      self.time_of_last_backup = attributes[:'time_of_last_backup'] if attributes[:'time_of_last_backup']

      self.db_split_threshold = attributes[:'dbSplitThreshold'] if attributes[:'dbSplitThreshold']

      raise 'You cannot provide both :dbSplitThreshold and :db_split_threshold' if attributes.key?(:'dbSplitThreshold') && attributes.key?(:'db_split_threshold')

      self.db_split_threshold = attributes[:'db_split_threshold'] if attributes[:'db_split_threshold']

      self.vm_failover_reservation = attributes[:'vmFailoverReservation'] if attributes[:'vmFailoverReservation']

      raise 'You cannot provide both :vmFailoverReservation and :vm_failover_reservation' if attributes.key?(:'vmFailoverReservation') && attributes.key?(:'vm_failover_reservation')

      self.vm_failover_reservation = attributes[:'vm_failover_reservation'] if attributes[:'vm_failover_reservation']

      self.distribution_affinity = attributes[:'distributionAffinity'] if attributes[:'distributionAffinity']

      raise 'You cannot provide both :distributionAffinity and :distribution_affinity' if attributes.key?(:'distributionAffinity') && attributes.key?(:'distribution_affinity')

      self.distribution_affinity = attributes[:'distribution_affinity'] if attributes[:'distribution_affinity']

      self.net_services_architecture = attributes[:'netServicesArchitecture'] if attributes[:'netServicesArchitecture']

      raise 'You cannot provide both :netServicesArchitecture and :net_services_architecture' if attributes.key?(:'netServicesArchitecture') && attributes.key?(:'net_services_architecture')

      self.net_services_architecture = attributes[:'net_services_architecture'] if attributes[:'net_services_architecture']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] service_level_agreement_type Object to be assigned
    def service_level_agreement_type=(service_level_agreement_type)
      # rubocop:disable Style/ConditionalAssignment
      if service_level_agreement_type && !SERVICE_LEVEL_AGREEMENT_TYPE_ENUM.include?(service_level_agreement_type)
        OCI.logger.debug("Unknown value for 'service_level_agreement_type' [" + service_level_agreement_type + "]. Mapping to 'SERVICE_LEVEL_AGREEMENT_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @service_level_agreement_type = SERVICE_LEVEL_AGREEMENT_TYPE_UNKNOWN_ENUM_VALUE
      else
        @service_level_agreement_type = service_level_agreement_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] infrastructure_type Object to be assigned
    def infrastructure_type=(infrastructure_type)
      # rubocop:disable Style/ConditionalAssignment
      if infrastructure_type && !INFRASTRUCTURE_TYPE_ENUM.include?(infrastructure_type)
        OCI.logger.debug("Unknown value for 'infrastructure_type' [" + infrastructure_type + "]. Mapping to 'INFRASTRUCTURE_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @infrastructure_type = INFRASTRUCTURE_TYPE_UNKNOWN_ENUM_VALUE
      else
        @infrastructure_type = infrastructure_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] patch_model Object to be assigned
    def patch_model=(patch_model)
      # rubocop:disable Style/ConditionalAssignment
      if patch_model && !PATCH_MODEL_ENUM.include?(patch_model)
        OCI.logger.debug("Unknown value for 'patch_model' [" + patch_model + "]. Mapping to 'PATCH_MODEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @patch_model = PATCH_MODEL_UNKNOWN_ENUM_VALUE
      else
        @patch_model = patch_model
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] version_preference Object to be assigned
    def version_preference=(version_preference)
      # rubocop:disable Style/ConditionalAssignment
      if version_preference && !VERSION_PREFERENCE_ENUM.include?(version_preference)
        OCI.logger.debug("Unknown value for 'version_preference' [" + version_preference + "]. Mapping to 'VERSION_PREFERENCE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @version_preference = VERSION_PREFERENCE_UNKNOWN_ENUM_VALUE
      else
        @version_preference = version_preference
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] role Object to be assigned
    def role=(role)
      # rubocop:disable Style/ConditionalAssignment
      if role && !ROLE_ENUM.include?(role)
        OCI.logger.debug("Unknown value for 'role' [" + role + "]. Mapping to 'ROLE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @role = ROLE_UNKNOWN_ENUM_VALUE
      else
        @role = role
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] compute_model Object to be assigned
    def compute_model=(compute_model)
      # rubocop:disable Style/ConditionalAssignment
      if compute_model && !COMPUTE_MODEL_ENUM.include?(compute_model)
        OCI.logger.debug("Unknown value for 'compute_model' [" + compute_model + "]. Mapping to 'COMPUTE_MODEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @compute_model = COMPUTE_MODEL_UNKNOWN_ENUM_VALUE
      else
        @compute_model = compute_model
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] distribution_affinity Object to be assigned
    def distribution_affinity=(distribution_affinity)
      # rubocop:disable Style/ConditionalAssignment
      if distribution_affinity && !DISTRIBUTION_AFFINITY_ENUM.include?(distribution_affinity)
        OCI.logger.debug("Unknown value for 'distribution_affinity' [" + distribution_affinity + "]. Mapping to 'DISTRIBUTION_AFFINITY_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @distribution_affinity = DISTRIBUTION_AFFINITY_UNKNOWN_ENUM_VALUE
      else
        @distribution_affinity = distribution_affinity
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] net_services_architecture Object to be assigned
    def net_services_architecture=(net_services_architecture)
      # rubocop:disable Style/ConditionalAssignment
      if net_services_architecture && !NET_SERVICES_ARCHITECTURE_ENUM.include?(net_services_architecture)
        OCI.logger.debug("Unknown value for 'net_services_architecture' [" + net_services_architecture + "]. Mapping to 'NET_SERVICES_ARCHITECTURE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @net_services_architecture = NET_SERVICES_ARCHITECTURE_UNKNOWN_ENUM_VALUE
      else
        @net_services_architecture = net_services_architecture
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        compartment_id == other.compartment_id &&
        display_name == other.display_name &&
        db_unique_name == other.db_unique_name &&
        db_name == other.db_name &&
        service_level_agreement_type == other.service_level_agreement_type &&
        autonomous_exadata_infrastructure_id == other.autonomous_exadata_infrastructure_id &&
        autonomous_vm_cluster_id == other.autonomous_vm_cluster_id &&
        infrastructure_type == other.infrastructure_type &&
        cloud_autonomous_vm_cluster_id == other.cloud_autonomous_vm_cluster_id &&
        kms_key_id == other.kms_key_id &&
        vault_id == other.vault_id &&
        kms_key_version_id == other.kms_key_version_id &&
        key_history_entry == other.key_history_entry &&
        lifecycle_state == other.lifecycle_state &&
        lifecycle_details == other.lifecycle_details &&
        time_created == other.time_created &&
        time_snapshot_standby_revert == other.time_snapshot_standby_revert &&
        patch_model == other.patch_model &&
        patch_id == other.patch_id &&
        last_maintenance_run_id == other.last_maintenance_run_id &&
        next_maintenance_run_id == other.next_maintenance_run_id &&
        maintenance_window == other.maintenance_window &&
        standby_maintenance_buffer_in_days == other.standby_maintenance_buffer_in_days &&
        version_preference == other.version_preference &&
        is_dst_file_update_enabled == other.is_dst_file_update_enabled &&
        dst_file_version == other.dst_file_version &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        role == other.role &&
        availability_domain == other.availability_domain &&
        db_version == other.db_version &&
        backup_config == other.backup_config &&
        key_store_id == other.key_store_id &&
        key_store_wallet_name == other.key_store_wallet_name &&
        memory_per_oracle_compute_unit_in_gbs == other.memory_per_oracle_compute_unit_in_gbs &&
        available_cpus == other.available_cpus &&
        total_cpus == other.total_cpus &&
        reclaimable_cpus == other.reclaimable_cpus &&
        provisionable_cpus == other.provisionable_cpus &&
        list_one_off_patches == other.list_one_off_patches &&
        compute_model == other.compute_model &&
        provisioned_cpus == other.provisioned_cpus &&
        reserved_cpus == other.reserved_cpus &&
        largest_provisionable_autonomous_database_in_cpus == other.largest_provisionable_autonomous_database_in_cpus &&
        time_of_last_backup == other.time_of_last_backup &&
        db_split_threshold == other.db_split_threshold &&
        vm_failover_reservation == other.vm_failover_reservation &&
        distribution_affinity == other.distribution_affinity &&
        net_services_architecture == other.net_services_architecture
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, compartment_id, display_name, db_unique_name, db_name, service_level_agreement_type, autonomous_exadata_infrastructure_id, autonomous_vm_cluster_id, infrastructure_type, cloud_autonomous_vm_cluster_id, kms_key_id, vault_id, kms_key_version_id, key_history_entry, lifecycle_state, lifecycle_details, time_created, time_snapshot_standby_revert, patch_model, patch_id, last_maintenance_run_id, next_maintenance_run_id, maintenance_window, standby_maintenance_buffer_in_days, version_preference, is_dst_file_update_enabled, dst_file_version, freeform_tags, defined_tags, role, availability_domain, db_version, backup_config, key_store_id, key_store_wallet_name, memory_per_oracle_compute_unit_in_gbs, available_cpus, total_cpus, reclaimable_cpus, provisionable_cpus, list_one_off_patches, compute_model, provisioned_cpus, reserved_cpus, largest_provisionable_autonomous_database_in_cpus, time_of_last_backup, db_split_threshold, vm_failover_reservation, distribution_affinity, net_services_architecture].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
