# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20201101
require 'date'
require 'logger'
require_relative 'dbm_resource'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The details of the Exadata storage server.
  class DatabaseManagement::Models::ExternalExadataStorageServer < DatabaseManagement::Models::DbmResource
    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Exadata infrastructure.
    # @return [String]
    attr_accessor :exadata_infrastructure_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Exadata storage server grid.
    # @return [String]
    attr_accessor :storage_grid_id

    # The make model of the Exadata storage server.
    # @return [String]
    attr_accessor :make_model

    # The IP address of the Exadata storage server.
    # @return [String]
    attr_accessor :ip_address

    # The CPU count of the Exadata storage server.
    # @return [Float]
    attr_accessor :cpu_count

    # The Exadata storage server memory size in GB.
    # @return [Float]
    attr_accessor :memory_gb

    # The maximum hard disk IO operations per second of the Exadata storage server.
    # @return [Integer]
    attr_accessor :max_hard_disk_iops

    # The maximum hard disk IO throughput in MB/s of the Exadata storage server.
    # @return [Integer]
    attr_accessor :max_hard_disk_throughput

    # The maximum flash disk IO operations per second of the Exadata storage server.
    # @return [Integer]
    attr_accessor :max_flash_disk_iops

    # The maximum flash disk IO throughput in MB/s of the Exadata storage server.
    # @return [Integer]
    attr_accessor :max_flash_disk_throughput

    # @return [OCI::DatabaseManagement::Models::ExternalExadataStorageConnectorSummary]
    attr_accessor :connector

    # Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # System tags for this resource. Each key is predefined and scoped to a namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # System tags can be viewed by users, but can only be created by the system.
    #
    # Example: `{\"orcl-cloud\": {\"free-tier-retained\": \"true\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :system_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'display_name': :'displayName',
        'version': :'version',
        'internal_id': :'internalId',
        'status': :'status',
        'lifecycle_state': :'lifecycleState',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'lifecycle_details': :'lifecycleDetails',
        'additional_details': :'additionalDetails',
        'resource_type': :'resourceType',
        'exadata_infrastructure_id': :'exadataInfrastructureId',
        'storage_grid_id': :'storageGridId',
        'make_model': :'makeModel',
        'ip_address': :'ipAddress',
        'cpu_count': :'cpuCount',
        'memory_gb': :'memoryGB',
        'max_hard_disk_iops': :'maxHardDiskIOPS',
        'max_hard_disk_throughput': :'maxHardDiskThroughput',
        'max_flash_disk_iops': :'maxFlashDiskIOPS',
        'max_flash_disk_throughput': :'maxFlashDiskThroughput',
        'connector': :'connector',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'system_tags': :'systemTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'display_name': :'String',
        'version': :'String',
        'internal_id': :'String',
        'status': :'String',
        'lifecycle_state': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'lifecycle_details': :'String',
        'additional_details': :'Hash<String, String>',
        'resource_type': :'String',
        'exadata_infrastructure_id': :'String',
        'storage_grid_id': :'String',
        'make_model': :'String',
        'ip_address': :'String',
        'cpu_count': :'Float',
        'memory_gb': :'Float',
        'max_hard_disk_iops': :'Integer',
        'max_hard_disk_throughput': :'Integer',
        'max_flash_disk_iops': :'Integer',
        'max_flash_disk_throughput': :'Integer',
        'connector': :'OCI::DatabaseManagement::Models::ExternalExadataStorageConnectorSummary',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'system_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#id #id} proprety
    # @option attributes [String] :display_name The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#display_name #display_name} proprety
    # @option attributes [String] :version The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#version #version} proprety
    # @option attributes [String] :internal_id The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#internal_id #internal_id} proprety
    # @option attributes [String] :status The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#status #status} proprety
    # @option attributes [String] :lifecycle_state The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#lifecycle_state #lifecycle_state} proprety
    # @option attributes [DateTime] :time_created The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#time_created #time_created} proprety
    # @option attributes [DateTime] :time_updated The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#time_updated #time_updated} proprety
    # @option attributes [String] :lifecycle_details The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#lifecycle_details #lifecycle_details} proprety
    # @option attributes [Hash<String, String>] :additional_details The value to assign to the {OCI::DatabaseManagement::Models::DbmResource#additional_details #additional_details} proprety
    # @option attributes [String] :exadata_infrastructure_id The value to assign to the {#exadata_infrastructure_id} property
    # @option attributes [String] :storage_grid_id The value to assign to the {#storage_grid_id} property
    # @option attributes [String] :make_model The value to assign to the {#make_model} property
    # @option attributes [String] :ip_address The value to assign to the {#ip_address} property
    # @option attributes [Float] :cpu_count The value to assign to the {#cpu_count} property
    # @option attributes [Float] :memory_gb The value to assign to the {#memory_gb} property
    # @option attributes [Integer] :max_hard_disk_iops The value to assign to the {#max_hard_disk_iops} property
    # @option attributes [Integer] :max_hard_disk_throughput The value to assign to the {#max_hard_disk_throughput} property
    # @option attributes [Integer] :max_flash_disk_iops The value to assign to the {#max_flash_disk_iops} property
    # @option attributes [Integer] :max_flash_disk_throughput The value to assign to the {#max_flash_disk_throughput} property
    # @option attributes [OCI::DatabaseManagement::Models::ExternalExadataStorageConnectorSummary] :connector The value to assign to the {#connector} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :system_tags The value to assign to the {#system_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['resourceType'] = 'STORAGE_SERVER'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.exadata_infrastructure_id = attributes[:'exadataInfrastructureId'] if attributes[:'exadataInfrastructureId']

      raise 'You cannot provide both :exadataInfrastructureId and :exadata_infrastructure_id' if attributes.key?(:'exadataInfrastructureId') && attributes.key?(:'exadata_infrastructure_id')

      self.exadata_infrastructure_id = attributes[:'exadata_infrastructure_id'] if attributes[:'exadata_infrastructure_id']

      self.storage_grid_id = attributes[:'storageGridId'] if attributes[:'storageGridId']

      raise 'You cannot provide both :storageGridId and :storage_grid_id' if attributes.key?(:'storageGridId') && attributes.key?(:'storage_grid_id')

      self.storage_grid_id = attributes[:'storage_grid_id'] if attributes[:'storage_grid_id']

      self.make_model = attributes[:'makeModel'] if attributes[:'makeModel']

      raise 'You cannot provide both :makeModel and :make_model' if attributes.key?(:'makeModel') && attributes.key?(:'make_model')

      self.make_model = attributes[:'make_model'] if attributes[:'make_model']

      self.ip_address = attributes[:'ipAddress'] if attributes[:'ipAddress']

      raise 'You cannot provide both :ipAddress and :ip_address' if attributes.key?(:'ipAddress') && attributes.key?(:'ip_address')

      self.ip_address = attributes[:'ip_address'] if attributes[:'ip_address']

      self.cpu_count = attributes[:'cpuCount'] if attributes[:'cpuCount']

      raise 'You cannot provide both :cpuCount and :cpu_count' if attributes.key?(:'cpuCount') && attributes.key?(:'cpu_count')

      self.cpu_count = attributes[:'cpu_count'] if attributes[:'cpu_count']

      self.memory_gb = attributes[:'memoryGB'] if attributes[:'memoryGB']

      raise 'You cannot provide both :memoryGB and :memory_gb' if attributes.key?(:'memoryGB') && attributes.key?(:'memory_gb')

      self.memory_gb = attributes[:'memory_gb'] if attributes[:'memory_gb']

      self.max_hard_disk_iops = attributes[:'maxHardDiskIOPS'] if attributes[:'maxHardDiskIOPS']

      raise 'You cannot provide both :maxHardDiskIOPS and :max_hard_disk_iops' if attributes.key?(:'maxHardDiskIOPS') && attributes.key?(:'max_hard_disk_iops')

      self.max_hard_disk_iops = attributes[:'max_hard_disk_iops'] if attributes[:'max_hard_disk_iops']

      self.max_hard_disk_throughput = attributes[:'maxHardDiskThroughput'] if attributes[:'maxHardDiskThroughput']

      raise 'You cannot provide both :maxHardDiskThroughput and :max_hard_disk_throughput' if attributes.key?(:'maxHardDiskThroughput') && attributes.key?(:'max_hard_disk_throughput')

      self.max_hard_disk_throughput = attributes[:'max_hard_disk_throughput'] if attributes[:'max_hard_disk_throughput']

      self.max_flash_disk_iops = attributes[:'maxFlashDiskIOPS'] if attributes[:'maxFlashDiskIOPS']

      raise 'You cannot provide both :maxFlashDiskIOPS and :max_flash_disk_iops' if attributes.key?(:'maxFlashDiskIOPS') && attributes.key?(:'max_flash_disk_iops')

      self.max_flash_disk_iops = attributes[:'max_flash_disk_iops'] if attributes[:'max_flash_disk_iops']

      self.max_flash_disk_throughput = attributes[:'maxFlashDiskThroughput'] if attributes[:'maxFlashDiskThroughput']

      raise 'You cannot provide both :maxFlashDiskThroughput and :max_flash_disk_throughput' if attributes.key?(:'maxFlashDiskThroughput') && attributes.key?(:'max_flash_disk_throughput')

      self.max_flash_disk_throughput = attributes[:'max_flash_disk_throughput'] if attributes[:'max_flash_disk_throughput']

      self.connector = attributes[:'connector'] if attributes[:'connector']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.system_tags = attributes[:'systemTags'] if attributes[:'systemTags']

      raise 'You cannot provide both :systemTags and :system_tags' if attributes.key?(:'systemTags') && attributes.key?(:'system_tags')

      self.system_tags = attributes[:'system_tags'] if attributes[:'system_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        display_name == other.display_name &&
        version == other.version &&
        internal_id == other.internal_id &&
        status == other.status &&
        lifecycle_state == other.lifecycle_state &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        lifecycle_details == other.lifecycle_details &&
        additional_details == other.additional_details &&
        resource_type == other.resource_type &&
        exadata_infrastructure_id == other.exadata_infrastructure_id &&
        storage_grid_id == other.storage_grid_id &&
        make_model == other.make_model &&
        ip_address == other.ip_address &&
        cpu_count == other.cpu_count &&
        memory_gb == other.memory_gb &&
        max_hard_disk_iops == other.max_hard_disk_iops &&
        max_hard_disk_throughput == other.max_hard_disk_throughput &&
        max_flash_disk_iops == other.max_flash_disk_iops &&
        max_flash_disk_throughput == other.max_flash_disk_throughput &&
        connector == other.connector &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        system_tags == other.system_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, display_name, version, internal_id, status, lifecycle_state, time_created, time_updated, lifecycle_details, additional_details, resource_type, exadata_infrastructure_id, storage_grid_id, make_model, ip_address, cpu_count, memory_gb, max_hard_disk_iops, max_hard_disk_throughput, max_flash_disk_iops, max_flash_disk_throughput, connector, freeform_tags, defined_tags, system_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
