# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20201101
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The subset information of the Managed Database resource, which is used by Optimizer Statistics.
  class DatabaseManagement::Models::OptimizerDatabase
    DB_TYPE_ENUM = [
      DB_TYPE_EXTERNAL_SIDB = 'EXTERNAL_SIDB'.freeze,
      DB_TYPE_EXTERNAL_RAC = 'EXTERNAL_RAC'.freeze,
      DB_TYPE_CLOUD_SIDB = 'CLOUD_SIDB'.freeze,
      DB_TYPE_CLOUD_RAC = 'CLOUD_RAC'.freeze,
      DB_TYPE_SHARED = 'SHARED'.freeze,
      DB_TYPE_DEDICATED = 'DEDICATED'.freeze,
      DB_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    DB_SUB_TYPE_ENUM = [
      DB_SUB_TYPE_CDB = 'CDB'.freeze,
      DB_SUB_TYPE_PDB = 'PDB'.freeze,
      DB_SUB_TYPE_NON_CDB = 'NON_CDB'.freeze,
      DB_SUB_TYPE_ACD = 'ACD'.freeze,
      DB_SUB_TYPE_ADB = 'ADB'.freeze,
      DB_SUB_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    DB_DEPLOYMENT_TYPE_ENUM = [
      DB_DEPLOYMENT_TYPE_ONPREMISE = 'ONPREMISE'.freeze,
      DB_DEPLOYMENT_TYPE_BM = 'BM'.freeze,
      DB_DEPLOYMENT_TYPE_VM = 'VM'.freeze,
      DB_DEPLOYMENT_TYPE_EXADATA = 'EXADATA'.freeze,
      DB_DEPLOYMENT_TYPE_EXADATA_CC = 'EXADATA_CC'.freeze,
      DB_DEPLOYMENT_TYPE_AUTONOMOUS = 'AUTONOMOUS'.freeze,
      DB_DEPLOYMENT_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Managed Database.
    # @return [String]
    attr_accessor :id

    # **[Required]** The name of the Managed Database.
    # @return [String]
    attr_accessor :name

    # **[Required]** The type of Oracle Database installation.
    # @return [String]
    attr_reader :db_type

    # **[Required]** The subtype of the Oracle Database. Indicates whether the database is a Container Database,
    # Pluggable Database, Non-container Database, Autonomous Database, or Autonomous Container Database.
    #
    # @return [String]
    attr_reader :db_sub_type

    # **[Required]** The infrastructure used to deploy the Oracle Database.
    # @return [String]
    attr_reader :db_deployment_type

    # **[Required]** The version of the Oracle Database.
    # @return [String]
    attr_accessor :db_version

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the compartment in which the Managed Database resides.
    # @return [String]
    attr_accessor :compartment_id

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'name': :'name',
        'db_type': :'dbType',
        'db_sub_type': :'dbSubType',
        'db_deployment_type': :'dbDeploymentType',
        'db_version': :'dbVersion',
        'compartment_id': :'compartmentId'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'name': :'String',
        'db_type': :'String',
        'db_sub_type': :'String',
        'db_deployment_type': :'String',
        'db_version': :'String',
        'compartment_id': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [String] :db_type The value to assign to the {#db_type} property
    # @option attributes [String] :db_sub_type The value to assign to the {#db_sub_type} property
    # @option attributes [String] :db_deployment_type The value to assign to the {#db_deployment_type} property
    # @option attributes [String] :db_version The value to assign to the {#db_version} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.name = attributes[:'name'] if attributes[:'name']

      self.db_type = attributes[:'dbType'] if attributes[:'dbType']

      raise 'You cannot provide both :dbType and :db_type' if attributes.key?(:'dbType') && attributes.key?(:'db_type')

      self.db_type = attributes[:'db_type'] if attributes[:'db_type']

      self.db_sub_type = attributes[:'dbSubType'] if attributes[:'dbSubType']

      raise 'You cannot provide both :dbSubType and :db_sub_type' if attributes.key?(:'dbSubType') && attributes.key?(:'db_sub_type')

      self.db_sub_type = attributes[:'db_sub_type'] if attributes[:'db_sub_type']

      self.db_deployment_type = attributes[:'dbDeploymentType'] if attributes[:'dbDeploymentType']

      raise 'You cannot provide both :dbDeploymentType and :db_deployment_type' if attributes.key?(:'dbDeploymentType') && attributes.key?(:'db_deployment_type')

      self.db_deployment_type = attributes[:'db_deployment_type'] if attributes[:'db_deployment_type']

      self.db_version = attributes[:'dbVersion'] if attributes[:'dbVersion']

      raise 'You cannot provide both :dbVersion and :db_version' if attributes.key?(:'dbVersion') && attributes.key?(:'db_version')

      self.db_version = attributes[:'db_version'] if attributes[:'db_version']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] db_type Object to be assigned
    def db_type=(db_type)
      # rubocop:disable Style/ConditionalAssignment
      if db_type && !DB_TYPE_ENUM.include?(db_type)
        OCI.logger.debug("Unknown value for 'db_type' [" + db_type + "]. Mapping to 'DB_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @db_type = DB_TYPE_UNKNOWN_ENUM_VALUE
      else
        @db_type = db_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] db_sub_type Object to be assigned
    def db_sub_type=(db_sub_type)
      # rubocop:disable Style/ConditionalAssignment
      if db_sub_type && !DB_SUB_TYPE_ENUM.include?(db_sub_type)
        OCI.logger.debug("Unknown value for 'db_sub_type' [" + db_sub_type + "]. Mapping to 'DB_SUB_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @db_sub_type = DB_SUB_TYPE_UNKNOWN_ENUM_VALUE
      else
        @db_sub_type = db_sub_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] db_deployment_type Object to be assigned
    def db_deployment_type=(db_deployment_type)
      # rubocop:disable Style/ConditionalAssignment
      if db_deployment_type && !DB_DEPLOYMENT_TYPE_ENUM.include?(db_deployment_type)
        OCI.logger.debug("Unknown value for 'db_deployment_type' [" + db_deployment_type + "]. Mapping to 'DB_DEPLOYMENT_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @db_deployment_type = DB_DEPLOYMENT_TYPE_UNKNOWN_ENUM_VALUE
      else
        @db_deployment_type = db_deployment_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        name == other.name &&
        db_type == other.db_type &&
        db_sub_type == other.db_sub_type &&
        db_deployment_type == other.db_deployment_type &&
        db_version == other.db_version &&
        compartment_id == other.compartment_id
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, name, db_type, db_sub_type, db_deployment_type, db_version, compartment_id].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
