# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20171215
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details for response from replication estimation.
  class FileStorage::Models::ReplicationEstimate
    # **[Required]** The rate of change on source filesystem which was used to provide the estimate in MegaBytes per second.
    # @return [Integer]
    attr_accessor :change_rate_in_m_bps

    # **[Required]** Specifies whether replication can be enabled on the file system.
    # @return [BOOLEAN]
    attr_accessor :is_replication_supported

    # **[Required]** The minimum supported replication interval for specified file system in minutes.
    # @return [Integer]
    attr_accessor :minimum_supported_interval_in_minutes

    # **[Required]** The approximate time required for the base sync between source and target to finish.
    # @return [Integer]
    attr_accessor :estimated_base_copy_time_in_minutes

    # **[Required]** Array of allowed target region names which can be paired with source file system.
    # @return [Array<String>]
    attr_accessor :allowed_target_regions

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'change_rate_in_m_bps': :'changeRateInMBps',
        'is_replication_supported': :'isReplicationSupported',
        'minimum_supported_interval_in_minutes': :'minimumSupportedIntervalInMinutes',
        'estimated_base_copy_time_in_minutes': :'estimatedBaseCopyTimeInMinutes',
        'allowed_target_regions': :'allowedTargetRegions'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'change_rate_in_m_bps': :'Integer',
        'is_replication_supported': :'BOOLEAN',
        'minimum_supported_interval_in_minutes': :'Integer',
        'estimated_base_copy_time_in_minutes': :'Integer',
        'allowed_target_regions': :'Array<String>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [Integer] :change_rate_in_m_bps The value to assign to the {#change_rate_in_m_bps} property
    # @option attributes [BOOLEAN] :is_replication_supported The value to assign to the {#is_replication_supported} property
    # @option attributes [Integer] :minimum_supported_interval_in_minutes The value to assign to the {#minimum_supported_interval_in_minutes} property
    # @option attributes [Integer] :estimated_base_copy_time_in_minutes The value to assign to the {#estimated_base_copy_time_in_minutes} property
    # @option attributes [Array<String>] :allowed_target_regions The value to assign to the {#allowed_target_regions} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.change_rate_in_m_bps = attributes[:'changeRateInMBps'] if attributes[:'changeRateInMBps']

      raise 'You cannot provide both :changeRateInMBps and :change_rate_in_m_bps' if attributes.key?(:'changeRateInMBps') && attributes.key?(:'change_rate_in_m_bps')

      self.change_rate_in_m_bps = attributes[:'change_rate_in_m_bps'] if attributes[:'change_rate_in_m_bps']

      self.is_replication_supported = attributes[:'isReplicationSupported'] unless attributes[:'isReplicationSupported'].nil?

      raise 'You cannot provide both :isReplicationSupported and :is_replication_supported' if attributes.key?(:'isReplicationSupported') && attributes.key?(:'is_replication_supported')

      self.is_replication_supported = attributes[:'is_replication_supported'] unless attributes[:'is_replication_supported'].nil?

      self.minimum_supported_interval_in_minutes = attributes[:'minimumSupportedIntervalInMinutes'] if attributes[:'minimumSupportedIntervalInMinutes']

      raise 'You cannot provide both :minimumSupportedIntervalInMinutes and :minimum_supported_interval_in_minutes' if attributes.key?(:'minimumSupportedIntervalInMinutes') && attributes.key?(:'minimum_supported_interval_in_minutes')

      self.minimum_supported_interval_in_minutes = attributes[:'minimum_supported_interval_in_minutes'] if attributes[:'minimum_supported_interval_in_minutes']

      self.estimated_base_copy_time_in_minutes = attributes[:'estimatedBaseCopyTimeInMinutes'] if attributes[:'estimatedBaseCopyTimeInMinutes']

      raise 'You cannot provide both :estimatedBaseCopyTimeInMinutes and :estimated_base_copy_time_in_minutes' if attributes.key?(:'estimatedBaseCopyTimeInMinutes') && attributes.key?(:'estimated_base_copy_time_in_minutes')

      self.estimated_base_copy_time_in_minutes = attributes[:'estimated_base_copy_time_in_minutes'] if attributes[:'estimated_base_copy_time_in_minutes']

      self.allowed_target_regions = attributes[:'allowedTargetRegions'] if attributes[:'allowedTargetRegions']

      raise 'You cannot provide both :allowedTargetRegions and :allowed_target_regions' if attributes.key?(:'allowedTargetRegions') && attributes.key?(:'allowed_target_regions')

      self.allowed_target_regions = attributes[:'allowed_target_regions'] if attributes[:'allowed_target_regions']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        change_rate_in_m_bps == other.change_rate_in_m_bps &&
        is_replication_supported == other.is_replication_supported &&
        minimum_supported_interval_in_minutes == other.minimum_supported_interval_in_minutes &&
        estimated_base_copy_time_in_minutes == other.estimated_base_copy_time_in_minutes &&
        allowed_target_regions == other.allowed_target_regions
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [change_rate_in_m_bps, is_replication_supported, minimum_supported_interval_in_minutes, estimated_base_copy_time_in_minutes, allowed_target_regions].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
