# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220528
require 'date'
require_relative 'create_fsu_collection_details'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details to create a 'DB' type Exadata Fleet Update Collection.
  #
  class FleetSoftwareUpdate::Models::CreateDbFsuCollectionDetails < FleetSoftwareUpdate::Models::CreateFsuCollectionDetails
    SOURCE_MAJOR_VERSION_ENUM = [
      SOURCE_MAJOR_VERSION_DB_11204 = 'DB_11204'.freeze,
      SOURCE_MAJOR_VERSION_DB_121 = 'DB_121'.freeze,
      SOURCE_MAJOR_VERSION_DB_122 = 'DB_122'.freeze,
      SOURCE_MAJOR_VERSION_DB_18 = 'DB_18'.freeze,
      SOURCE_MAJOR_VERSION_DB_19 = 'DB_19'.freeze
    ].freeze

    # **[Required]** Database Major Version of targets to be included in the Exadata Fleet Update Collection.
    # https://docs.oracle.com/en-us/iaas/api/#/en/database/20160918/DbVersionSummary/ListDbVersions
    # Only Database targets that match the version specified in this value would be added to the Exadata Fleet Update Collection.
    #
    # @return [String]
    attr_reader :source_major_version

    # @return [OCI::FleetSoftwareUpdate::Models::DbFleetDiscoveryDetails]
    attr_accessor :fleet_discovery

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'display_name': :'displayName',
        'type': :'type',
        'service_type': :'serviceType',
        'compartment_id': :'compartmentId',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'source_major_version': :'sourceMajorVersion',
        'fleet_discovery': :'fleetDiscovery'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'display_name': :'String',
        'type': :'String',
        'service_type': :'String',
        'compartment_id': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'source_major_version': :'String',
        'fleet_discovery': :'OCI::FleetSoftwareUpdate::Models::DbFleetDiscoveryDetails'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :display_name The value to assign to the {OCI::FleetSoftwareUpdate::Models::CreateFsuCollectionDetails#display_name #display_name} proprety
    # @option attributes [String] :service_type The value to assign to the {OCI::FleetSoftwareUpdate::Models::CreateFsuCollectionDetails#service_type #service_type} proprety
    # @option attributes [String] :compartment_id The value to assign to the {OCI::FleetSoftwareUpdate::Models::CreateFsuCollectionDetails#compartment_id #compartment_id} proprety
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {OCI::FleetSoftwareUpdate::Models::CreateFsuCollectionDetails#freeform_tags #freeform_tags} proprety
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {OCI::FleetSoftwareUpdate::Models::CreateFsuCollectionDetails#defined_tags #defined_tags} proprety
    # @option attributes [String] :source_major_version The value to assign to the {#source_major_version} property
    # @option attributes [OCI::FleetSoftwareUpdate::Models::DbFleetDiscoveryDetails] :fleet_discovery The value to assign to the {#fleet_discovery} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['type'] = 'DB'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.source_major_version = attributes[:'sourceMajorVersion'] if attributes[:'sourceMajorVersion']

      raise 'You cannot provide both :sourceMajorVersion and :source_major_version' if attributes.key?(:'sourceMajorVersion') && attributes.key?(:'source_major_version')

      self.source_major_version = attributes[:'source_major_version'] if attributes[:'source_major_version']

      self.fleet_discovery = attributes[:'fleetDiscovery'] if attributes[:'fleetDiscovery']

      raise 'You cannot provide both :fleetDiscovery and :fleet_discovery' if attributes.key?(:'fleetDiscovery') && attributes.key?(:'fleet_discovery')

      self.fleet_discovery = attributes[:'fleet_discovery'] if attributes[:'fleet_discovery']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] source_major_version Object to be assigned
    def source_major_version=(source_major_version)
      raise "Invalid value for 'source_major_version': this must be one of the values in SOURCE_MAJOR_VERSION_ENUM." if source_major_version && !SOURCE_MAJOR_VERSION_ENUM.include?(source_major_version)

      @source_major_version = source_major_version
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        display_name == other.display_name &&
        type == other.type &&
        service_type == other.service_type &&
        compartment_id == other.compartment_id &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        source_major_version == other.source_major_version &&
        fleet_discovery == other.fleet_discovery
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [display_name, type, service_type, compartment_id, freeform_tags, defined_tags, source_major_version, fleet_discovery].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
