# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220528
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Goal version or image details for the Exadata Fleet Update Cycle.
  #
  # This class has direct subclasses. If you are using this class as input to a service operations then you should favor using a subclass over the base class
  class FleetSoftwareUpdate::Models::FsuGoalVersionDetails
    TYPE_ENUM = [
      TYPE_VERSION = 'VERSION'.freeze,
      TYPE_IMAGE_ID = 'IMAGE_ID'.freeze,
      TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    HOME_POLICY_ENUM = [
      HOME_POLICY_CREATE_NEW = 'CREATE_NEW'.freeze,
      HOME_POLICY_USE_EXISTING = 'USE_EXISTING'.freeze,
      HOME_POLICY_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Type of goal target version specified
    #
    # @return [String]
    attr_reader :type

    # Goal home policy to use when Staging the Goal Version during patching.
    # CREATE_NEW: Create a new DBHome (for Database Collections) for the specified image or version.
    # USE_EXISTING: All database targets in the same VMCluster or CloudVmCluster will be moved to a shared database home.
    #   If an existing home for the selected image or version is not found in the VM Cluster for a target database, then a new home will be created.
    #   If more than one existing home for the selected image is found, then the home with the least number of databases will be used.
    #   If multiple homes have the least number of databases, then a home will be selected at random.
    #
    # @return [String]
    attr_reader :home_policy

    # Prefix name used for new DB home resources created as part of the Stage Action.
    # Format: <specified_prefix>_<timestamp>
    # If not specified, a default OCI DB home resource will be generated for the new DB home resources created.
    #
    # @return [String]
    attr_accessor :new_home_prefix

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'type': :'type',
        'home_policy': :'homePolicy',
        'new_home_prefix': :'newHomePrefix'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'type': :'String',
        'home_policy': :'String',
        'new_home_prefix': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize


    # Given the hash representation of a subtype of this class,
    # use the info in the hash to return the class of the subtype.
    def self.get_subtype(object_hash)
      type = object_hash[:'type'] # rubocop:disable Style/SymbolLiteral

      return 'OCI::FleetSoftwareUpdate::Models::VersionFsuTargetDetails' if type == 'VERSION'
      return 'OCI::FleetSoftwareUpdate::Models::ImageIdFsuTargetDetails' if type == 'IMAGE_ID'

      # TODO: Log a warning when the subtype is not found.
      'OCI::FleetSoftwareUpdate::Models::FsuGoalVersionDetails'
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :type The value to assign to the {#type} property
    # @option attributes [String] :home_policy The value to assign to the {#home_policy} property
    # @option attributes [String] :new_home_prefix The value to assign to the {#new_home_prefix} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.type = attributes[:'type'] if attributes[:'type']

      self.home_policy = attributes[:'homePolicy'] if attributes[:'homePolicy']
      self.home_policy = "CREATE_NEW" if home_policy.nil? && !attributes.key?(:'homePolicy') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :homePolicy and :home_policy' if attributes.key?(:'homePolicy') && attributes.key?(:'home_policy')

      self.home_policy = attributes[:'home_policy'] if attributes[:'home_policy']
      self.home_policy = "CREATE_NEW" if home_policy.nil? && !attributes.key?(:'homePolicy') && !attributes.key?(:'home_policy') # rubocop:disable Style/StringLiterals

      self.new_home_prefix = attributes[:'newHomePrefix'] if attributes[:'newHomePrefix']

      raise 'You cannot provide both :newHomePrefix and :new_home_prefix' if attributes.key?(:'newHomePrefix') && attributes.key?(:'new_home_prefix')

      self.new_home_prefix = attributes[:'new_home_prefix'] if attributes[:'new_home_prefix']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] type Object to be assigned
    def type=(type)
      # rubocop:disable Style/ConditionalAssignment
      if type && !TYPE_ENUM.include?(type)
        OCI.logger.debug("Unknown value for 'type' [" + type + "]. Mapping to 'TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @type = TYPE_UNKNOWN_ENUM_VALUE
      else
        @type = type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] home_policy Object to be assigned
    def home_policy=(home_policy)
      # rubocop:disable Style/ConditionalAssignment
      if home_policy && !HOME_POLICY_ENUM.include?(home_policy)
        OCI.logger.debug("Unknown value for 'home_policy' [" + home_policy + "]. Mapping to 'HOME_POLICY_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @home_policy = HOME_POLICY_UNKNOWN_ENUM_VALUE
      else
        @home_policy = home_policy
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        type == other.type &&
        home_policy == other.home_policy &&
        new_home_prefix == other.new_home_prefix
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [type, home_policy, new_home_prefix].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
