# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20211201
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Describes the role of the FA Environment.
  class FusionApps::Models::EnvironmentRole
    CURRENT_ROLE_ENUM = [
      CURRENT_ROLE_PRIMARY = 'PRIMARY'.freeze,
      CURRENT_ROLE_STANDBY = 'STANDBY'.freeze,
      CURRENT_ROLE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # The current role of the environment
    # @return [String]
    attr_reader :current_role

    # Region the standby environment is in
    # @return [String]
    attr_accessor :standby_environment_region

    # Fusion Environment ID of the standby environment
    # @return [String]
    attr_accessor :standby_environment_id

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'current_role': :'currentRole',
        'standby_environment_region': :'standbyEnvironmentRegion',
        'standby_environment_id': :'standbyEnvironmentId'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'current_role': :'String',
        'standby_environment_region': :'String',
        'standby_environment_id': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :current_role The value to assign to the {#current_role} property
    # @option attributes [String] :standby_environment_region The value to assign to the {#standby_environment_region} property
    # @option attributes [String] :standby_environment_id The value to assign to the {#standby_environment_id} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.current_role = attributes[:'currentRole'] if attributes[:'currentRole']

      raise 'You cannot provide both :currentRole and :current_role' if attributes.key?(:'currentRole') && attributes.key?(:'current_role')

      self.current_role = attributes[:'current_role'] if attributes[:'current_role']

      self.standby_environment_region = attributes[:'standbyEnvironmentRegion'] if attributes[:'standbyEnvironmentRegion']

      raise 'You cannot provide both :standbyEnvironmentRegion and :standby_environment_region' if attributes.key?(:'standbyEnvironmentRegion') && attributes.key?(:'standby_environment_region')

      self.standby_environment_region = attributes[:'standby_environment_region'] if attributes[:'standby_environment_region']

      self.standby_environment_id = attributes[:'standbyEnvironmentId'] if attributes[:'standbyEnvironmentId']

      raise 'You cannot provide both :standbyEnvironmentId and :standby_environment_id' if attributes.key?(:'standbyEnvironmentId') && attributes.key?(:'standby_environment_id')

      self.standby_environment_id = attributes[:'standby_environment_id'] if attributes[:'standby_environment_id']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] current_role Object to be assigned
    def current_role=(current_role)
      # rubocop:disable Style/ConditionalAssignment
      if current_role && !CURRENT_ROLE_ENUM.include?(current_role)
        OCI.logger.debug("Unknown value for 'current_role' [" + current_role + "]. Mapping to 'CURRENT_ROLE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @current_role = CURRENT_ROLE_UNKNOWN_ENUM_VALUE
      else
        @current_role = current_role
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        current_role == other.current_role &&
        standby_environment_region == other.standby_environment_region &&
        standby_environment_id == other.standby_environment_id
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [current_role, standby_environment_region, standby_environment_id].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
