# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20231130
require 'date'
require_relative 'base_chat_request'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details for the chat request for Cohere models.
  class GenerativeAiInference::Models::CohereChatRequest < GenerativeAiInference::Models::BaseChatRequest
    # **[Required]** Text input for the model to respond to.
    # @return [String]
    attr_accessor :message

    # A list of previous messages between the user and the model, meant to give the model conversational context for responding to the user's message.
    # @return [Array<OCI::GenerativeAiInference::Models::CohereMessage>]
    attr_accessor :chat_history

    # list of relevant documents that the model can cite to generate a more accurate reply.
    # Some suggested keys are \"text\", \"author\", and \"date\". For better generation quality, it is
    # recommended to keep the total word count of the strings in the dictionary to under 300
    # words.
    #
    # @return [Array<Object>]
    attr_accessor :documents

    # When true, the response will only contain a list of generated search queries, but no search will take place, and no reply from the model to the user's message will be generated.
    # @return [BOOLEAN]
    attr_accessor :is_search_queries_only

    # When specified, the default Cohere preamble will be replaced with the provided one. Preambles are a part of the prompt used to adjust the model's overall behavior and conversation style. Default preambles vary for different models.
    # @return [String]
    attr_accessor :preamble_override

    # Whether to stream back partial progress. If set, tokens are sent as data-only server-sent events as they become available.
    # @return [BOOLEAN]
    attr_accessor :is_stream

    # The maximum number of tokens to predict for each response. Includes input plus output tokens.
    # @return [Integer]
    attr_accessor :max_tokens

    # A number that sets the randomness of the generated output. A lower temperature means a less random generations.
    # Use lower numbers for tasks with a correct answer such as question answering or summarizing. High temperatures can generate hallucinations or factually incorrect information. Start with temperatures lower than 1.0 and increase the temperature for more creative outputs, as you regenerate the prompts to refine the outputs.
    #
    # @return [Float]
    attr_accessor :temperature

    # An integer that sets up the model to use only the top k most likely tokens in the generated output. A higher k introduces more randomness into the output making the output text sound more natural. Default value is 0 which disables this method and considers all tokens. To set a number for the likely tokens, choose an integer between 1 and 500.
    #
    # If also using top p, then the model considers only the top tokens whose probabilities add up to p percent and ignores the rest of the k tokens. For example, if k is 20, but the probabilities of the top 10 add up to .75, then only the top 10 tokens are chosen.
    #
    # @return [Integer]
    attr_accessor :top_k

    # If set to a probability 0.0 < p < 1.0, it ensures that only the most likely tokens, with total probability mass of p, are considered for generation at each step.
    #
    # To eliminate tokens with low likelihood, assign p a minimum percentage for the next token's likelihood. For example, when p is set to 0.75, the model eliminates the bottom 25 percent for the next token. Set to 1.0 to consider all tokens and set to 0 to disable. If both k and p are enabled, p acts after k.
    #
    # @return [Float]
    attr_accessor :top_p

    # To reduce repetitiveness of generated tokens, this number penalizes new tokens based on their frequency in the generated text so far. Greater numbers encourage the model to use new tokens, while lower numbers encourage the model to repeat the tokens. Set to 0 to disable.
    #
    # @return [Float]
    attr_accessor :frequency_penalty

    # To reduce repetitiveness of generated tokens, this number penalizes new tokens based on whether they've appeared in the generated text so far. Greater numbers encourage the model to use new tokens, while lower numbers encourage the model to repeat the tokens.
    #
    # Similar to frequency penalty, a penalty is applied to previously present tokens, except that this penalty is applied equally to all tokens that have already appeared, regardless of how many times they've appeared. Set to 0 to disable.
    #
    # @return [Float]
    attr_accessor :presence_penalty

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'api_format': :'apiFormat',
        'message': :'message',
        'chat_history': :'chatHistory',
        'documents': :'documents',
        'is_search_queries_only': :'isSearchQueriesOnly',
        'preamble_override': :'preambleOverride',
        'is_stream': :'isStream',
        'max_tokens': :'maxTokens',
        'temperature': :'temperature',
        'top_k': :'topK',
        'top_p': :'topP',
        'frequency_penalty': :'frequencyPenalty',
        'presence_penalty': :'presencePenalty'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'api_format': :'String',
        'message': :'String',
        'chat_history': :'Array<OCI::GenerativeAiInference::Models::CohereMessage>',
        'documents': :'Array<Object>',
        'is_search_queries_only': :'BOOLEAN',
        'preamble_override': :'String',
        'is_stream': :'BOOLEAN',
        'max_tokens': :'Integer',
        'temperature': :'Float',
        'top_k': :'Integer',
        'top_p': :'Float',
        'frequency_penalty': :'Float',
        'presence_penalty': :'Float'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :message The value to assign to the {#message} property
    # @option attributes [Array<OCI::GenerativeAiInference::Models::CohereMessage>] :chat_history The value to assign to the {#chat_history} property
    # @option attributes [Array<Object>] :documents The value to assign to the {#documents} property
    # @option attributes [BOOLEAN] :is_search_queries_only The value to assign to the {#is_search_queries_only} property
    # @option attributes [String] :preamble_override The value to assign to the {#preamble_override} property
    # @option attributes [BOOLEAN] :is_stream The value to assign to the {#is_stream} property
    # @option attributes [Integer] :max_tokens The value to assign to the {#max_tokens} property
    # @option attributes [Float] :temperature The value to assign to the {#temperature} property
    # @option attributes [Integer] :top_k The value to assign to the {#top_k} property
    # @option attributes [Float] :top_p The value to assign to the {#top_p} property
    # @option attributes [Float] :frequency_penalty The value to assign to the {#frequency_penalty} property
    # @option attributes [Float] :presence_penalty The value to assign to the {#presence_penalty} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['apiFormat'] = 'COHERE'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.message = attributes[:'message'] if attributes[:'message']

      self.chat_history = attributes[:'chatHistory'] if attributes[:'chatHistory']

      raise 'You cannot provide both :chatHistory and :chat_history' if attributes.key?(:'chatHistory') && attributes.key?(:'chat_history')

      self.chat_history = attributes[:'chat_history'] if attributes[:'chat_history']

      self.documents = attributes[:'documents'] if attributes[:'documents']

      self.is_search_queries_only = attributes[:'isSearchQueriesOnly'] unless attributes[:'isSearchQueriesOnly'].nil?
      self.is_search_queries_only = false if is_search_queries_only.nil? && !attributes.key?(:'isSearchQueriesOnly') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isSearchQueriesOnly and :is_search_queries_only' if attributes.key?(:'isSearchQueriesOnly') && attributes.key?(:'is_search_queries_only')

      self.is_search_queries_only = attributes[:'is_search_queries_only'] unless attributes[:'is_search_queries_only'].nil?
      self.is_search_queries_only = false if is_search_queries_only.nil? && !attributes.key?(:'isSearchQueriesOnly') && !attributes.key?(:'is_search_queries_only') # rubocop:disable Style/StringLiterals

      self.preamble_override = attributes[:'preambleOverride'] if attributes[:'preambleOverride']

      raise 'You cannot provide both :preambleOverride and :preamble_override' if attributes.key?(:'preambleOverride') && attributes.key?(:'preamble_override')

      self.preamble_override = attributes[:'preamble_override'] if attributes[:'preamble_override']

      self.is_stream = attributes[:'isStream'] unless attributes[:'isStream'].nil?
      self.is_stream = false if is_stream.nil? && !attributes.key?(:'isStream') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isStream and :is_stream' if attributes.key?(:'isStream') && attributes.key?(:'is_stream')

      self.is_stream = attributes[:'is_stream'] unless attributes[:'is_stream'].nil?
      self.is_stream = false if is_stream.nil? && !attributes.key?(:'isStream') && !attributes.key?(:'is_stream') # rubocop:disable Style/StringLiterals

      self.max_tokens = attributes[:'maxTokens'] if attributes[:'maxTokens']

      raise 'You cannot provide both :maxTokens and :max_tokens' if attributes.key?(:'maxTokens') && attributes.key?(:'max_tokens')

      self.max_tokens = attributes[:'max_tokens'] if attributes[:'max_tokens']

      self.temperature = attributes[:'temperature'] if attributes[:'temperature']
      self.temperature = 0.3 if temperature.nil? && !attributes.key?(:'temperature') # rubocop:disable Style/StringLiterals

      self.top_k = attributes[:'topK'] if attributes[:'topK']

      raise 'You cannot provide both :topK and :top_k' if attributes.key?(:'topK') && attributes.key?(:'top_k')

      self.top_k = attributes[:'top_k'] if attributes[:'top_k']

      self.top_p = attributes[:'topP'] if attributes[:'topP']
      self.top_p = 0.75 if top_p.nil? && !attributes.key?(:'topP') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :topP and :top_p' if attributes.key?(:'topP') && attributes.key?(:'top_p')

      self.top_p = attributes[:'top_p'] if attributes[:'top_p']
      self.top_p = 0.75 if top_p.nil? && !attributes.key?(:'topP') && !attributes.key?(:'top_p') # rubocop:disable Style/StringLiterals

      self.frequency_penalty = attributes[:'frequencyPenalty'] if attributes[:'frequencyPenalty']
      self.frequency_penalty = 0.0 if frequency_penalty.nil? && !attributes.key?(:'frequencyPenalty') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :frequencyPenalty and :frequency_penalty' if attributes.key?(:'frequencyPenalty') && attributes.key?(:'frequency_penalty')

      self.frequency_penalty = attributes[:'frequency_penalty'] if attributes[:'frequency_penalty']
      self.frequency_penalty = 0.0 if frequency_penalty.nil? && !attributes.key?(:'frequencyPenalty') && !attributes.key?(:'frequency_penalty') # rubocop:disable Style/StringLiterals

      self.presence_penalty = attributes[:'presencePenalty'] if attributes[:'presencePenalty']
      self.presence_penalty = 0.0 if presence_penalty.nil? && !attributes.key?(:'presencePenalty') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :presencePenalty and :presence_penalty' if attributes.key?(:'presencePenalty') && attributes.key?(:'presence_penalty')

      self.presence_penalty = attributes[:'presence_penalty'] if attributes[:'presence_penalty']
      self.presence_penalty = 0.0 if presence_penalty.nil? && !attributes.key?(:'presencePenalty') && !attributes.key?(:'presence_penalty') # rubocop:disable Style/StringLiterals
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        api_format == other.api_format &&
        message == other.message &&
        chat_history == other.chat_history &&
        documents == other.documents &&
        is_search_queries_only == other.is_search_queries_only &&
        preamble_override == other.preamble_override &&
        is_stream == other.is_stream &&
        max_tokens == other.max_tokens &&
        temperature == other.temperature &&
        top_k == other.top_k &&
        top_p == other.top_p &&
        frequency_penalty == other.frequency_penalty &&
        presence_penalty == other.presence_penalty
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [api_format, message, chat_history, documents, is_search_queries_only, preamble_override, is_stream, max_tokens, temperature, top_k, top_p, frequency_penalty, presence_penalty].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
