# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20230301
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details of ATP-D based shard.
  class GloballyDistributedDatabase::Models::DedicatedShardDetails
    STATUS_ENUM = [
      STATUS_FAILED = 'FAILED'.freeze,
      STATUS_DELETING = 'DELETING'.freeze,
      STATUS_DELETED = 'DELETED'.freeze,
      STATUS_UPDATING = 'UPDATING'.freeze,
      STATUS_CREATING = 'CREATING'.freeze,
      STATUS_CREATED = 'CREATED'.freeze,
      STATUS_READY_FOR_CONFIGURATION = 'READY_FOR_CONFIGURATION'.freeze,
      STATUS_CONFIGURED = 'CONFIGURED'.freeze,
      STATUS_NEEDS_ATTENTION = 'NEEDS_ATTENTION'.freeze,
      STATUS_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # @return [OCI::GloballyDistributedDatabase::Models::DedicatedShardOrCatalogEncryptionKeyDetails]
    attr_accessor :encryption_key_details

    # **[Required]** Name of the shard.
    # @return [String]
    attr_accessor :name

    # **[Required]** The compute amount available to the underlying autonomous database associated with shard.
    # @return [Float]
    attr_accessor :compute_count

    # **[Required]** The data disk group size to be allocated in GBs.
    # @return [Float]
    attr_accessor :data_storage_size_in_gbs

    # **[Required]** Name of the shard-group to which the shard belongs.
    # @return [String]
    attr_accessor :shard_group

    # **[Required]** The time the the shard was created. An RFC3339 formatted datetime string
    # @return [DateTime]
    attr_accessor :time_created

    # **[Required]** The time the shard was last updated. An RFC3339 formatted datetime string
    # @return [DateTime]
    attr_accessor :time_updated

    # The time the ssl certificate associated with shard expires. An RFC3339 formatted datetime string
    # @return [DateTime]
    attr_accessor :time_ssl_certificate_expires

    # **[Required]** Status of shard or catalog or gsm for the sharded database.
    # @return [String]
    attr_reader :status

    # Shard space name.
    # @return [String]
    attr_accessor :shard_space

    # Identifier of the underlying supporting resource.
    #
    # @return [String]
    attr_accessor :supporting_resource_id

    # Identifier of the underlying container database.
    #
    # @return [String]
    attr_accessor :container_database_id

    # Identifier of the underlying container database parent.
    #
    # @return [String]
    attr_accessor :container_database_parent_id

    # **[Required]** Determines the auto-scaling mode.
    # @return [BOOLEAN]
    attr_accessor :is_auto_scaling_enabled

    # **[Required]** Identifier of the primary cloudAutonomousVmCluster for the shard.
    #
    # @return [String]
    attr_accessor :cloud_autonomous_vm_cluster_id

    # Identifier of the peer cloudAutonomousVmCluster for the shard.
    #
    # @return [String]
    attr_accessor :peer_cloud_autonomous_vm_cluster_id

    # Additional metadata related to shard's underlying supporting resource.
    # @return [Hash<String, Object>]
    attr_accessor :metadata

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'encryption_key_details': :'encryptionKeyDetails',
        'name': :'name',
        'compute_count': :'computeCount',
        'data_storage_size_in_gbs': :'dataStorageSizeInGbs',
        'shard_group': :'shardGroup',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'time_ssl_certificate_expires': :'timeSslCertificateExpires',
        'status': :'status',
        'shard_space': :'shardSpace',
        'supporting_resource_id': :'supportingResourceId',
        'container_database_id': :'containerDatabaseId',
        'container_database_parent_id': :'containerDatabaseParentId',
        'is_auto_scaling_enabled': :'isAutoScalingEnabled',
        'cloud_autonomous_vm_cluster_id': :'cloudAutonomousVmClusterId',
        'peer_cloud_autonomous_vm_cluster_id': :'peerCloudAutonomousVmClusterId',
        'metadata': :'metadata'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'encryption_key_details': :'OCI::GloballyDistributedDatabase::Models::DedicatedShardOrCatalogEncryptionKeyDetails',
        'name': :'String',
        'compute_count': :'Float',
        'data_storage_size_in_gbs': :'Float',
        'shard_group': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'time_ssl_certificate_expires': :'DateTime',
        'status': :'String',
        'shard_space': :'String',
        'supporting_resource_id': :'String',
        'container_database_id': :'String',
        'container_database_parent_id': :'String',
        'is_auto_scaling_enabled': :'BOOLEAN',
        'cloud_autonomous_vm_cluster_id': :'String',
        'peer_cloud_autonomous_vm_cluster_id': :'String',
        'metadata': :'Hash<String, Object>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [OCI::GloballyDistributedDatabase::Models::DedicatedShardOrCatalogEncryptionKeyDetails] :encryption_key_details The value to assign to the {#encryption_key_details} property
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [Float] :compute_count The value to assign to the {#compute_count} property
    # @option attributes [Float] :data_storage_size_in_gbs The value to assign to the {#data_storage_size_in_gbs} property
    # @option attributes [String] :shard_group The value to assign to the {#shard_group} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [DateTime] :time_ssl_certificate_expires The value to assign to the {#time_ssl_certificate_expires} property
    # @option attributes [String] :status The value to assign to the {#status} property
    # @option attributes [String] :shard_space The value to assign to the {#shard_space} property
    # @option attributes [String] :supporting_resource_id The value to assign to the {#supporting_resource_id} property
    # @option attributes [String] :container_database_id The value to assign to the {#container_database_id} property
    # @option attributes [String] :container_database_parent_id The value to assign to the {#container_database_parent_id} property
    # @option attributes [BOOLEAN] :is_auto_scaling_enabled The value to assign to the {#is_auto_scaling_enabled} property
    # @option attributes [String] :cloud_autonomous_vm_cluster_id The value to assign to the {#cloud_autonomous_vm_cluster_id} property
    # @option attributes [String] :peer_cloud_autonomous_vm_cluster_id The value to assign to the {#peer_cloud_autonomous_vm_cluster_id} property
    # @option attributes [Hash<String, Object>] :metadata The value to assign to the {#metadata} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.encryption_key_details = attributes[:'encryptionKeyDetails'] if attributes[:'encryptionKeyDetails']

      raise 'You cannot provide both :encryptionKeyDetails and :encryption_key_details' if attributes.key?(:'encryptionKeyDetails') && attributes.key?(:'encryption_key_details')

      self.encryption_key_details = attributes[:'encryption_key_details'] if attributes[:'encryption_key_details']

      self.name = attributes[:'name'] if attributes[:'name']

      self.compute_count = attributes[:'computeCount'] if attributes[:'computeCount']

      raise 'You cannot provide both :computeCount and :compute_count' if attributes.key?(:'computeCount') && attributes.key?(:'compute_count')

      self.compute_count = attributes[:'compute_count'] if attributes[:'compute_count']

      self.data_storage_size_in_gbs = attributes[:'dataStorageSizeInGbs'] if attributes[:'dataStorageSizeInGbs']

      raise 'You cannot provide both :dataStorageSizeInGbs and :data_storage_size_in_gbs' if attributes.key?(:'dataStorageSizeInGbs') && attributes.key?(:'data_storage_size_in_gbs')

      self.data_storage_size_in_gbs = attributes[:'data_storage_size_in_gbs'] if attributes[:'data_storage_size_in_gbs']

      self.shard_group = attributes[:'shardGroup'] if attributes[:'shardGroup']

      raise 'You cannot provide both :shardGroup and :shard_group' if attributes.key?(:'shardGroup') && attributes.key?(:'shard_group')

      self.shard_group = attributes[:'shard_group'] if attributes[:'shard_group']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.time_ssl_certificate_expires = attributes[:'timeSslCertificateExpires'] if attributes[:'timeSslCertificateExpires']

      raise 'You cannot provide both :timeSslCertificateExpires and :time_ssl_certificate_expires' if attributes.key?(:'timeSslCertificateExpires') && attributes.key?(:'time_ssl_certificate_expires')

      self.time_ssl_certificate_expires = attributes[:'time_ssl_certificate_expires'] if attributes[:'time_ssl_certificate_expires']

      self.status = attributes[:'status'] if attributes[:'status']

      self.shard_space = attributes[:'shardSpace'] if attributes[:'shardSpace']

      raise 'You cannot provide both :shardSpace and :shard_space' if attributes.key?(:'shardSpace') && attributes.key?(:'shard_space')

      self.shard_space = attributes[:'shard_space'] if attributes[:'shard_space']

      self.supporting_resource_id = attributes[:'supportingResourceId'] if attributes[:'supportingResourceId']

      raise 'You cannot provide both :supportingResourceId and :supporting_resource_id' if attributes.key?(:'supportingResourceId') && attributes.key?(:'supporting_resource_id')

      self.supporting_resource_id = attributes[:'supporting_resource_id'] if attributes[:'supporting_resource_id']

      self.container_database_id = attributes[:'containerDatabaseId'] if attributes[:'containerDatabaseId']

      raise 'You cannot provide both :containerDatabaseId and :container_database_id' if attributes.key?(:'containerDatabaseId') && attributes.key?(:'container_database_id')

      self.container_database_id = attributes[:'container_database_id'] if attributes[:'container_database_id']

      self.container_database_parent_id = attributes[:'containerDatabaseParentId'] if attributes[:'containerDatabaseParentId']

      raise 'You cannot provide both :containerDatabaseParentId and :container_database_parent_id' if attributes.key?(:'containerDatabaseParentId') && attributes.key?(:'container_database_parent_id')

      self.container_database_parent_id = attributes[:'container_database_parent_id'] if attributes[:'container_database_parent_id']

      self.is_auto_scaling_enabled = attributes[:'isAutoScalingEnabled'] unless attributes[:'isAutoScalingEnabled'].nil?

      raise 'You cannot provide both :isAutoScalingEnabled and :is_auto_scaling_enabled' if attributes.key?(:'isAutoScalingEnabled') && attributes.key?(:'is_auto_scaling_enabled')

      self.is_auto_scaling_enabled = attributes[:'is_auto_scaling_enabled'] unless attributes[:'is_auto_scaling_enabled'].nil?

      self.cloud_autonomous_vm_cluster_id = attributes[:'cloudAutonomousVmClusterId'] if attributes[:'cloudAutonomousVmClusterId']

      raise 'You cannot provide both :cloudAutonomousVmClusterId and :cloud_autonomous_vm_cluster_id' if attributes.key?(:'cloudAutonomousVmClusterId') && attributes.key?(:'cloud_autonomous_vm_cluster_id')

      self.cloud_autonomous_vm_cluster_id = attributes[:'cloud_autonomous_vm_cluster_id'] if attributes[:'cloud_autonomous_vm_cluster_id']

      self.peer_cloud_autonomous_vm_cluster_id = attributes[:'peerCloudAutonomousVmClusterId'] if attributes[:'peerCloudAutonomousVmClusterId']

      raise 'You cannot provide both :peerCloudAutonomousVmClusterId and :peer_cloud_autonomous_vm_cluster_id' if attributes.key?(:'peerCloudAutonomousVmClusterId') && attributes.key?(:'peer_cloud_autonomous_vm_cluster_id')

      self.peer_cloud_autonomous_vm_cluster_id = attributes[:'peer_cloud_autonomous_vm_cluster_id'] if attributes[:'peer_cloud_autonomous_vm_cluster_id']

      self.metadata = attributes[:'metadata'] if attributes[:'metadata']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] status Object to be assigned
    def status=(status)
      # rubocop:disable Style/ConditionalAssignment
      if status && !STATUS_ENUM.include?(status)
        OCI.logger.debug("Unknown value for 'status' [" + status + "]. Mapping to 'STATUS_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @status = STATUS_UNKNOWN_ENUM_VALUE
      else
        @status = status
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        encryption_key_details == other.encryption_key_details &&
        name == other.name &&
        compute_count == other.compute_count &&
        data_storage_size_in_gbs == other.data_storage_size_in_gbs &&
        shard_group == other.shard_group &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        time_ssl_certificate_expires == other.time_ssl_certificate_expires &&
        status == other.status &&
        shard_space == other.shard_space &&
        supporting_resource_id == other.supporting_resource_id &&
        container_database_id == other.container_database_id &&
        container_database_parent_id == other.container_database_parent_id &&
        is_auto_scaling_enabled == other.is_auto_scaling_enabled &&
        cloud_autonomous_vm_cluster_id == other.cloud_autonomous_vm_cluster_id &&
        peer_cloud_autonomous_vm_cluster_id == other.peer_cloud_autonomous_vm_cluster_id &&
        metadata == other.metadata
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [encryption_key_details, name, compute_count, data_storage_size_in_gbs, shard_group, time_created, time_updated, time_ssl_certificate_expires, status, shard_space, supporting_resource_id, container_database_id, container_database_parent_id, is_auto_scaling_enabled, cloud_autonomous_vm_cluster_id, peer_cloud_autonomous_vm_cluster_id, metadata].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
