# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: v1
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Configurable options maintaining a pool of ICF connector instances. Values for sub attributes can be set only if the ConnectorBundle referenced in the App has connectorPoolingSupported set to true
  #
  # **SCIM++ Properties:**
  #  - idcsSearchable: false
  #  - multiValued: false
  #  - mutability: readWrite
  #  - required: false
  #  - returned: default
  #  - type: complex
  #  - uniqueness: none
  class IdentityDomains::Models::AppBundlePoolConfiguration
    # Maximum number of connector instances in the pool that are idle and active.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: integer
    #  - uniqueness: none
    # @return [Integer]
    attr_accessor :max_idle

    # Maximum number of connector instances in the pool that are idle and active.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: integer
    #  - uniqueness: none
    # @return [Integer]
    attr_accessor :max_objects

    # Maximum time (in milliseconds) to wait for a free connector instance to become available before failing.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: integer
    #  - uniqueness: none
    # @return [Integer]
    attr_accessor :max_wait

    # Minimum time (in milliseconds) to wait before evicting an idle conenctor instance from the pool.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: integer
    #  - uniqueness: none
    # @return [Integer]
    attr_accessor :min_evictable_idle_time_millis

    # Minimum number of idle connector instances in the pool.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: integer
    #  - uniqueness: none
    # @return [Integer]
    attr_accessor :min_idle

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'max_idle': :'maxIdle',
        'max_objects': :'maxObjects',
        'max_wait': :'maxWait',
        'min_evictable_idle_time_millis': :'minEvictableIdleTimeMillis',
        'min_idle': :'minIdle'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'max_idle': :'Integer',
        'max_objects': :'Integer',
        'max_wait': :'Integer',
        'min_evictable_idle_time_millis': :'Integer',
        'min_idle': :'Integer'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [Integer] :max_idle The value to assign to the {#max_idle} property
    # @option attributes [Integer] :max_objects The value to assign to the {#max_objects} property
    # @option attributes [Integer] :max_wait The value to assign to the {#max_wait} property
    # @option attributes [Integer] :min_evictable_idle_time_millis The value to assign to the {#min_evictable_idle_time_millis} property
    # @option attributes [Integer] :min_idle The value to assign to the {#min_idle} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.max_idle = attributes[:'maxIdle'] if attributes[:'maxIdle']

      raise 'You cannot provide both :maxIdle and :max_idle' if attributes.key?(:'maxIdle') && attributes.key?(:'max_idle')

      self.max_idle = attributes[:'max_idle'] if attributes[:'max_idle']

      self.max_objects = attributes[:'maxObjects'] if attributes[:'maxObjects']

      raise 'You cannot provide both :maxObjects and :max_objects' if attributes.key?(:'maxObjects') && attributes.key?(:'max_objects')

      self.max_objects = attributes[:'max_objects'] if attributes[:'max_objects']

      self.max_wait = attributes[:'maxWait'] if attributes[:'maxWait']

      raise 'You cannot provide both :maxWait and :max_wait' if attributes.key?(:'maxWait') && attributes.key?(:'max_wait')

      self.max_wait = attributes[:'max_wait'] if attributes[:'max_wait']

      self.min_evictable_idle_time_millis = attributes[:'minEvictableIdleTimeMillis'] if attributes[:'minEvictableIdleTimeMillis']

      raise 'You cannot provide both :minEvictableIdleTimeMillis and :min_evictable_idle_time_millis' if attributes.key?(:'minEvictableIdleTimeMillis') && attributes.key?(:'min_evictable_idle_time_millis')

      self.min_evictable_idle_time_millis = attributes[:'min_evictable_idle_time_millis'] if attributes[:'min_evictable_idle_time_millis']

      self.min_idle = attributes[:'minIdle'] if attributes[:'minIdle']

      raise 'You cannot provide both :minIdle and :min_idle' if attributes.key?(:'minIdle') && attributes.key?(:'min_idle')

      self.min_idle = attributes[:'min_idle'] if attributes[:'min_idle']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        max_idle == other.max_idle &&
        max_objects == other.max_objects &&
        max_wait == other.max_wait &&
        min_evictable_idle_time_millis == other.min_evictable_idle_time_millis &&
        min_idle == other.min_idle
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [max_idle, max_objects, max_wait, min_evictable_idle_time_millis, min_idle].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
