# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: v1
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # This extension defines attributes used to manage account passwords within a Service Provider. The extension is typically applied to a User resource, but MAY be applied to other resources that use passwords.
  class IdentityDomains::Models::ExtensionPasswordStateUser
    # A DateTime that specifies the date and time when the current password was set
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: dateTime
    #  - uniqueness: none
    # @return [String]
    attr_accessor :last_successful_set_date

    # Indicates that the current password MAY NOT be changed and all other password expiry settings SHALL be ignored
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: boolean
    #  - uniqueness: none
    # @return [BOOLEAN]
    attr_accessor :cant_change

    # Indicates that the password expiry policy will not be applied for the current Resource
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: boolean
    #  - uniqueness: none
    # @return [BOOLEAN]
    attr_accessor :cant_expire

    # Indicates that the subject password value MUST change on next login. If not changed, typically the account is locked. The value may be set indirectly when the subject's current password expires or directly set by an administrator.
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: boolean
    #  - uniqueness: none
    # @return [BOOLEAN]
    attr_accessor :must_change

    # Indicates that the password has expired
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: boolean
    #  - uniqueness: none
    # @return [BOOLEAN]
    attr_accessor :expired

    # A DateTime that specifies the date and time when last successful password validation was set
    #
    # **Added In:** 2011192329
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: dateTime
    #  - uniqueness: none
    # @return [String]
    attr_accessor :last_successful_validation_date

    # A DateTime that specifies the date and time when last failed password validation was set
    #
    # **Added In:** 2011192329
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: dateTime
    #  - uniqueness: none
    # @return [String]
    attr_accessor :last_failed_validation_date

    # @return [OCI::IdentityDomains::Models::UserExtApplicablePasswordPolicy]
    attr_accessor :applicable_password_policy

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'last_successful_set_date': :'lastSuccessfulSetDate',
        'cant_change': :'cantChange',
        'cant_expire': :'cantExpire',
        'must_change': :'mustChange',
        'expired': :'expired',
        'last_successful_validation_date': :'lastSuccessfulValidationDate',
        'last_failed_validation_date': :'lastFailedValidationDate',
        'applicable_password_policy': :'applicablePasswordPolicy'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'last_successful_set_date': :'String',
        'cant_change': :'BOOLEAN',
        'cant_expire': :'BOOLEAN',
        'must_change': :'BOOLEAN',
        'expired': :'BOOLEAN',
        'last_successful_validation_date': :'String',
        'last_failed_validation_date': :'String',
        'applicable_password_policy': :'OCI::IdentityDomains::Models::UserExtApplicablePasswordPolicy'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :last_successful_set_date The value to assign to the {#last_successful_set_date} property
    # @option attributes [BOOLEAN] :cant_change The value to assign to the {#cant_change} property
    # @option attributes [BOOLEAN] :cant_expire The value to assign to the {#cant_expire} property
    # @option attributes [BOOLEAN] :must_change The value to assign to the {#must_change} property
    # @option attributes [BOOLEAN] :expired The value to assign to the {#expired} property
    # @option attributes [String] :last_successful_validation_date The value to assign to the {#last_successful_validation_date} property
    # @option attributes [String] :last_failed_validation_date The value to assign to the {#last_failed_validation_date} property
    # @option attributes [OCI::IdentityDomains::Models::UserExtApplicablePasswordPolicy] :applicable_password_policy The value to assign to the {#applicable_password_policy} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.last_successful_set_date = attributes[:'lastSuccessfulSetDate'] if attributes[:'lastSuccessfulSetDate']

      raise 'You cannot provide both :lastSuccessfulSetDate and :last_successful_set_date' if attributes.key?(:'lastSuccessfulSetDate') && attributes.key?(:'last_successful_set_date')

      self.last_successful_set_date = attributes[:'last_successful_set_date'] if attributes[:'last_successful_set_date']

      self.cant_change = attributes[:'cantChange'] unless attributes[:'cantChange'].nil?

      raise 'You cannot provide both :cantChange and :cant_change' if attributes.key?(:'cantChange') && attributes.key?(:'cant_change')

      self.cant_change = attributes[:'cant_change'] unless attributes[:'cant_change'].nil?

      self.cant_expire = attributes[:'cantExpire'] unless attributes[:'cantExpire'].nil?

      raise 'You cannot provide both :cantExpire and :cant_expire' if attributes.key?(:'cantExpire') && attributes.key?(:'cant_expire')

      self.cant_expire = attributes[:'cant_expire'] unless attributes[:'cant_expire'].nil?

      self.must_change = attributes[:'mustChange'] unless attributes[:'mustChange'].nil?

      raise 'You cannot provide both :mustChange and :must_change' if attributes.key?(:'mustChange') && attributes.key?(:'must_change')

      self.must_change = attributes[:'must_change'] unless attributes[:'must_change'].nil?

      self.expired = attributes[:'expired'] unless attributes[:'expired'].nil?

      self.last_successful_validation_date = attributes[:'lastSuccessfulValidationDate'] if attributes[:'lastSuccessfulValidationDate']

      raise 'You cannot provide both :lastSuccessfulValidationDate and :last_successful_validation_date' if attributes.key?(:'lastSuccessfulValidationDate') && attributes.key?(:'last_successful_validation_date')

      self.last_successful_validation_date = attributes[:'last_successful_validation_date'] if attributes[:'last_successful_validation_date']

      self.last_failed_validation_date = attributes[:'lastFailedValidationDate'] if attributes[:'lastFailedValidationDate']

      raise 'You cannot provide both :lastFailedValidationDate and :last_failed_validation_date' if attributes.key?(:'lastFailedValidationDate') && attributes.key?(:'last_failed_validation_date')

      self.last_failed_validation_date = attributes[:'last_failed_validation_date'] if attributes[:'last_failed_validation_date']

      self.applicable_password_policy = attributes[:'applicablePasswordPolicy'] if attributes[:'applicablePasswordPolicy']

      raise 'You cannot provide both :applicablePasswordPolicy and :applicable_password_policy' if attributes.key?(:'applicablePasswordPolicy') && attributes.key?(:'applicable_password_policy')

      self.applicable_password_policy = attributes[:'applicable_password_policy'] if attributes[:'applicable_password_policy']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        last_successful_set_date == other.last_successful_set_date &&
        cant_change == other.cant_change &&
        cant_expire == other.cant_expire &&
        must_change == other.must_change &&
        expired == other.expired &&
        last_successful_validation_date == other.last_successful_validation_date &&
        last_failed_validation_date == other.last_failed_validation_date &&
        applicable_password_policy == other.applicable_password_policy
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [last_successful_set_date, cant_change, cant_expire, must_change, expired, last_successful_validation_date, last_failed_validation_date, applicable_password_policy].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
