# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: v1
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # User Account
  class IdentityDomains::Models::User
    IDCS_PREVENTED_OPERATIONS_ENUM = [
      IDCS_PREVENTED_OPERATIONS_REPLACE = 'replace'.freeze,
      IDCS_PREVENTED_OPERATIONS_UPDATE = 'update'.freeze,
      IDCS_PREVENTED_OPERATIONS_DELETE = 'delete'.freeze,
      IDCS_PREVENTED_OPERATIONS_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    USER_TYPE_ENUM = [
      USER_TYPE_CONTRACTOR = 'Contractor'.freeze,
      USER_TYPE_EMPLOYEE = 'Employee'.freeze,
      USER_TYPE_INTERN = 'Intern'.freeze,
      USER_TYPE_TEMP = 'Temp'.freeze,
      USER_TYPE_EXTERNAL = 'External'.freeze,
      USER_TYPE_SERVICE = 'Service'.freeze,
      USER_TYPE_GENERIC = 'Generic'.freeze,
      USER_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # Unique identifier for the SCIM Resource as defined by the Service Provider. Each representation of the Resource MUST include a non-empty id value. This identifier MUST be unique across the Service Provider's entire set of Resources. It MUST be a stable, non-reassignable identifier that does not change when the same Resource is returned in subsequent requests. The value of the id attribute is always issued by the Service Provider and MUST never be specified by the Service Consumer. bulkId: is a reserved keyword and MUST NOT be used in the unique identifier.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: always
    #  - type: string
    #  - uniqueness: global
    # @return [String]
    attr_accessor :id

    # Unique OCI identifier for the SCIM Resource.
    #
    # **SCIM++ Properties:**
    #  - caseExact: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: immutable
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: global
    # @return [String]
    attr_accessor :ocid

    # **[Required]** REQUIRED. The schemas attribute is an array of Strings which allows introspection of the supported schema version for a SCIM representation as well any schema extensions supported by that representation. Each String value must be a unique URI. This specification defines URIs for User, Group, and a standard \\\"enterprise\\\" extension. All representations of SCIM schema MUST include a non-zero value array with value(s) of the URIs supported by that representation. Duplicate values MUST NOT be included. Value order is not specified and MUST not impact behavior.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: false
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: true
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [Array<String>]
    attr_accessor :schemas

    # @return [OCI::IdentityDomains::Models::Meta]
    attr_accessor :meta

    # @return [OCI::IdentityDomains::Models::IdcsCreatedBy]
    attr_accessor :idcs_created_by

    # @return [OCI::IdentityDomains::Models::IdcsLastModifiedBy]
    attr_accessor :idcs_last_modified_by

    # Each value of this attribute specifies an operation that only an internal client may perform on this particular resource.
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: true
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: string
    #  - uniqueness: none
    # @return [Array<String>]
    attr_reader :idcs_prevented_operations

    # A list of tags on this resource.
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [key, value]
    #  - idcsSearchable: true
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: request
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::Tags>]
    attr_accessor :tags

    # A boolean flag indicating this resource in the process of being deleted. Usually set to true when synchronous deletion of the resource would take too long.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: boolean
    #  - uniqueness: none
    # @return [BOOLEAN]
    attr_accessor :delete_in_progress

    # The release number when the resource was upgraded.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :idcs_last_upgraded_in_release

    # OCI Domain Id (ocid) in which the resource lives.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :domain_ocid

    # OCI Compartment Id (ocid) in which the resource lives.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :compartment_ocid

    # OCI Tenant Id (ocid) in which the resource lives.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :tenancy_ocid

    # An identifier for the Resource as defined by the Service Consumer. The externalId may simplify identification of the Resource between Service Consumer and Service Provider by allowing the Consumer to refer to the Resource with its own identifier, obviating the need to store a local mapping between the local identifier of the Resource and the identifier used by the Service Provider. Each Resource MAY include a non-empty externalId value. The value of the externalId attribute is always issued by the Service Consumer and can never be specified by the Service Provider. The Service Provider MUST always interpret the externalId as scoped to the Service Consumer's tenant.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:External Id]]
    #  - idcsPii: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :external_id

    # **[Required]** User name
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: User ID
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:User Name, deprecatedColumnHeaderName:User ID]]
    #  - idcsPii: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: true
    #  - returned: always
    #  - type: string
    #  - uniqueness: global
    # @return [String]
    attr_accessor :user_name

    # Description of the user
    #
    # **Added In:** 2012271618
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsPii: true
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :description

    # Display name
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: Display Name
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Display Name]]
    #  - idcsPii: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :display_name

    # Nick name
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: Nick Name
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Nick Name]]
    #  - idcsPii: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :nick_name

    # A fully-qualified URL to a page representing the User's online profile
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: Profile URL
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Profile Url]]
    #  - idcsPii: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: reference
    #  - uniqueness: none
    # @return [String]
    attr_accessor :profile_url

    # Title
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: Title
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Title]]
    #  - idcsPii: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :title

    # Used to identify the organization-to-user relationship
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: User Type
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:User Type]]
    #  - idcsPii: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_reader :user_type

    # Used to indicate the User's default location for purposes of localizing items such as currency, date and time format, numerical representations, and so on.
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: Locale
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Locale]]
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :locale

    # User's preferred written or spoken language used for localized user interfaces
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: Preferred Language
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Preferred Language]]
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :preferred_language

    # User's timezone
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCanonicalValueSourceFilter: attrName eq \"timezones\" and attrValues.value eq \"$(timezone)\"
    #  - idcsCanonicalValueSourceResourceType: AllowedValue
    #  - idcsCsvAttributeName: TimeZone
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Time Zone, deprecatedColumnHeaderName:TimeZone]]
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :timezone

    # User status
    #
    # **SCIM++ Properties:**
    #  - caseExact: false
    #  - idcsCsvAttributeName: Active
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Active]]
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: boolean
    #  - uniqueness: none
    # @return [BOOLEAN]
    attr_accessor :active

    # Password attribute. Max length for password is controlled via Password Policy.
    #
    # **SCIM++ Properties:**
    #  - idcsCsvAttributeName: Password
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Password]]
    #  - idcsPii: true
    #  - idcsSearchable: false
    #  - idcsSensitive: hash
    #  - multiValued: false
    #  - mutability: writeOnly
    #  - required: false
    #  - returned: never
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :password

    # @return [OCI::IdentityDomains::Models::UserName]
    attr_accessor :name

    # A complex attribute representing emails
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [value, type]
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Work Email, mapsTo:emails[work].value], [columnHeaderName:Home Email, mapsTo:emails[home].value], [columnHeaderName:Primary Email Type, mapsTo:emails[$(type)].primary], [columnHeaderName:Other Email, mapsTo:emails[other].value], [columnHeaderName:Recovery Email, mapsTo:emails[recovery].value], [columnHeaderName:Work Email Verified, mapsTo:emails[work].verified], [columnHeaderName:Home Email Verified, mapsTo:emails[home].verified], [columnHeaderName:Other Email Verified, mapsTo:emails[other].verified], [columnHeaderName:Recovery Email Verified, mapsTo:emails[recovery].verified]]
    #  - idcsPii: true
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::UserEmails>]
    attr_accessor :emails

    # Phone numbers
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [value, type]
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Work Phone, mapsTo:phoneNumbers[work].value], [columnHeaderName:Mobile No, mapsTo:phoneNumbers[mobile].value], [columnHeaderName:Home Phone, mapsTo:phoneNumbers[home].value], [columnHeaderName:Fax, mapsTo:phoneNumbers[fax].value], [columnHeaderName:Pager, mapsTo:phoneNumbers[pager].value], [columnHeaderName:Other Phone, mapsTo:phoneNumbers[other].value], [columnHeaderName:Recovery Phone, mapsTo:phoneNumbers[recovery].value], [columnHeaderName:Primary Phone Type, mapsTo:phoneNumbers[$(type)].primary]]
    #  - idcsPii: true
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::UserPhoneNumbers>]
    attr_accessor :phone_numbers

    # User's instant messaging addresses
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [value, type]
    #  - idcsPii: true
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::UserIms>]
    attr_accessor :ims

    # URLs of photos for the User
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [value, type]
    #  - idcsPii: true
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::UserPhotos>]
    attr_accessor :photos

    # A physical mailing address for this User, as described in (address Element). Canonical Type Values of work, home, and other. The value attribute is a complex type with the following sub-attributes.
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [type]
    #  - idcsCsvAttributeNameMappings: [[columnHeaderName:Work Address Street, deprecatedColumnHeaderName:Work Street Address, mapsTo:addresses[work].streetAddress], [columnHeaderName:Work Address Locality, deprecatedColumnHeaderName:Work City, mapsTo:addresses[work].locality], [columnHeaderName:Work Address Region, deprecatedColumnHeaderName:Work State, mapsTo:addresses[work].region], [columnHeaderName:Work Address Postal Code, deprecatedColumnHeaderName:Work Postal Code, mapsTo:addresses[work].postalCode], [columnHeaderName:Work Address Country, deprecatedColumnHeaderName:Work Country, mapsTo:addresses[work].country], [columnHeaderName:Work Address Formatted, mapsTo:addresses[work].formatted], [columnHeaderName:Home Address Formatted, mapsTo:addresses[home].formatted], [columnHeaderName:Other Address Formatted, mapsTo:addresses[other].formatted], [columnHeaderName:Home Address Street, mapsTo:addresses[home].streetAddress], [columnHeaderName:Other Address Street, mapsTo:addresses[other].streetAddress], [columnHeaderName:Home Address Locality, mapsTo:addresses[home].locality], [columnHeaderName:Other Address Locality, mapsTo:addresses[other].locality], [columnHeaderName:Home Address Region, mapsTo:addresses[home].region], [columnHeaderName:Other Address Region, mapsTo:addresses[other].region], [columnHeaderName:Home Address Country, mapsTo:addresses[home].country], [columnHeaderName:Other Address Country, mapsTo:addresses[other].country], [columnHeaderName:Home Address Postal Code, mapsTo:addresses[home].postalCode], [columnHeaderName:Other Address Postal Code, mapsTo:addresses[other].postalCode], [columnHeaderName:Primary Address Type, mapsTo:addresses[$(type)].primary]]
    #  - idcsPii: true
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::Addresses>]
    attr_accessor :addresses

    # A list of groups that the user belongs to, either thorough direct membership, nested groups, or dynamically calculated
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [value]
    #  - idcsSearchable: true
    #  - multiValued: true
    #  - mutability: readOnly
    #  - required: false
    #  - returned: request
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::UserGroups>]
    attr_accessor :groups

    # A list of entitlements for the User that represent a thing the User has.
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [value, type]
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::UserEntitlements>]
    attr_accessor :entitlements

    # A list of roles for the User that collectively represent who the User is; e.g., 'Student', 'Faculty'.
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [value, type]
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::UserRoles>]
    attr_accessor :roles

    # A list of certificates issued to the User.
    #
    # **SCIM++ Properties:**
    #  - idcsCompositeKey: [value]
    #  - multiValued: true
    #  - mutability: readWrite
    #  - required: false
    #  - returned: default
    #  - type: complex
    #  - uniqueness: none
    # @return [Array<OCI::IdentityDomains::Models::UserX509Certificates>]
    attr_accessor :x509_certificates

    # @return [OCI::IdentityDomains::Models::ExtensionEnterprise20User]
    attr_accessor :urn_ietf_params_scim_schemas_extension_enterprise2_0_user

    # @return [OCI::IdentityDomains::Models::ExtensionUserUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user

    # @return [OCI::IdentityDomains::Models::ExtensionPasswordStateUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user

    # @return [OCI::IdentityDomains::Models::ExtensionUserStateUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user

    # @return [OCI::IdentityDomains::Models::ExtensionPosixUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user

    # @return [OCI::IdentityDomains::Models::ExtensionKerberosUserUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user

    # @return [OCI::IdentityDomains::Models::ExtensionMfaUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user

    # @return [OCI::IdentityDomains::Models::ExtensionAdaptiveUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user

    # @return [OCI::IdentityDomains::Models::ExtensionSffUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user

    # @return [OCI::IdentityDomains::Models::ExtensionSecurityQuestionsUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user

    # @return [OCI::IdentityDomains::Models::ExtensionSelfRegistrationUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user

    # @return [OCI::IdentityDomains::Models::ExtensionSocialAccountUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user

    # @return [OCI::IdentityDomains::Models::ExtensionDbUserUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user

    # @return [OCI::IdentityDomains::Models::ExtensionTermsOfUseUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user

    # @return [OCI::IdentityDomains::Models::ExtensionPasswordlessUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user

    # @return [OCI::IdentityDomains::Models::ExtensionOCITags]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags

    # @return [OCI::IdentityDomains::Models::ExtensionUserCredentialsUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user

    # @return [OCI::IdentityDomains::Models::ExtensionCapabilitiesUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user

    # @return [OCI::IdentityDomains::Models::ExtensionDbCredentialsUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user

    # @return [OCI::IdentityDomains::Models::ExtensionSelfChangeUser]
    attr_accessor :urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'ocid': :'ocid',
        'schemas': :'schemas',
        'meta': :'meta',
        'idcs_created_by': :'idcsCreatedBy',
        'idcs_last_modified_by': :'idcsLastModifiedBy',
        'idcs_prevented_operations': :'idcsPreventedOperations',
        'tags': :'tags',
        'delete_in_progress': :'deleteInProgress',
        'idcs_last_upgraded_in_release': :'idcsLastUpgradedInRelease',
        'domain_ocid': :'domainOcid',
        'compartment_ocid': :'compartmentOcid',
        'tenancy_ocid': :'tenancyOcid',
        'external_id': :'externalId',
        'user_name': :'userName',
        'description': :'description',
        'display_name': :'displayName',
        'nick_name': :'nickName',
        'profile_url': :'profileUrl',
        'title': :'title',
        'user_type': :'userType',
        'locale': :'locale',
        'preferred_language': :'preferredLanguage',
        'timezone': :'timezone',
        'active': :'active',
        'password': :'password',
        'name': :'name',
        'emails': :'emails',
        'phone_numbers': :'phoneNumbers',
        'ims': :'ims',
        'photos': :'photos',
        'addresses': :'addresses',
        'groups': :'groups',
        'entitlements': :'entitlements',
        'roles': :'roles',
        'x509_certificates': :'x509Certificates',
        'urn_ietf_params_scim_schemas_extension_enterprise2_0_user': :'urn:ietf:params:scim:schemas:extension:enterprise:2.0:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:user:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordState:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:userState:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:posix:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:kerberosUser:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:mfa:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:adaptive:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:sff:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:securityQuestions:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:selfRegistration:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:socialAccount:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:dbUser:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:termsOfUse:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordless:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:OCITags',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:userCredentials:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:capabilities:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:dbCredentials:User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user': :'urn:ietf:params:scim:schemas:oracle:idcs:extension:selfChange:User'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'ocid': :'String',
        'schemas': :'Array<String>',
        'meta': :'OCI::IdentityDomains::Models::Meta',
        'idcs_created_by': :'OCI::IdentityDomains::Models::IdcsCreatedBy',
        'idcs_last_modified_by': :'OCI::IdentityDomains::Models::IdcsLastModifiedBy',
        'idcs_prevented_operations': :'Array<String>',
        'tags': :'Array<OCI::IdentityDomains::Models::Tags>',
        'delete_in_progress': :'BOOLEAN',
        'idcs_last_upgraded_in_release': :'String',
        'domain_ocid': :'String',
        'compartment_ocid': :'String',
        'tenancy_ocid': :'String',
        'external_id': :'String',
        'user_name': :'String',
        'description': :'String',
        'display_name': :'String',
        'nick_name': :'String',
        'profile_url': :'String',
        'title': :'String',
        'user_type': :'String',
        'locale': :'String',
        'preferred_language': :'String',
        'timezone': :'String',
        'active': :'BOOLEAN',
        'password': :'String',
        'name': :'OCI::IdentityDomains::Models::UserName',
        'emails': :'Array<OCI::IdentityDomains::Models::UserEmails>',
        'phone_numbers': :'Array<OCI::IdentityDomains::Models::UserPhoneNumbers>',
        'ims': :'Array<OCI::IdentityDomains::Models::UserIms>',
        'photos': :'Array<OCI::IdentityDomains::Models::UserPhotos>',
        'addresses': :'Array<OCI::IdentityDomains::Models::Addresses>',
        'groups': :'Array<OCI::IdentityDomains::Models::UserGroups>',
        'entitlements': :'Array<OCI::IdentityDomains::Models::UserEntitlements>',
        'roles': :'Array<OCI::IdentityDomains::Models::UserRoles>',
        'x509_certificates': :'Array<OCI::IdentityDomains::Models::UserX509Certificates>',
        'urn_ietf_params_scim_schemas_extension_enterprise2_0_user': :'OCI::IdentityDomains::Models::ExtensionEnterprise20User',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user': :'OCI::IdentityDomains::Models::ExtensionUserUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user': :'OCI::IdentityDomains::Models::ExtensionPasswordStateUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user': :'OCI::IdentityDomains::Models::ExtensionUserStateUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user': :'OCI::IdentityDomains::Models::ExtensionPosixUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user': :'OCI::IdentityDomains::Models::ExtensionKerberosUserUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user': :'OCI::IdentityDomains::Models::ExtensionMfaUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user': :'OCI::IdentityDomains::Models::ExtensionAdaptiveUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user': :'OCI::IdentityDomains::Models::ExtensionSffUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user': :'OCI::IdentityDomains::Models::ExtensionSecurityQuestionsUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user': :'OCI::IdentityDomains::Models::ExtensionSelfRegistrationUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user': :'OCI::IdentityDomains::Models::ExtensionSocialAccountUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user': :'OCI::IdentityDomains::Models::ExtensionDbUserUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user': :'OCI::IdentityDomains::Models::ExtensionTermsOfUseUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user': :'OCI::IdentityDomains::Models::ExtensionPasswordlessUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags': :'OCI::IdentityDomains::Models::ExtensionOCITags',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user': :'OCI::IdentityDomains::Models::ExtensionUserCredentialsUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user': :'OCI::IdentityDomains::Models::ExtensionCapabilitiesUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user': :'OCI::IdentityDomains::Models::ExtensionDbCredentialsUser',
        'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user': :'OCI::IdentityDomains::Models::ExtensionSelfChangeUser'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :ocid The value to assign to the {#ocid} property
    # @option attributes [Array<String>] :schemas The value to assign to the {#schemas} property
    # @option attributes [OCI::IdentityDomains::Models::Meta] :meta The value to assign to the {#meta} property
    # @option attributes [OCI::IdentityDomains::Models::IdcsCreatedBy] :idcs_created_by The value to assign to the {#idcs_created_by} property
    # @option attributes [OCI::IdentityDomains::Models::IdcsLastModifiedBy] :idcs_last_modified_by The value to assign to the {#idcs_last_modified_by} property
    # @option attributes [Array<String>] :idcs_prevented_operations The value to assign to the {#idcs_prevented_operations} property
    # @option attributes [Array<OCI::IdentityDomains::Models::Tags>] :tags The value to assign to the {#tags} property
    # @option attributes [BOOLEAN] :delete_in_progress The value to assign to the {#delete_in_progress} property
    # @option attributes [String] :idcs_last_upgraded_in_release The value to assign to the {#idcs_last_upgraded_in_release} property
    # @option attributes [String] :domain_ocid The value to assign to the {#domain_ocid} property
    # @option attributes [String] :compartment_ocid The value to assign to the {#compartment_ocid} property
    # @option attributes [String] :tenancy_ocid The value to assign to the {#tenancy_ocid} property
    # @option attributes [String] :external_id The value to assign to the {#external_id} property
    # @option attributes [String] :user_name The value to assign to the {#user_name} property
    # @option attributes [String] :description The value to assign to the {#description} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :nick_name The value to assign to the {#nick_name} property
    # @option attributes [String] :profile_url The value to assign to the {#profile_url} property
    # @option attributes [String] :title The value to assign to the {#title} property
    # @option attributes [String] :user_type The value to assign to the {#user_type} property
    # @option attributes [String] :locale The value to assign to the {#locale} property
    # @option attributes [String] :preferred_language The value to assign to the {#preferred_language} property
    # @option attributes [String] :timezone The value to assign to the {#timezone} property
    # @option attributes [BOOLEAN] :active The value to assign to the {#active} property
    # @option attributes [String] :password The value to assign to the {#password} property
    # @option attributes [OCI::IdentityDomains::Models::UserName] :name The value to assign to the {#name} property
    # @option attributes [Array<OCI::IdentityDomains::Models::UserEmails>] :emails The value to assign to the {#emails} property
    # @option attributes [Array<OCI::IdentityDomains::Models::UserPhoneNumbers>] :phone_numbers The value to assign to the {#phone_numbers} property
    # @option attributes [Array<OCI::IdentityDomains::Models::UserIms>] :ims The value to assign to the {#ims} property
    # @option attributes [Array<OCI::IdentityDomains::Models::UserPhotos>] :photos The value to assign to the {#photos} property
    # @option attributes [Array<OCI::IdentityDomains::Models::Addresses>] :addresses The value to assign to the {#addresses} property
    # @option attributes [Array<OCI::IdentityDomains::Models::UserGroups>] :groups The value to assign to the {#groups} property
    # @option attributes [Array<OCI::IdentityDomains::Models::UserEntitlements>] :entitlements The value to assign to the {#entitlements} property
    # @option attributes [Array<OCI::IdentityDomains::Models::UserRoles>] :roles The value to assign to the {#roles} property
    # @option attributes [Array<OCI::IdentityDomains::Models::UserX509Certificates>] :x509_certificates The value to assign to the {#x509_certificates} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionEnterprise20User] :urn_ietf_params_scim_schemas_extension_enterprise2_0_user The value to assign to the {#urn_ietf_params_scim_schemas_extension_enterprise2_0_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionUserUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionPasswordStateUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionUserStateUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionPosixUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionKerberosUserUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionMfaUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionAdaptiveUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionSffUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionSecurityQuestionsUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionSelfRegistrationUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionSocialAccountUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionDbUserUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionTermsOfUseUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionPasswordlessUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionOCITags] :urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionUserCredentialsUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionCapabilitiesUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionDbCredentialsUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user} property
    # @option attributes [OCI::IdentityDomains::Models::ExtensionSelfChangeUser] :urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user The value to assign to the {#urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.ocid = attributes[:'ocid'] if attributes[:'ocid']

      self.schemas = attributes[:'schemas'] if attributes[:'schemas']

      self.meta = attributes[:'meta'] if attributes[:'meta']

      self.idcs_created_by = attributes[:'idcsCreatedBy'] if attributes[:'idcsCreatedBy']

      raise 'You cannot provide both :idcsCreatedBy and :idcs_created_by' if attributes.key?(:'idcsCreatedBy') && attributes.key?(:'idcs_created_by')

      self.idcs_created_by = attributes[:'idcs_created_by'] if attributes[:'idcs_created_by']

      self.idcs_last_modified_by = attributes[:'idcsLastModifiedBy'] if attributes[:'idcsLastModifiedBy']

      raise 'You cannot provide both :idcsLastModifiedBy and :idcs_last_modified_by' if attributes.key?(:'idcsLastModifiedBy') && attributes.key?(:'idcs_last_modified_by')

      self.idcs_last_modified_by = attributes[:'idcs_last_modified_by'] if attributes[:'idcs_last_modified_by']

      self.idcs_prevented_operations = attributes[:'idcsPreventedOperations'] if attributes[:'idcsPreventedOperations']

      raise 'You cannot provide both :idcsPreventedOperations and :idcs_prevented_operations' if attributes.key?(:'idcsPreventedOperations') && attributes.key?(:'idcs_prevented_operations')

      self.idcs_prevented_operations = attributes[:'idcs_prevented_operations'] if attributes[:'idcs_prevented_operations']

      self.tags = attributes[:'tags'] if attributes[:'tags']

      self.delete_in_progress = attributes[:'deleteInProgress'] unless attributes[:'deleteInProgress'].nil?

      raise 'You cannot provide both :deleteInProgress and :delete_in_progress' if attributes.key?(:'deleteInProgress') && attributes.key?(:'delete_in_progress')

      self.delete_in_progress = attributes[:'delete_in_progress'] unless attributes[:'delete_in_progress'].nil?

      self.idcs_last_upgraded_in_release = attributes[:'idcsLastUpgradedInRelease'] if attributes[:'idcsLastUpgradedInRelease']

      raise 'You cannot provide both :idcsLastUpgradedInRelease and :idcs_last_upgraded_in_release' if attributes.key?(:'idcsLastUpgradedInRelease') && attributes.key?(:'idcs_last_upgraded_in_release')

      self.idcs_last_upgraded_in_release = attributes[:'idcs_last_upgraded_in_release'] if attributes[:'idcs_last_upgraded_in_release']

      self.domain_ocid = attributes[:'domainOcid'] if attributes[:'domainOcid']

      raise 'You cannot provide both :domainOcid and :domain_ocid' if attributes.key?(:'domainOcid') && attributes.key?(:'domain_ocid')

      self.domain_ocid = attributes[:'domain_ocid'] if attributes[:'domain_ocid']

      self.compartment_ocid = attributes[:'compartmentOcid'] if attributes[:'compartmentOcid']

      raise 'You cannot provide both :compartmentOcid and :compartment_ocid' if attributes.key?(:'compartmentOcid') && attributes.key?(:'compartment_ocid')

      self.compartment_ocid = attributes[:'compartment_ocid'] if attributes[:'compartment_ocid']

      self.tenancy_ocid = attributes[:'tenancyOcid'] if attributes[:'tenancyOcid']

      raise 'You cannot provide both :tenancyOcid and :tenancy_ocid' if attributes.key?(:'tenancyOcid') && attributes.key?(:'tenancy_ocid')

      self.tenancy_ocid = attributes[:'tenancy_ocid'] if attributes[:'tenancy_ocid']

      self.external_id = attributes[:'externalId'] if attributes[:'externalId']

      raise 'You cannot provide both :externalId and :external_id' if attributes.key?(:'externalId') && attributes.key?(:'external_id')

      self.external_id = attributes[:'external_id'] if attributes[:'external_id']

      self.user_name = attributes[:'userName'] if attributes[:'userName']

      raise 'You cannot provide both :userName and :user_name' if attributes.key?(:'userName') && attributes.key?(:'user_name')

      self.user_name = attributes[:'user_name'] if attributes[:'user_name']

      self.description = attributes[:'description'] if attributes[:'description']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.nick_name = attributes[:'nickName'] if attributes[:'nickName']

      raise 'You cannot provide both :nickName and :nick_name' if attributes.key?(:'nickName') && attributes.key?(:'nick_name')

      self.nick_name = attributes[:'nick_name'] if attributes[:'nick_name']

      self.profile_url = attributes[:'profileUrl'] if attributes[:'profileUrl']

      raise 'You cannot provide both :profileUrl and :profile_url' if attributes.key?(:'profileUrl') && attributes.key?(:'profile_url')

      self.profile_url = attributes[:'profile_url'] if attributes[:'profile_url']

      self.title = attributes[:'title'] if attributes[:'title']

      self.user_type = attributes[:'userType'] if attributes[:'userType']

      raise 'You cannot provide both :userType and :user_type' if attributes.key?(:'userType') && attributes.key?(:'user_type')

      self.user_type = attributes[:'user_type'] if attributes[:'user_type']

      self.locale = attributes[:'locale'] if attributes[:'locale']

      self.preferred_language = attributes[:'preferredLanguage'] if attributes[:'preferredLanguage']

      raise 'You cannot provide both :preferredLanguage and :preferred_language' if attributes.key?(:'preferredLanguage') && attributes.key?(:'preferred_language')

      self.preferred_language = attributes[:'preferred_language'] if attributes[:'preferred_language']

      self.timezone = attributes[:'timezone'] if attributes[:'timezone']

      self.active = attributes[:'active'] unless attributes[:'active'].nil?

      self.password = attributes[:'password'] if attributes[:'password']

      self.name = attributes[:'name'] if attributes[:'name']

      self.emails = attributes[:'emails'] if attributes[:'emails']

      self.phone_numbers = attributes[:'phoneNumbers'] if attributes[:'phoneNumbers']

      raise 'You cannot provide both :phoneNumbers and :phone_numbers' if attributes.key?(:'phoneNumbers') && attributes.key?(:'phone_numbers')

      self.phone_numbers = attributes[:'phone_numbers'] if attributes[:'phone_numbers']

      self.ims = attributes[:'ims'] if attributes[:'ims']

      self.photos = attributes[:'photos'] if attributes[:'photos']

      self.addresses = attributes[:'addresses'] if attributes[:'addresses']

      self.groups = attributes[:'groups'] if attributes[:'groups']

      self.entitlements = attributes[:'entitlements'] if attributes[:'entitlements']

      self.roles = attributes[:'roles'] if attributes[:'roles']

      self.x509_certificates = attributes[:'x509Certificates'] if attributes[:'x509Certificates']

      raise 'You cannot provide both :x509Certificates and :x509_certificates' if attributes.key?(:'x509Certificates') && attributes.key?(:'x509_certificates')

      self.x509_certificates = attributes[:'x509_certificates'] if attributes[:'x509_certificates']

      self.urn_ietf_params_scim_schemas_extension_enterprise2_0_user = attributes[:'urn:ietf:params:scim:schemas:extension:enterprise:2.0:User'] if attributes[:'urn:ietf:params:scim:schemas:extension:enterprise:2.0:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:extension:enterprise:2.0:User and :urn_ietf_params_scim_schemas_extension_enterprise2_0_user' if attributes.key?(:'urn:ietf:params:scim:schemas:extension:enterprise:2.0:User') && attributes.key?(:'urn_ietf_params_scim_schemas_extension_enterprise2_0_user')

      self.urn_ietf_params_scim_schemas_extension_enterprise2_0_user = attributes[:'urn_ietf_params_scim_schemas_extension_enterprise2_0_user'] if attributes[:'urn_ietf_params_scim_schemas_extension_enterprise2_0_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:user:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:user:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:user:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:user:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordState:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordState:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordState:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordState:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:userState:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:userState:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:userState:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:userState:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:posix:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:posix:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:posix:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:posix:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:kerberosUser:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:kerberosUser:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:kerberosUser:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:kerberosUser:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:mfa:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:mfa:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:mfa:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:mfa:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:adaptive:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:adaptive:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:adaptive:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:adaptive:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:sff:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:sff:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:sff:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:sff:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:securityQuestions:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:securityQuestions:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:securityQuestions:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:securityQuestions:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:selfRegistration:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:selfRegistration:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:selfRegistration:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:selfRegistration:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:socialAccount:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:socialAccount:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:socialAccount:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:socialAccount:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:dbUser:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:dbUser:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:dbUser:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:dbUser:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:termsOfUse:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:termsOfUse:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:termsOfUse:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:termsOfUse:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordless:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordless:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordless:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:passwordless:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:OCITags'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:OCITags']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:OCITags and :urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:OCITags') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:userCredentials:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:userCredentials:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:userCredentials:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:userCredentials:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:capabilities:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:capabilities:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:capabilities:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:capabilities:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:dbCredentials:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:dbCredentials:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:dbCredentials:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:dbCredentials:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user']

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user = attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:selfChange:User'] if attributes[:'urn:ietf:params:scim:schemas:oracle:idcs:extension:selfChange:User']

      raise 'You cannot provide both :urn:ietf:params:scim:schemas:oracle:idcs:extension:selfChange:User and :urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user' if attributes.key?(:'urn:ietf:params:scim:schemas:oracle:idcs:extension:selfChange:User') && attributes.key?(:'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user')

      self.urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user = attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user'] if attributes[:'urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] idcs_prevented_operations Object to be assigned
    def idcs_prevented_operations=(idcs_prevented_operations)
      # rubocop:disable Style/ConditionalAssignment
      if idcs_prevented_operations.nil?
        @idcs_prevented_operations = nil
      else
        @idcs_prevented_operations =
          idcs_prevented_operations.collect do |item|
            if IDCS_PREVENTED_OPERATIONS_ENUM.include?(item)
              item
            else
              OCI.logger.debug("Unknown value for 'idcs_prevented_operations' [#{item}]. Mapping to 'IDCS_PREVENTED_OPERATIONS_UNKNOWN_ENUM_VALUE'") if OCI.logger
              IDCS_PREVENTED_OPERATIONS_UNKNOWN_ENUM_VALUE
            end
          end
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] user_type Object to be assigned
    def user_type=(user_type)
      # rubocop:disable Style/ConditionalAssignment
      if user_type && !USER_TYPE_ENUM.include?(user_type)
        OCI.logger.debug("Unknown value for 'user_type' [" + user_type + "]. Mapping to 'USER_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @user_type = USER_TYPE_UNKNOWN_ENUM_VALUE
      else
        @user_type = user_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        ocid == other.ocid &&
        schemas == other.schemas &&
        meta == other.meta &&
        idcs_created_by == other.idcs_created_by &&
        idcs_last_modified_by == other.idcs_last_modified_by &&
        idcs_prevented_operations == other.idcs_prevented_operations &&
        tags == other.tags &&
        delete_in_progress == other.delete_in_progress &&
        idcs_last_upgraded_in_release == other.idcs_last_upgraded_in_release &&
        domain_ocid == other.domain_ocid &&
        compartment_ocid == other.compartment_ocid &&
        tenancy_ocid == other.tenancy_ocid &&
        external_id == other.external_id &&
        user_name == other.user_name &&
        description == other.description &&
        display_name == other.display_name &&
        nick_name == other.nick_name &&
        profile_url == other.profile_url &&
        title == other.title &&
        user_type == other.user_type &&
        locale == other.locale &&
        preferred_language == other.preferred_language &&
        timezone == other.timezone &&
        active == other.active &&
        password == other.password &&
        name == other.name &&
        emails == other.emails &&
        phone_numbers == other.phone_numbers &&
        ims == other.ims &&
        photos == other.photos &&
        addresses == other.addresses &&
        groups == other.groups &&
        entitlements == other.entitlements &&
        roles == other.roles &&
        x509_certificates == other.x509_certificates &&
        urn_ietf_params_scim_schemas_extension_enterprise2_0_user == other.urn_ietf_params_scim_schemas_extension_enterprise2_0_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user &&
        urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user == other.urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, ocid, schemas, meta, idcs_created_by, idcs_last_modified_by, idcs_prevented_operations, tags, delete_in_progress, idcs_last_upgraded_in_release, domain_ocid, compartment_ocid, tenancy_ocid, external_id, user_name, description, display_name, nick_name, profile_url, title, user_type, locale, preferred_language, timezone, active, password, name, emails, phone_numbers, ims, photos, addresses, groups, entitlements, roles, x509_certificates, urn_ietf_params_scim_schemas_extension_enterprise2_0_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_user_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_password_state_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_user_state_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_posix_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_kerberos_user_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_mfa_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_adaptive_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_sff_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_security_questions_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_self_registration_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_social_account_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_db_user_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_terms_of_use_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_passwordless_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_oci_tags, urn_ietf_params_scim_schemas_oracle_idcs_extension_user_credentials_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_capabilities_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_db_credentials_user, urn_ietf_params_scim_schemas_oracle_idcs_extension_self_change_user].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
