# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: v1
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # A list of devices enrolled by the user.
  #
  # **Added In:** 18.3.6
  #
  # **SCIM++ Properties:**
  #  - idcsCompositeKey: [value]
  #  - multiValued: true
  #  - mutability: readOnly
  #  - required: false
  #  - returned: request
  #  - type: complex
  #  - uniqueness: none
  class IdentityDomains::Models::UserExtDevices
    # **[Required]** The user's device identifier.
    #
    # **Added In:** 18.3.6
    #
    # **SCIM++ Properties:**
    #  - caseExact: true
    #  - idcsSearchable: true
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: true
    #  - returned: always
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :value

    # The URI of the corresponding Device resource which belongs to user.
    #
    # **Added In:** 18.3.6
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: reference
    #  - uniqueness: none
    # @return [String]
    attr_accessor :ref

    # A human readable name, primarily used for display purposes. READ-ONLY.
    #
    # **Added In:** 18.3.6
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :display

    # The device's status.
    #
    # **Added In:** 18.4.2
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :status

    # The last sync time for device.
    #
    # **Added In:** 18.4.2
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: dateTime
    #  - uniqueness: none
    # @return [String]
    attr_accessor :last_sync_time

    # The device authentication factor type.
    #
    # **Added In:** 18.4.2
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :factor_type

    # The device authentication factor status.
    #
    # **Added In:** 18.4.2
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :factor_status

    # The authentication method.
    #
    # **Added In:** 2009232244
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :authentication_method

    # The third-party factor vendor name.
    #
    # **Added In:** 2009232244
    #
    # **SCIM++ Properties:**
    #  - idcsSearchable: false
    #  - multiValued: false
    #  - mutability: readOnly
    #  - required: false
    #  - returned: default
    #  - type: string
    #  - uniqueness: none
    # @return [String]
    attr_accessor :third_party_vendor_name

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'value': :'value',
        'ref': :'$ref',
        'display': :'display',
        'status': :'status',
        'last_sync_time': :'lastSyncTime',
        'factor_type': :'factorType',
        'factor_status': :'factorStatus',
        'authentication_method': :'authenticationMethod',
        'third_party_vendor_name': :'thirdPartyVendorName'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'value': :'String',
        'ref': :'String',
        'display': :'String',
        'status': :'String',
        'last_sync_time': :'String',
        'factor_type': :'String',
        'factor_status': :'String',
        'authentication_method': :'String',
        'third_party_vendor_name': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :value The value to assign to the {#value} property
    # @option attributes [String] :ref The value to assign to the {#ref} property
    # @option attributes [String] :display The value to assign to the {#display} property
    # @option attributes [String] :status The value to assign to the {#status} property
    # @option attributes [String] :last_sync_time The value to assign to the {#last_sync_time} property
    # @option attributes [String] :factor_type The value to assign to the {#factor_type} property
    # @option attributes [String] :factor_status The value to assign to the {#factor_status} property
    # @option attributes [String] :authentication_method The value to assign to the {#authentication_method} property
    # @option attributes [String] :third_party_vendor_name The value to assign to the {#third_party_vendor_name} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.value = attributes[:'value'] if attributes[:'value']

      self.ref = attributes[:'$ref'] if attributes[:'$ref']

      self.display = attributes[:'display'] if attributes[:'display']

      self.status = attributes[:'status'] if attributes[:'status']

      self.last_sync_time = attributes[:'lastSyncTime'] if attributes[:'lastSyncTime']

      raise 'You cannot provide both :lastSyncTime and :last_sync_time' if attributes.key?(:'lastSyncTime') && attributes.key?(:'last_sync_time')

      self.last_sync_time = attributes[:'last_sync_time'] if attributes[:'last_sync_time']

      self.factor_type = attributes[:'factorType'] if attributes[:'factorType']

      raise 'You cannot provide both :factorType and :factor_type' if attributes.key?(:'factorType') && attributes.key?(:'factor_type')

      self.factor_type = attributes[:'factor_type'] if attributes[:'factor_type']

      self.factor_status = attributes[:'factorStatus'] if attributes[:'factorStatus']

      raise 'You cannot provide both :factorStatus and :factor_status' if attributes.key?(:'factorStatus') && attributes.key?(:'factor_status')

      self.factor_status = attributes[:'factor_status'] if attributes[:'factor_status']

      self.authentication_method = attributes[:'authenticationMethod'] if attributes[:'authenticationMethod']

      raise 'You cannot provide both :authenticationMethod and :authentication_method' if attributes.key?(:'authenticationMethod') && attributes.key?(:'authentication_method')

      self.authentication_method = attributes[:'authentication_method'] if attributes[:'authentication_method']

      self.third_party_vendor_name = attributes[:'thirdPartyVendorName'] if attributes[:'thirdPartyVendorName']

      raise 'You cannot provide both :thirdPartyVendorName and :third_party_vendor_name' if attributes.key?(:'thirdPartyVendorName') && attributes.key?(:'third_party_vendor_name')

      self.third_party_vendor_name = attributes[:'third_party_vendor_name'] if attributes[:'third_party_vendor_name']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        value == other.value &&
        ref == other.ref &&
        display == other.display &&
        status == other.status &&
        last_sync_time == other.last_sync_time &&
        factor_type == other.factor_type &&
        factor_status == other.factor_status &&
        authentication_method == other.authentication_method &&
        third_party_vendor_name == other.third_party_vendor_name
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [value, ref, display, status, last_sync_time, factor_type, factor_status, authentication_method, third_party_vendor_name].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
