# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: release
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The details of the KeyVersion associated with the Key.
  class KeyManagement::Models::KeyVersion
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_CREATING = 'CREATING'.freeze,
      LIFECYCLE_STATE_ENABLING = 'ENABLING'.freeze,
      LIFECYCLE_STATE_ENABLED = 'ENABLED'.freeze,
      LIFECYCLE_STATE_DISABLING = 'DISABLING'.freeze,
      LIFECYCLE_STATE_DISABLED = 'DISABLED'.freeze,
      LIFECYCLE_STATE_DELETING = 'DELETING'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_PENDING_DELETION = 'PENDING_DELETION'.freeze,
      LIFECYCLE_STATE_SCHEDULING_DELETION = 'SCHEDULING_DELETION'.freeze,
      LIFECYCLE_STATE_CANCELLING_DELETION = 'CANCELLING_DELETION'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    ORIGIN_ENUM = [
      ORIGIN_INTERNAL = 'INTERNAL'.freeze,
      ORIGIN_EXTERNAL = 'EXTERNAL'.freeze,
      ORIGIN_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The OCID of the compartment that contains this key version.
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** The OCID of the key version.
    # @return [String]
    attr_accessor :id

    # **[Required]** The OCID of the key associated with this key version.
    # @return [String]
    attr_accessor :key_id

    # The public key in PEM format. (This value pertains only to RSA and ECDSA keys.)
    #
    # @return [String]
    attr_accessor :public_key

    # The key version's current lifecycle state.
    #
    # Example: `ENABLED`
    #
    # @return [String]
    attr_reader :lifecycle_state

    # The source of the key material. When this value is `INTERNAL`, Key Management
    # created the key material. When this value is `EXTERNAL`, the key material
    # was imported from an external source.
    #
    # @return [String]
    attr_reader :origin

    # **[Required]** The date and time this key version was created, expressed in [RFC 3339](https://tools.ietf.org/html/rfc3339) timestamp format.
    #
    # Example: \"2018-04-03T21:10:29.600Z\"
    #
    # @return [DateTime]
    attr_accessor :time_created

    # An optional property indicating when to delete the key version, expressed in [RFC 3339](https://tools.ietf.org/html/rfc3339) timestamp format.
    # Example: `2019-04-03T21:10:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :time_of_deletion

    # **[Required]** The OCID of the vault that contains this key version.
    # @return [String]
    attr_accessor :vault_id

    # The OCID of the key version from which this key version was restored.
    # @return [String]
    attr_accessor :restored_from_key_version_id

    # @return [OCI::KeyManagement::Models::KeyVersionReplicaDetails]
    attr_accessor :replica_details

    # A Boolean value that indicates whether the KeyVersion belongs to primary Vault or replica Vault.
    # @return [BOOLEAN]
    attr_accessor :is_primary

    # @return [OCI::KeyManagement::Models::ExternalKeyReferenceDetails]
    attr_accessor :external_key_reference_details

    # A Boolean Value indicating whether this keyversion is generated from auto rotation of key
    # @return [BOOLEAN]
    attr_accessor :is_auto_rotated

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'compartmentId',
        'id': :'id',
        'key_id': :'keyId',
        'public_key': :'publicKey',
        'lifecycle_state': :'lifecycleState',
        'origin': :'origin',
        'time_created': :'timeCreated',
        'time_of_deletion': :'timeOfDeletion',
        'vault_id': :'vaultId',
        'restored_from_key_version_id': :'restoredFromKeyVersionId',
        'replica_details': :'replicaDetails',
        'is_primary': :'isPrimary',
        'external_key_reference_details': :'externalKeyReferenceDetails',
        'is_auto_rotated': :'isAutoRotated'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'String',
        'id': :'String',
        'key_id': :'String',
        'public_key': :'String',
        'lifecycle_state': :'String',
        'origin': :'String',
        'time_created': :'DateTime',
        'time_of_deletion': :'DateTime',
        'vault_id': :'String',
        'restored_from_key_version_id': :'String',
        'replica_details': :'OCI::KeyManagement::Models::KeyVersionReplicaDetails',
        'is_primary': :'BOOLEAN',
        'external_key_reference_details': :'OCI::KeyManagement::Models::ExternalKeyReferenceDetails',
        'is_auto_rotated': :'BOOLEAN'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :key_id The value to assign to the {#key_id} property
    # @option attributes [String] :public_key The value to assign to the {#public_key} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :origin The value to assign to the {#origin} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_of_deletion The value to assign to the {#time_of_deletion} property
    # @option attributes [String] :vault_id The value to assign to the {#vault_id} property
    # @option attributes [String] :restored_from_key_version_id The value to assign to the {#restored_from_key_version_id} property
    # @option attributes [OCI::KeyManagement::Models::KeyVersionReplicaDetails] :replica_details The value to assign to the {#replica_details} property
    # @option attributes [BOOLEAN] :is_primary The value to assign to the {#is_primary} property
    # @option attributes [OCI::KeyManagement::Models::ExternalKeyReferenceDetails] :external_key_reference_details The value to assign to the {#external_key_reference_details} property
    # @option attributes [BOOLEAN] :is_auto_rotated The value to assign to the {#is_auto_rotated} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.id = attributes[:'id'] if attributes[:'id']

      self.key_id = attributes[:'keyId'] if attributes[:'keyId']

      raise 'You cannot provide both :keyId and :key_id' if attributes.key?(:'keyId') && attributes.key?(:'key_id')

      self.key_id = attributes[:'key_id'] if attributes[:'key_id']

      self.public_key = attributes[:'publicKey'] if attributes[:'publicKey']

      raise 'You cannot provide both :publicKey and :public_key' if attributes.key?(:'publicKey') && attributes.key?(:'public_key')

      self.public_key = attributes[:'public_key'] if attributes[:'public_key']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.origin = attributes[:'origin'] if attributes[:'origin']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_of_deletion = attributes[:'timeOfDeletion'] if attributes[:'timeOfDeletion']

      raise 'You cannot provide both :timeOfDeletion and :time_of_deletion' if attributes.key?(:'timeOfDeletion') && attributes.key?(:'time_of_deletion')

      self.time_of_deletion = attributes[:'time_of_deletion'] if attributes[:'time_of_deletion']

      self.vault_id = attributes[:'vaultId'] if attributes[:'vaultId']

      raise 'You cannot provide both :vaultId and :vault_id' if attributes.key?(:'vaultId') && attributes.key?(:'vault_id')

      self.vault_id = attributes[:'vault_id'] if attributes[:'vault_id']

      self.restored_from_key_version_id = attributes[:'restoredFromKeyVersionId'] if attributes[:'restoredFromKeyVersionId']

      raise 'You cannot provide both :restoredFromKeyVersionId and :restored_from_key_version_id' if attributes.key?(:'restoredFromKeyVersionId') && attributes.key?(:'restored_from_key_version_id')

      self.restored_from_key_version_id = attributes[:'restored_from_key_version_id'] if attributes[:'restored_from_key_version_id']

      self.replica_details = attributes[:'replicaDetails'] if attributes[:'replicaDetails']

      raise 'You cannot provide both :replicaDetails and :replica_details' if attributes.key?(:'replicaDetails') && attributes.key?(:'replica_details')

      self.replica_details = attributes[:'replica_details'] if attributes[:'replica_details']

      self.is_primary = attributes[:'isPrimary'] unless attributes[:'isPrimary'].nil?

      raise 'You cannot provide both :isPrimary and :is_primary' if attributes.key?(:'isPrimary') && attributes.key?(:'is_primary')

      self.is_primary = attributes[:'is_primary'] unless attributes[:'is_primary'].nil?

      self.external_key_reference_details = attributes[:'externalKeyReferenceDetails'] if attributes[:'externalKeyReferenceDetails']

      raise 'You cannot provide both :externalKeyReferenceDetails and :external_key_reference_details' if attributes.key?(:'externalKeyReferenceDetails') && attributes.key?(:'external_key_reference_details')

      self.external_key_reference_details = attributes[:'external_key_reference_details'] if attributes[:'external_key_reference_details']

      self.is_auto_rotated = attributes[:'isAutoRotated'] unless attributes[:'isAutoRotated'].nil?

      raise 'You cannot provide both :isAutoRotated and :is_auto_rotated' if attributes.key?(:'isAutoRotated') && attributes.key?(:'is_auto_rotated')

      self.is_auto_rotated = attributes[:'is_auto_rotated'] unless attributes[:'is_auto_rotated'].nil?
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] origin Object to be assigned
    def origin=(origin)
      # rubocop:disable Style/ConditionalAssignment
      if origin && !ORIGIN_ENUM.include?(origin)
        OCI.logger.debug("Unknown value for 'origin' [" + origin + "]. Mapping to 'ORIGIN_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @origin = ORIGIN_UNKNOWN_ENUM_VALUE
      else
        @origin = origin
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        compartment_id == other.compartment_id &&
        id == other.id &&
        key_id == other.key_id &&
        public_key == other.public_key &&
        lifecycle_state == other.lifecycle_state &&
        origin == other.origin &&
        time_created == other.time_created &&
        time_of_deletion == other.time_of_deletion &&
        vault_id == other.vault_id &&
        restored_from_key_version_id == other.restored_from_key_version_id &&
        replica_details == other.replica_details &&
        is_primary == other.is_primary &&
        external_key_reference_details == other.external_key_reference_details &&
        is_auto_rotated == other.is_auto_rotated
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [compartment_id, id, key_id, public_key, lifecycle_state, origin, time_created, time_of_deletion, vault_id, restored_from_key_version_id, replica_details, is_primary, external_key_reference_details, is_auto_rotated].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
