# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200601
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Description of a log analytics entity.
  #
  class LogAnalytics::Models::LogAnalyticsEntity
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The log analytics entity OCID. This ID is a reference used by log analytics features and it represents
    # a resource that is provisioned and managed by the customer on their premises or on the cloud.
    #
    # @return [String]
    attr_accessor :id

    # **[Required]** Log analytics entity name.
    #
    # @return [String]
    attr_accessor :name

    # **[Required]** Compartment Identifier [OCID] (https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm).
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** Log analytics entity type name.
    #
    # @return [String]
    attr_accessor :entity_type_name

    # **[Required]** Internal name for the log analytics entity type.
    #
    # @return [String]
    attr_accessor :entity_type_internal_name

    # **[Required]** The current state of the log analytics entity.
    #
    # @return [String]
    attr_reader :lifecycle_state

    # **[Required]** lifecycleDetails has additional information regarding substeps such as management agent plugin deployment.
    #
    # @return [String]
    attr_accessor :lifecycle_details

    # The OCID of the Management Agent.
    #
    # @return [String]
    attr_accessor :management_agent_id

    # Management agent (management-agents resource kind) display name
    #
    # @return [String]
    attr_accessor :management_agent_display_name

    # Management agent (management-agents resource kind) compartment OCID
    #
    # @return [String]
    attr_accessor :management_agent_compartment_id

    # The timezone region of the log analytics entity.
    #
    # @return [String]
    attr_accessor :timezone_region

    # The name/value pairs for parameter values to be used in file patterns specified in log sources.
    #
    # @return [Hash<String, String>]
    attr_accessor :properties

    # @return [OCI::LogAnalytics::Models::CreationSource]
    attr_accessor :creation_source

    # **[Required]** The date and time the resource was created, in the format defined by RFC3339.
    #
    # @return [DateTime]
    attr_accessor :time_created

    # **[Required]** The date and time the resource was last updated, in the format defined by RFC3339.
    #
    # @return [DateTime]
    attr_accessor :time_updated

    # The date and time the resource was last discovered, in the format defined by RFC3339.
    #
    # @return [DateTime]
    attr_accessor :time_last_discovered

    # @return [OCI::LogAnalytics::Models::LogAnalyticsMetadataSummary]
    attr_accessor :metadata

    # The Boolean flag to indicate if logs are collected for an entity for log analytics usage.
    #
    # @return [BOOLEAN]
    attr_accessor :are_logs_collected

    # The OCID of the Cloud resource which this entity is a representation of. This may be blank when the entity
    # represents a non-cloud resource that the customer may have on their premises.
    #
    # @return [String]
    attr_accessor :cloud_resource_id

    # The hostname where the entity represented here is actually present. This would be the output one would get if
    # they run `echo $HOSTNAME` on Linux or an equivalent OS command. This may be different from
    # management agents host since logs may be collected remotely.
    #
    # @return [String]
    attr_accessor :hostname

    # This indicates the type of source. It is primarily for Enterprise Manager Repository ID.
    #
    # @return [String]
    attr_accessor :source_id

    # Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
    # Example: `{\"bar-key\": \"value\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'name': :'name',
        'compartment_id': :'compartmentId',
        'entity_type_name': :'entityTypeName',
        'entity_type_internal_name': :'entityTypeInternalName',
        'lifecycle_state': :'lifecycleState',
        'lifecycle_details': :'lifecycleDetails',
        'management_agent_id': :'managementAgentId',
        'management_agent_display_name': :'managementAgentDisplayName',
        'management_agent_compartment_id': :'managementAgentCompartmentId',
        'timezone_region': :'timezoneRegion',
        'properties': :'properties',
        'creation_source': :'creationSource',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'time_last_discovered': :'timeLastDiscovered',
        'metadata': :'metadata',
        'are_logs_collected': :'areLogsCollected',
        'cloud_resource_id': :'cloudResourceId',
        'hostname': :'hostname',
        'source_id': :'sourceId',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'name': :'String',
        'compartment_id': :'String',
        'entity_type_name': :'String',
        'entity_type_internal_name': :'String',
        'lifecycle_state': :'String',
        'lifecycle_details': :'String',
        'management_agent_id': :'String',
        'management_agent_display_name': :'String',
        'management_agent_compartment_id': :'String',
        'timezone_region': :'String',
        'properties': :'Hash<String, String>',
        'creation_source': :'OCI::LogAnalytics::Models::CreationSource',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'time_last_discovered': :'DateTime',
        'metadata': :'OCI::LogAnalytics::Models::LogAnalyticsMetadataSummary',
        'are_logs_collected': :'BOOLEAN',
        'cloud_resource_id': :'String',
        'hostname': :'String',
        'source_id': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :entity_type_name The value to assign to the {#entity_type_name} property
    # @option attributes [String] :entity_type_internal_name The value to assign to the {#entity_type_internal_name} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :lifecycle_details The value to assign to the {#lifecycle_details} property
    # @option attributes [String] :management_agent_id The value to assign to the {#management_agent_id} property
    # @option attributes [String] :management_agent_display_name The value to assign to the {#management_agent_display_name} property
    # @option attributes [String] :management_agent_compartment_id The value to assign to the {#management_agent_compartment_id} property
    # @option attributes [String] :timezone_region The value to assign to the {#timezone_region} property
    # @option attributes [Hash<String, String>] :properties The value to assign to the {#properties} property
    # @option attributes [OCI::LogAnalytics::Models::CreationSource] :creation_source The value to assign to the {#creation_source} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [DateTime] :time_last_discovered The value to assign to the {#time_last_discovered} property
    # @option attributes [OCI::LogAnalytics::Models::LogAnalyticsMetadataSummary] :metadata The value to assign to the {#metadata} property
    # @option attributes [BOOLEAN] :are_logs_collected The value to assign to the {#are_logs_collected} property
    # @option attributes [String] :cloud_resource_id The value to assign to the {#cloud_resource_id} property
    # @option attributes [String] :hostname The value to assign to the {#hostname} property
    # @option attributes [String] :source_id The value to assign to the {#source_id} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.name = attributes[:'name'] if attributes[:'name']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.entity_type_name = attributes[:'entityTypeName'] if attributes[:'entityTypeName']

      raise 'You cannot provide both :entityTypeName and :entity_type_name' if attributes.key?(:'entityTypeName') && attributes.key?(:'entity_type_name')

      self.entity_type_name = attributes[:'entity_type_name'] if attributes[:'entity_type_name']

      self.entity_type_internal_name = attributes[:'entityTypeInternalName'] if attributes[:'entityTypeInternalName']

      raise 'You cannot provide both :entityTypeInternalName and :entity_type_internal_name' if attributes.key?(:'entityTypeInternalName') && attributes.key?(:'entity_type_internal_name')

      self.entity_type_internal_name = attributes[:'entity_type_internal_name'] if attributes[:'entity_type_internal_name']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.lifecycle_details = attributes[:'lifecycleDetails'] if attributes[:'lifecycleDetails']

      raise 'You cannot provide both :lifecycleDetails and :lifecycle_details' if attributes.key?(:'lifecycleDetails') && attributes.key?(:'lifecycle_details')

      self.lifecycle_details = attributes[:'lifecycle_details'] if attributes[:'lifecycle_details']

      self.management_agent_id = attributes[:'managementAgentId'] if attributes[:'managementAgentId']

      raise 'You cannot provide both :managementAgentId and :management_agent_id' if attributes.key?(:'managementAgentId') && attributes.key?(:'management_agent_id')

      self.management_agent_id = attributes[:'management_agent_id'] if attributes[:'management_agent_id']

      self.management_agent_display_name = attributes[:'managementAgentDisplayName'] if attributes[:'managementAgentDisplayName']

      raise 'You cannot provide both :managementAgentDisplayName and :management_agent_display_name' if attributes.key?(:'managementAgentDisplayName') && attributes.key?(:'management_agent_display_name')

      self.management_agent_display_name = attributes[:'management_agent_display_name'] if attributes[:'management_agent_display_name']

      self.management_agent_compartment_id = attributes[:'managementAgentCompartmentId'] if attributes[:'managementAgentCompartmentId']

      raise 'You cannot provide both :managementAgentCompartmentId and :management_agent_compartment_id' if attributes.key?(:'managementAgentCompartmentId') && attributes.key?(:'management_agent_compartment_id')

      self.management_agent_compartment_id = attributes[:'management_agent_compartment_id'] if attributes[:'management_agent_compartment_id']

      self.timezone_region = attributes[:'timezoneRegion'] if attributes[:'timezoneRegion']

      raise 'You cannot provide both :timezoneRegion and :timezone_region' if attributes.key?(:'timezoneRegion') && attributes.key?(:'timezone_region')

      self.timezone_region = attributes[:'timezone_region'] if attributes[:'timezone_region']

      self.properties = attributes[:'properties'] if attributes[:'properties']

      self.creation_source = attributes[:'creationSource'] if attributes[:'creationSource']

      raise 'You cannot provide both :creationSource and :creation_source' if attributes.key?(:'creationSource') && attributes.key?(:'creation_source')

      self.creation_source = attributes[:'creation_source'] if attributes[:'creation_source']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.time_last_discovered = attributes[:'timeLastDiscovered'] if attributes[:'timeLastDiscovered']

      raise 'You cannot provide both :timeLastDiscovered and :time_last_discovered' if attributes.key?(:'timeLastDiscovered') && attributes.key?(:'time_last_discovered')

      self.time_last_discovered = attributes[:'time_last_discovered'] if attributes[:'time_last_discovered']

      self.metadata = attributes[:'metadata'] if attributes[:'metadata']

      self.are_logs_collected = attributes[:'areLogsCollected'] unless attributes[:'areLogsCollected'].nil?

      raise 'You cannot provide both :areLogsCollected and :are_logs_collected' if attributes.key?(:'areLogsCollected') && attributes.key?(:'are_logs_collected')

      self.are_logs_collected = attributes[:'are_logs_collected'] unless attributes[:'are_logs_collected'].nil?

      self.cloud_resource_id = attributes[:'cloudResourceId'] if attributes[:'cloudResourceId']

      raise 'You cannot provide both :cloudResourceId and :cloud_resource_id' if attributes.key?(:'cloudResourceId') && attributes.key?(:'cloud_resource_id')

      self.cloud_resource_id = attributes[:'cloud_resource_id'] if attributes[:'cloud_resource_id']

      self.hostname = attributes[:'hostname'] if attributes[:'hostname']

      self.source_id = attributes[:'sourceId'] if attributes[:'sourceId']

      raise 'You cannot provide both :sourceId and :source_id' if attributes.key?(:'sourceId') && attributes.key?(:'source_id')

      self.source_id = attributes[:'source_id'] if attributes[:'source_id']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        name == other.name &&
        compartment_id == other.compartment_id &&
        entity_type_name == other.entity_type_name &&
        entity_type_internal_name == other.entity_type_internal_name &&
        lifecycle_state == other.lifecycle_state &&
        lifecycle_details == other.lifecycle_details &&
        management_agent_id == other.management_agent_id &&
        management_agent_display_name == other.management_agent_display_name &&
        management_agent_compartment_id == other.management_agent_compartment_id &&
        timezone_region == other.timezone_region &&
        properties == other.properties &&
        creation_source == other.creation_source &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        time_last_discovered == other.time_last_discovered &&
        metadata == other.metadata &&
        are_logs_collected == other.are_logs_collected &&
        cloud_resource_id == other.cloud_resource_id &&
        hostname == other.hostname &&
        source_id == other.source_id &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, name, compartment_id, entity_type_name, entity_type_internal_name, lifecycle_state, lifecycle_details, management_agent_id, management_agent_display_name, management_agent_compartment_id, timezone_region, properties, creation_source, time_created, time_updated, time_last_discovered, metadata, are_logs_collected, cloud_resource_id, hostname, source_id, freeform_tags, defined_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
