# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200531
require 'date'
require_relative 'unified_agent_logging_filter'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Logging custom filter plugin.
  class Logging::Models::UnifiedAgentCustomFilter < Logging::Models::UnifiedAgentLoggingFilter
    # **[Required]** Type of the custom filter
    # @return [String]
    attr_accessor :custom_filter_type

    # Parameters of the custom filter
    # @return [Hash<String, String>]
    attr_accessor :params

    # List of custom sections in custom filter
    # @return [Array<OCI::Logging::Models::UnifiedAgentCustomSection>]
    attr_accessor :custom_sections

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'name': :'name',
        'filter_type': :'filterType',
        'custom_filter_type': :'customFilterType',
        'params': :'params',
        'custom_sections': :'customSections'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'name': :'String',
        'filter_type': :'String',
        'custom_filter_type': :'String',
        'params': :'Hash<String, String>',
        'custom_sections': :'Array<OCI::Logging::Models::UnifiedAgentCustomSection>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :name The value to assign to the {OCI::Logging::Models::UnifiedAgentLoggingFilter#name #name} proprety
    # @option attributes [String] :custom_filter_type The value to assign to the {#custom_filter_type} property
    # @option attributes [Hash<String, String>] :params The value to assign to the {#params} property
    # @option attributes [Array<OCI::Logging::Models::UnifiedAgentCustomSection>] :custom_sections The value to assign to the {#custom_sections} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['filterType'] = 'CUSTOM_FILTER'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.custom_filter_type = attributes[:'customFilterType'] if attributes[:'customFilterType']

      raise 'You cannot provide both :customFilterType and :custom_filter_type' if attributes.key?(:'customFilterType') && attributes.key?(:'custom_filter_type')

      self.custom_filter_type = attributes[:'custom_filter_type'] if attributes[:'custom_filter_type']

      self.params = attributes[:'params'] if attributes[:'params']

      self.custom_sections = attributes[:'customSections'] if attributes[:'customSections']

      raise 'You cannot provide both :customSections and :custom_sections' if attributes.key?(:'customSections') && attributes.key?(:'custom_sections')

      self.custom_sections = attributes[:'custom_sections'] if attributes[:'custom_sections']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        name == other.name &&
        filter_type == other.filter_type &&
        custom_filter_type == other.custom_filter_type &&
        params == other.params &&
        custom_sections == other.custom_sections
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [name, filter_type, custom_filter_type, params, custom_sections].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
