# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200202
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Supported Agent downloads
  class ManagementAgent::Models::ManagementAgentImage
    PLATFORM_TYPE_ENUM = [
      PLATFORM_TYPE_LINUX = 'LINUX'.freeze,
      PLATFORM_TYPE_WINDOWS = 'WINDOWS'.freeze,
      PLATFORM_TYPE_SOLARIS = 'SOLARIS'.freeze,
      PLATFORM_TYPE_MACOSX = 'MACOSX'.freeze
    ].freeze

    PACKAGE_TYPE_ENUM = [
      PACKAGE_TYPE_RPM = 'RPM'.freeze,
      PACKAGE_TYPE_ZIP = 'ZIP'.freeze
    ].freeze

    PACKAGE_ARCHITECTURE_TYPE_ENUM = [
      PACKAGE_ARCHITECTURE_TYPE_X86_64 = 'X86_64'.freeze,
      PACKAGE_ARCHITECTURE_TYPE_SPARC = 'SPARC'.freeze,
      PACKAGE_ARCHITECTURE_TYPE_X86 = 'X86'.freeze,
      PACKAGE_ARCHITECTURE_TYPE_M1 = 'M1'.freeze,
      PACKAGE_ARCHITECTURE_TYPE_AARCH64 = 'AARCH64'.freeze
    ].freeze

    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_CREATING = 'CREATING'.freeze,
      LIFECYCLE_STATE_UPDATING = 'UPDATING'.freeze,
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_INACTIVE = 'INACTIVE'.freeze,
      LIFECYCLE_STATE_TERMINATED = 'TERMINATED'.freeze,
      LIFECYCLE_STATE_DELETING = 'DELETING'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze
    ].freeze

    # **[Required]** Agent image resource id
    # @return [String]
    attr_accessor :id

    # **[Required]** Agent image platform type
    # @return [String]
    attr_reader :platform_type

    # Agent image platform display name
    # @return [String]
    attr_accessor :platform_name

    # The installation package type
    # @return [String]
    attr_reader :package_type

    # The installation package target architecture type
    # @return [String]
    attr_reader :package_architecture_type

    # **[Required]** Agent image version
    # @return [String]
    attr_accessor :version

    # Agent image size in bytes
    # @return [Float]
    attr_accessor :size

    # Agent image content SHA256 Hash
    # @return [String]
    attr_accessor :checksum

    # Object storage URL for download
    # @return [String]
    attr_accessor :object_url

    # @return [OCI::ManagementAgent::Models::ObjectDetails]
    attr_accessor :image_object_storage_details

    # The current state of Management Agent Image
    # @return [String]
    attr_reader :lifecycle_state

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'platform_type': :'platformType',
        'platform_name': :'platformName',
        'package_type': :'packageType',
        'package_architecture_type': :'packageArchitectureType',
        'version': :'version',
        'size': :'size',
        'checksum': :'checksum',
        'object_url': :'objectUrl',
        'image_object_storage_details': :'imageObjectStorageDetails',
        'lifecycle_state': :'lifecycleState'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'platform_type': :'String',
        'platform_name': :'String',
        'package_type': :'String',
        'package_architecture_type': :'String',
        'version': :'String',
        'size': :'Float',
        'checksum': :'String',
        'object_url': :'String',
        'image_object_storage_details': :'OCI::ManagementAgent::Models::ObjectDetails',
        'lifecycle_state': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :platform_type The value to assign to the {#platform_type} property
    # @option attributes [String] :platform_name The value to assign to the {#platform_name} property
    # @option attributes [String] :package_type The value to assign to the {#package_type} property
    # @option attributes [String] :package_architecture_type The value to assign to the {#package_architecture_type} property
    # @option attributes [String] :version The value to assign to the {#version} property
    # @option attributes [Float] :size The value to assign to the {#size} property
    # @option attributes [String] :checksum The value to assign to the {#checksum} property
    # @option attributes [String] :object_url The value to assign to the {#object_url} property
    # @option attributes [OCI::ManagementAgent::Models::ObjectDetails] :image_object_storage_details The value to assign to the {#image_object_storage_details} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.platform_type = attributes[:'platformType'] if attributes[:'platformType']

      raise 'You cannot provide both :platformType and :platform_type' if attributes.key?(:'platformType') && attributes.key?(:'platform_type')

      self.platform_type = attributes[:'platform_type'] if attributes[:'platform_type']

      self.platform_name = attributes[:'platformName'] if attributes[:'platformName']

      raise 'You cannot provide both :platformName and :platform_name' if attributes.key?(:'platformName') && attributes.key?(:'platform_name')

      self.platform_name = attributes[:'platform_name'] if attributes[:'platform_name']

      self.package_type = attributes[:'packageType'] if attributes[:'packageType']

      raise 'You cannot provide both :packageType and :package_type' if attributes.key?(:'packageType') && attributes.key?(:'package_type')

      self.package_type = attributes[:'package_type'] if attributes[:'package_type']

      self.package_architecture_type = attributes[:'packageArchitectureType'] if attributes[:'packageArchitectureType']

      raise 'You cannot provide both :packageArchitectureType and :package_architecture_type' if attributes.key?(:'packageArchitectureType') && attributes.key?(:'package_architecture_type')

      self.package_architecture_type = attributes[:'package_architecture_type'] if attributes[:'package_architecture_type']

      self.version = attributes[:'version'] if attributes[:'version']

      self.size = attributes[:'size'] if attributes[:'size']

      self.checksum = attributes[:'checksum'] if attributes[:'checksum']

      self.object_url = attributes[:'objectUrl'] if attributes[:'objectUrl']

      raise 'You cannot provide both :objectUrl and :object_url' if attributes.key?(:'objectUrl') && attributes.key?(:'object_url')

      self.object_url = attributes[:'object_url'] if attributes[:'object_url']

      self.image_object_storage_details = attributes[:'imageObjectStorageDetails'] if attributes[:'imageObjectStorageDetails']

      raise 'You cannot provide both :imageObjectStorageDetails and :image_object_storage_details' if attributes.key?(:'imageObjectStorageDetails') && attributes.key?(:'image_object_storage_details')

      self.image_object_storage_details = attributes[:'image_object_storage_details'] if attributes[:'image_object_storage_details']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] platform_type Object to be assigned
    def platform_type=(platform_type)
      raise "Invalid value for 'platform_type': this must be one of the values in PLATFORM_TYPE_ENUM." if platform_type && !PLATFORM_TYPE_ENUM.include?(platform_type)

      @platform_type = platform_type
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] package_type Object to be assigned
    def package_type=(package_type)
      raise "Invalid value for 'package_type': this must be one of the values in PACKAGE_TYPE_ENUM." if package_type && !PACKAGE_TYPE_ENUM.include?(package_type)

      @package_type = package_type
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] package_architecture_type Object to be assigned
    def package_architecture_type=(package_architecture_type)
      raise "Invalid value for 'package_architecture_type': this must be one of the values in PACKAGE_ARCHITECTURE_TYPE_ENUM." if package_architecture_type && !PACKAGE_ARCHITECTURE_TYPE_ENUM.include?(package_architecture_type)

      @package_architecture_type = package_architecture_type
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      raise "Invalid value for 'lifecycle_state': this must be one of the values in LIFECYCLE_STATE_ENUM." if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)

      @lifecycle_state = lifecycle_state
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        platform_type == other.platform_type &&
        platform_name == other.platform_name &&
        package_type == other.package_type &&
        package_architecture_type == other.package_architecture_type &&
        version == other.version &&
        size == other.size &&
        checksum == other.checksum &&
        object_url == other.object_url &&
        image_object_storage_details == other.image_object_storage_details &&
        lifecycle_state == other.lifecycle_state
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, platform_type, platform_name, package_type, package_architecture_type, version, size, checksum, object_url, image_object_storage_details, lifecycle_state].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
