# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20180401
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # A summary of properties for the specified alarm suppression history item.
  #
  class Monitoring::Models::AlarmSuppressionHistoryItem
    LEVEL_ENUM = [
      LEVEL_ALARM = 'ALARM'.freeze,
      LEVEL_DIMENSION = 'DIMENSION'.freeze,
      LEVEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the alarm suppression.
    # @return [String]
    attr_accessor :suppression_id

    # This attribute is required.
    # @return [OCI::Monitoring::Models::AlarmSuppressionTarget]
    attr_accessor :alarm_suppression_target

    # **[Required]** The level of this alarm suppression.
    # `ALARM` indicates a suppression of the entire alarm, regardless of dimension.
    # `DIMENSION` indicates a suppression configured for specified dimensions.
    #
    # @return [String]
    attr_reader :level

    # **[Required]** A user-friendly name for the alarm suppression. It does not have to be unique, and it's changeable. Avoid entering confidential information.
    # @return [String]
    attr_accessor :display_name

    # Human-readable reason for this alarm suppression.
    # It does not have to be unique, and it's changeable.
    # Avoid entering confidential information.
    #
    # Oracle recommends including tracking information for the event or associated work,
    # such as a ticket number.
    #
    # Example: `Planned outage due to change IT-1234.`
    #
    # @return [String]
    attr_accessor :description

    # Configured dimension filter for suppressing alarm state entries that include the set of specified dimension key-value pairs.
    #
    # Example: `{\"resourceId\": \"ocid1.instance.region1.phx.exampleuniqueID\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :dimensions

    # **[Required]** The start date and time for the suppression actually starts, inclusive. Format defined by RFC3339.
    #
    # Example: `2023-02-01T01:02:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :time_effective_from

    # **[Required]** The end date and time for the suppression actually ends, inclusive. Format defined by RFC3339.
    #
    # Example: `2023-02-01T02:02:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :time_effective_until

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'suppression_id': :'suppressionId',
        'alarm_suppression_target': :'alarmSuppressionTarget',
        'level': :'level',
        'display_name': :'displayName',
        'description': :'description',
        'dimensions': :'dimensions',
        'time_effective_from': :'timeEffectiveFrom',
        'time_effective_until': :'timeEffectiveUntil'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'suppression_id': :'String',
        'alarm_suppression_target': :'OCI::Monitoring::Models::AlarmSuppressionTarget',
        'level': :'String',
        'display_name': :'String',
        'description': :'String',
        'dimensions': :'Hash<String, String>',
        'time_effective_from': :'DateTime',
        'time_effective_until': :'DateTime'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :suppression_id The value to assign to the {#suppression_id} property
    # @option attributes [OCI::Monitoring::Models::AlarmSuppressionTarget] :alarm_suppression_target The value to assign to the {#alarm_suppression_target} property
    # @option attributes [String] :level The value to assign to the {#level} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :description The value to assign to the {#description} property
    # @option attributes [Hash<String, String>] :dimensions The value to assign to the {#dimensions} property
    # @option attributes [DateTime] :time_effective_from The value to assign to the {#time_effective_from} property
    # @option attributes [DateTime] :time_effective_until The value to assign to the {#time_effective_until} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.suppression_id = attributes[:'suppressionId'] if attributes[:'suppressionId']

      raise 'You cannot provide both :suppressionId and :suppression_id' if attributes.key?(:'suppressionId') && attributes.key?(:'suppression_id')

      self.suppression_id = attributes[:'suppression_id'] if attributes[:'suppression_id']

      self.alarm_suppression_target = attributes[:'alarmSuppressionTarget'] if attributes[:'alarmSuppressionTarget']

      raise 'You cannot provide both :alarmSuppressionTarget and :alarm_suppression_target' if attributes.key?(:'alarmSuppressionTarget') && attributes.key?(:'alarm_suppression_target')

      self.alarm_suppression_target = attributes[:'alarm_suppression_target'] if attributes[:'alarm_suppression_target']

      self.level = attributes[:'level'] if attributes[:'level']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.description = attributes[:'description'] if attributes[:'description']

      self.dimensions = attributes[:'dimensions'] if attributes[:'dimensions']

      self.time_effective_from = attributes[:'timeEffectiveFrom'] if attributes[:'timeEffectiveFrom']

      raise 'You cannot provide both :timeEffectiveFrom and :time_effective_from' if attributes.key?(:'timeEffectiveFrom') && attributes.key?(:'time_effective_from')

      self.time_effective_from = attributes[:'time_effective_from'] if attributes[:'time_effective_from']

      self.time_effective_until = attributes[:'timeEffectiveUntil'] if attributes[:'timeEffectiveUntil']

      raise 'You cannot provide both :timeEffectiveUntil and :time_effective_until' if attributes.key?(:'timeEffectiveUntil') && attributes.key?(:'time_effective_until')

      self.time_effective_until = attributes[:'time_effective_until'] if attributes[:'time_effective_until']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] level Object to be assigned
    def level=(level)
      # rubocop:disable Style/ConditionalAssignment
      if level && !LEVEL_ENUM.include?(level)
        OCI.logger.debug("Unknown value for 'level' [" + level + "]. Mapping to 'LEVEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @level = LEVEL_UNKNOWN_ENUM_VALUE
      else
        @level = level
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        suppression_id == other.suppression_id &&
        alarm_suppression_target == other.alarm_suppression_target &&
        level == other.level &&
        display_name == other.display_name &&
        description == other.description &&
        dimensions == other.dimensions &&
        time_effective_from == other.time_effective_from &&
        time_effective_until == other.time_effective_until
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [suppression_id, alarm_suppression_target, level, display_name, description, dimensions, time_effective_from, time_effective_until].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
