# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20160918
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # To use any of the API operations, you must be authorized in an IAM policy. If you are not authorized,
  # talk to an administrator. If you are an administrator who needs to write policies to give users access, see
  # [Getting Started with Policies](https://docs.cloud.oracle.com/Content/Identity/Concepts/policygetstarted.htm).
  #
  class ObjectStorage::Models::UpdateBucketDetails
    PUBLIC_ACCESS_TYPE_ENUM = [
      PUBLIC_ACCESS_TYPE_NO_PUBLIC_ACCESS = 'NoPublicAccess'.freeze,
      PUBLIC_ACCESS_TYPE_OBJECT_READ = 'ObjectRead'.freeze,
      PUBLIC_ACCESS_TYPE_OBJECT_READ_WITHOUT_LIST = 'ObjectReadWithoutList'.freeze
    ].freeze

    VERSIONING_ENUM = [
      VERSIONING_ENABLED = 'Enabled'.freeze,
      VERSIONING_SUSPENDED = 'Suspended'.freeze
    ].freeze

    # The Object Storage namespace in which the bucket lives.
    # @return [String]
    attr_accessor :namespace

    # The compartmentId for the compartment to move the bucket to.
    # @return [String]
    attr_accessor :compartment_id

    # The name of the bucket. Valid characters are uppercase or lowercase letters, numbers, hyphens, underscores, and periods.
    # Bucket names must be unique within an Object Storage namespace. Avoid entering confidential information.
    # Example: my-new-bucket1
    #
    # @return [String]
    attr_accessor :name

    # Arbitrary string, up to 4KB, of keys and values for user-defined metadata.
    # @return [Hash<String, String>]
    attr_accessor :metadata

    # The type of public access enabled on this bucket. A bucket is set to `NoPublicAccess` by default, which only allows an
    # authenticated caller to access the bucket and its contents. When `ObjectRead` is enabled on the bucket, public access
    # is allowed for the `GetObject`, `HeadObject`, and `ListObjects` operations. When `ObjectReadWithoutList` is enabled
    # on the bucket, public access is allowed for the `GetObject` and `HeadObject` operations.
    #
    # @return [String]
    attr_reader :public_access_type

    # Whether or not events are emitted for object state changes in this bucket. By default, `objectEventsEnabled` is
    # set to `false`. Set `objectEventsEnabled` to `true` to emit events for object state changes. For more information
    # about events, see [Overview of Events](https://docs.cloud.oracle.com/Content/Events/Concepts/eventsoverview.htm).
    #
    # @return [BOOLEAN]
    attr_accessor :object_events_enabled

    # Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Operations\": {\"CostCenter\": \"42\"}}
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Key Management master encryption key to associate
    # with the specified bucket. If this value is empty, the Update operation will remove the associated key, if
    # there is one, from the bucket. (The bucket will continue to be encrypted, but with an encryption key managed
    # by Oracle.)
    #
    # @return [String]
    attr_accessor :kms_key_id

    # The versioning status on the bucket. If in state `Enabled`, multiple versions of the same object can be kept in the bucket.
    # When the object is overwritten or deleted, previous versions will still be available. When versioning is `Suspended`, the previous versions will still remain but new versions will no longer be created when overwitten or deleted.
    # Versioning cannot be disabled on a bucket once enabled.
    #
    # @return [String]
    attr_reader :versioning

    # The auto tiering status on the bucket. If in state `InfrequentAccess`, objects are transitioned
    # automatically between the 'Standard' and 'InfrequentAccess' tiers based on the access pattern of the objects.
    # When auto tiering is `Disabled`, there will be no automatic transitions between storage tiers.
    #
    # @return [String]
    attr_accessor :auto_tiering

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'namespace': :'namespace',
        'compartment_id': :'compartmentId',
        'name': :'name',
        'metadata': :'metadata',
        'public_access_type': :'publicAccessType',
        'object_events_enabled': :'objectEventsEnabled',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'kms_key_id': :'kmsKeyId',
        'versioning': :'versioning',
        'auto_tiering': :'autoTiering'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'namespace': :'String',
        'compartment_id': :'String',
        'name': :'String',
        'metadata': :'Hash<String, String>',
        'public_access_type': :'String',
        'object_events_enabled': :'BOOLEAN',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'kms_key_id': :'String',
        'versioning': :'String',
        'auto_tiering': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :namespace The value to assign to the {#namespace} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [Hash<String, String>] :metadata The value to assign to the {#metadata} property
    # @option attributes [String] :public_access_type The value to assign to the {#public_access_type} property
    # @option attributes [BOOLEAN] :object_events_enabled The value to assign to the {#object_events_enabled} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [String] :kms_key_id The value to assign to the {#kms_key_id} property
    # @option attributes [String] :versioning The value to assign to the {#versioning} property
    # @option attributes [String] :auto_tiering The value to assign to the {#auto_tiering} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.namespace = attributes[:'namespace'] if attributes[:'namespace']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.name = attributes[:'name'] if attributes[:'name']

      self.metadata = attributes[:'metadata'] if attributes[:'metadata']

      self.public_access_type = attributes[:'publicAccessType'] if attributes[:'publicAccessType']

      raise 'You cannot provide both :publicAccessType and :public_access_type' if attributes.key?(:'publicAccessType') && attributes.key?(:'public_access_type')

      self.public_access_type = attributes[:'public_access_type'] if attributes[:'public_access_type']

      self.object_events_enabled = attributes[:'objectEventsEnabled'] unless attributes[:'objectEventsEnabled'].nil?

      raise 'You cannot provide both :objectEventsEnabled and :object_events_enabled' if attributes.key?(:'objectEventsEnabled') && attributes.key?(:'object_events_enabled')

      self.object_events_enabled = attributes[:'object_events_enabled'] unless attributes[:'object_events_enabled'].nil?

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.kms_key_id = attributes[:'kmsKeyId'] if attributes[:'kmsKeyId']

      raise 'You cannot provide both :kmsKeyId and :kms_key_id' if attributes.key?(:'kmsKeyId') && attributes.key?(:'kms_key_id')

      self.kms_key_id = attributes[:'kms_key_id'] if attributes[:'kms_key_id']

      self.versioning = attributes[:'versioning'] if attributes[:'versioning']

      self.auto_tiering = attributes[:'autoTiering'] if attributes[:'autoTiering']

      raise 'You cannot provide both :autoTiering and :auto_tiering' if attributes.key?(:'autoTiering') && attributes.key?(:'auto_tiering')

      self.auto_tiering = attributes[:'auto_tiering'] if attributes[:'auto_tiering']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] public_access_type Object to be assigned
    def public_access_type=(public_access_type)
      raise "Invalid value for 'public_access_type': this must be one of the values in PUBLIC_ACCESS_TYPE_ENUM." if public_access_type && !PUBLIC_ACCESS_TYPE_ENUM.include?(public_access_type)

      @public_access_type = public_access_type
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] versioning Object to be assigned
    def versioning=(versioning)
      raise "Invalid value for 'versioning': this must be one of the values in VERSIONING_ENUM." if versioning && !VERSIONING_ENUM.include?(versioning)

      @versioning = versioning
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        namespace == other.namespace &&
        compartment_id == other.compartment_id &&
        name == other.name &&
        metadata == other.metadata &&
        public_access_type == other.public_access_type &&
        object_events_enabled == other.object_events_enabled &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        kms_key_id == other.kms_key_id &&
        versioning == other.versioning &&
        auto_tiering == other.auto_tiering
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [namespace, compartment_id, name, metadata, public_access_type, object_events_enabled, freeform_tags, defined_tags, kms_key_id, versioning, auto_tiering].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
