# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20230701
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details of the Cluster.
  class Ocvp::Models::CreateClusterDetails
    INITIAL_COMMITMENT_ENUM = [
      INITIAL_COMMITMENT_HOUR = 'HOUR'.freeze,
      INITIAL_COMMITMENT_MONTH = 'MONTH'.freeze,
      INITIAL_COMMITMENT_ONE_YEAR = 'ONE_YEAR'.freeze,
      INITIAL_COMMITMENT_THREE_YEARS = 'THREE_YEARS'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the SDDC that the
    # Cluster belongs to.
    #
    # @return [String]
    attr_accessor :sddc_id

    # **[Required]** The availability domain to create the Cluster's ESXi hosts in. For multi-AD Cluster deployment, set to `multi-AD`.
    #
    # @return [String]
    attr_accessor :compute_availability_domain

    # A descriptive name for the Cluster.
    # Cluster name requirements are 1-16 character length limit, Must start with a letter, Must be English letters, numbers, - only, No repeating hyphens, Must be unique within the region.
    # Avoid entering confidential information.
    #
    # @return [String]
    attr_accessor :display_name

    # A prefix used in the name of each ESXi host and Compute instance in the Cluster.
    # If this isn't set, the Cluster's `displayName` is used as the prefix.
    #
    # For example, if the value is `myCluster`, the ESXi hosts are named `myCluster-1`,
    # `myCluster-2`, and so on.
    #
    # @return [String]
    attr_accessor :instance_display_name_prefix

    # **[Required]** The number of ESXi hosts to create in the Cluster. You can add more hosts later
    # (see {#create_esxi_host create_esxi_host}).
    #
    # **Note:** If you later delete EXSi hosts from a production Cluster to make SDDC
    # total host count less than 3, you are still billed for the 3 minimum recommended
    # ESXi hosts. Also, you cannot add more VMware workloads to the Cluster until the
    # SDDC again has at least 3 ESXi hosts.
    #
    # @return [Integer]
    attr_accessor :esxi_hosts_count

    # This attribute is required.
    # @return [OCI::Ocvp::Models::NetworkConfiguration]
    attr_accessor :network_configuration

    # The billing option selected during Cluster creation.
    # {#list_supported_commitments list_supported_commitments}.
    #
    # @return [String]
    attr_reader :initial_commitment

    # The CIDR block for the IP addresses that VMware VMs in the Cluster use to run application
    # workloads.
    #
    # @return [String]
    attr_accessor :workload_network_cidr

    # The initial compute shape of the Cluster's ESXi hosts.
    # {#list_supported_host_shapes list_supported_host_shapes}.
    #
    # @return [String]
    attr_accessor :initial_host_shape_name

    # The initial OCPU count of the Cluster's ESXi hosts.
    #
    # @return [Float]
    attr_accessor :initial_host_ocpu_count

    # Indicates whether shielded instance is enabled for this Cluster.
    #
    # @return [BOOLEAN]
    attr_accessor :is_shielded_instance_enabled

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Capacity Reservation.
    #
    # @return [String]
    attr_accessor :capacity_reservation_id

    # A list of datastore info for the Cluster.
    # This value is required only when `initialHostShapeName` is a standard shape.
    #
    # @return [Array<OCI::Ocvp::Models::DatastoreInfo>]
    attr_accessor :datastores

    # The VMware software bundle to install on the ESXi hosts in the Cluster. To get a list of the available versions, use
    # {#list_supported_vmware_software_versions list_supported_vmware_software_versions}.
    #
    # @return [String]
    attr_accessor :vmware_software_version

    # The ESXi software bundle to install on the ESXi hosts in the Cluster.
    # Only versions under the same vmwareSoftwareVersion and have been validate by Oracle Cloud VMware Solution will be accepted.
    # To get a list of the available versions, use
    # {#list_supported_vmware_software_versions list_supported_vmware_software_versions}.
    #
    # @return [String]
    attr_accessor :esxi_software_version

    # Free-form tags for this resource. Each tag is a simple key-value pair with no
    # predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a
    # namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'sddc_id': :'sddcId',
        'compute_availability_domain': :'computeAvailabilityDomain',
        'display_name': :'displayName',
        'instance_display_name_prefix': :'instanceDisplayNamePrefix',
        'esxi_hosts_count': :'esxiHostsCount',
        'network_configuration': :'networkConfiguration',
        'initial_commitment': :'initialCommitment',
        'workload_network_cidr': :'workloadNetworkCidr',
        'initial_host_shape_name': :'initialHostShapeName',
        'initial_host_ocpu_count': :'initialHostOcpuCount',
        'is_shielded_instance_enabled': :'isShieldedInstanceEnabled',
        'capacity_reservation_id': :'capacityReservationId',
        'datastores': :'datastores',
        'vmware_software_version': :'vmwareSoftwareVersion',
        'esxi_software_version': :'esxiSoftwareVersion',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'sddc_id': :'String',
        'compute_availability_domain': :'String',
        'display_name': :'String',
        'instance_display_name_prefix': :'String',
        'esxi_hosts_count': :'Integer',
        'network_configuration': :'OCI::Ocvp::Models::NetworkConfiguration',
        'initial_commitment': :'String',
        'workload_network_cidr': :'String',
        'initial_host_shape_name': :'String',
        'initial_host_ocpu_count': :'Float',
        'is_shielded_instance_enabled': :'BOOLEAN',
        'capacity_reservation_id': :'String',
        'datastores': :'Array<OCI::Ocvp::Models::DatastoreInfo>',
        'vmware_software_version': :'String',
        'esxi_software_version': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :sddc_id The value to assign to the {#sddc_id} property
    # @option attributes [String] :compute_availability_domain The value to assign to the {#compute_availability_domain} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :instance_display_name_prefix The value to assign to the {#instance_display_name_prefix} property
    # @option attributes [Integer] :esxi_hosts_count The value to assign to the {#esxi_hosts_count} property
    # @option attributes [OCI::Ocvp::Models::NetworkConfiguration] :network_configuration The value to assign to the {#network_configuration} property
    # @option attributes [String] :initial_commitment The value to assign to the {#initial_commitment} property
    # @option attributes [String] :workload_network_cidr The value to assign to the {#workload_network_cidr} property
    # @option attributes [String] :initial_host_shape_name The value to assign to the {#initial_host_shape_name} property
    # @option attributes [Float] :initial_host_ocpu_count The value to assign to the {#initial_host_ocpu_count} property
    # @option attributes [BOOLEAN] :is_shielded_instance_enabled The value to assign to the {#is_shielded_instance_enabled} property
    # @option attributes [String] :capacity_reservation_id The value to assign to the {#capacity_reservation_id} property
    # @option attributes [Array<OCI::Ocvp::Models::DatastoreInfo>] :datastores The value to assign to the {#datastores} property
    # @option attributes [String] :vmware_software_version The value to assign to the {#vmware_software_version} property
    # @option attributes [String] :esxi_software_version The value to assign to the {#esxi_software_version} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.sddc_id = attributes[:'sddcId'] if attributes[:'sddcId']

      raise 'You cannot provide both :sddcId and :sddc_id' if attributes.key?(:'sddcId') && attributes.key?(:'sddc_id')

      self.sddc_id = attributes[:'sddc_id'] if attributes[:'sddc_id']

      self.compute_availability_domain = attributes[:'computeAvailabilityDomain'] if attributes[:'computeAvailabilityDomain']

      raise 'You cannot provide both :computeAvailabilityDomain and :compute_availability_domain' if attributes.key?(:'computeAvailabilityDomain') && attributes.key?(:'compute_availability_domain')

      self.compute_availability_domain = attributes[:'compute_availability_domain'] if attributes[:'compute_availability_domain']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.instance_display_name_prefix = attributes[:'instanceDisplayNamePrefix'] if attributes[:'instanceDisplayNamePrefix']

      raise 'You cannot provide both :instanceDisplayNamePrefix and :instance_display_name_prefix' if attributes.key?(:'instanceDisplayNamePrefix') && attributes.key?(:'instance_display_name_prefix')

      self.instance_display_name_prefix = attributes[:'instance_display_name_prefix'] if attributes[:'instance_display_name_prefix']

      self.esxi_hosts_count = attributes[:'esxiHostsCount'] if attributes[:'esxiHostsCount']

      raise 'You cannot provide both :esxiHostsCount and :esxi_hosts_count' if attributes.key?(:'esxiHostsCount') && attributes.key?(:'esxi_hosts_count')

      self.esxi_hosts_count = attributes[:'esxi_hosts_count'] if attributes[:'esxi_hosts_count']

      self.network_configuration = attributes[:'networkConfiguration'] if attributes[:'networkConfiguration']

      raise 'You cannot provide both :networkConfiguration and :network_configuration' if attributes.key?(:'networkConfiguration') && attributes.key?(:'network_configuration')

      self.network_configuration = attributes[:'network_configuration'] if attributes[:'network_configuration']

      self.initial_commitment = attributes[:'initialCommitment'] if attributes[:'initialCommitment']
      self.initial_commitment = "HOUR" if initial_commitment.nil? && !attributes.key?(:'initialCommitment') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :initialCommitment and :initial_commitment' if attributes.key?(:'initialCommitment') && attributes.key?(:'initial_commitment')

      self.initial_commitment = attributes[:'initial_commitment'] if attributes[:'initial_commitment']
      self.initial_commitment = "HOUR" if initial_commitment.nil? && !attributes.key?(:'initialCommitment') && !attributes.key?(:'initial_commitment') # rubocop:disable Style/StringLiterals

      self.workload_network_cidr = attributes[:'workloadNetworkCidr'] if attributes[:'workloadNetworkCidr']

      raise 'You cannot provide both :workloadNetworkCidr and :workload_network_cidr' if attributes.key?(:'workloadNetworkCidr') && attributes.key?(:'workload_network_cidr')

      self.workload_network_cidr = attributes[:'workload_network_cidr'] if attributes[:'workload_network_cidr']

      self.initial_host_shape_name = attributes[:'initialHostShapeName'] if attributes[:'initialHostShapeName']
      self.initial_host_shape_name = "BM.DenseIO2.52" if initial_host_shape_name.nil? && !attributes.key?(:'initialHostShapeName') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :initialHostShapeName and :initial_host_shape_name' if attributes.key?(:'initialHostShapeName') && attributes.key?(:'initial_host_shape_name')

      self.initial_host_shape_name = attributes[:'initial_host_shape_name'] if attributes[:'initial_host_shape_name']
      self.initial_host_shape_name = "BM.DenseIO2.52" if initial_host_shape_name.nil? && !attributes.key?(:'initialHostShapeName') && !attributes.key?(:'initial_host_shape_name') # rubocop:disable Style/StringLiterals

      self.initial_host_ocpu_count = attributes[:'initialHostOcpuCount'] if attributes[:'initialHostOcpuCount']

      raise 'You cannot provide both :initialHostOcpuCount and :initial_host_ocpu_count' if attributes.key?(:'initialHostOcpuCount') && attributes.key?(:'initial_host_ocpu_count')

      self.initial_host_ocpu_count = attributes[:'initial_host_ocpu_count'] if attributes[:'initial_host_ocpu_count']

      self.is_shielded_instance_enabled = attributes[:'isShieldedInstanceEnabled'] unless attributes[:'isShieldedInstanceEnabled'].nil?
      self.is_shielded_instance_enabled = false if is_shielded_instance_enabled.nil? && !attributes.key?(:'isShieldedInstanceEnabled') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isShieldedInstanceEnabled and :is_shielded_instance_enabled' if attributes.key?(:'isShieldedInstanceEnabled') && attributes.key?(:'is_shielded_instance_enabled')

      self.is_shielded_instance_enabled = attributes[:'is_shielded_instance_enabled'] unless attributes[:'is_shielded_instance_enabled'].nil?
      self.is_shielded_instance_enabled = false if is_shielded_instance_enabled.nil? && !attributes.key?(:'isShieldedInstanceEnabled') && !attributes.key?(:'is_shielded_instance_enabled') # rubocop:disable Style/StringLiterals

      self.capacity_reservation_id = attributes[:'capacityReservationId'] if attributes[:'capacityReservationId']

      raise 'You cannot provide both :capacityReservationId and :capacity_reservation_id' if attributes.key?(:'capacityReservationId') && attributes.key?(:'capacity_reservation_id')

      self.capacity_reservation_id = attributes[:'capacity_reservation_id'] if attributes[:'capacity_reservation_id']

      self.datastores = attributes[:'datastores'] if attributes[:'datastores']

      self.vmware_software_version = attributes[:'vmwareSoftwareVersion'] if attributes[:'vmwareSoftwareVersion']

      raise 'You cannot provide both :vmwareSoftwareVersion and :vmware_software_version' if attributes.key?(:'vmwareSoftwareVersion') && attributes.key?(:'vmware_software_version')

      self.vmware_software_version = attributes[:'vmware_software_version'] if attributes[:'vmware_software_version']

      self.esxi_software_version = attributes[:'esxiSoftwareVersion'] if attributes[:'esxiSoftwareVersion']

      raise 'You cannot provide both :esxiSoftwareVersion and :esxi_software_version' if attributes.key?(:'esxiSoftwareVersion') && attributes.key?(:'esxi_software_version')

      self.esxi_software_version = attributes[:'esxi_software_version'] if attributes[:'esxi_software_version']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] initial_commitment Object to be assigned
    def initial_commitment=(initial_commitment)
      raise "Invalid value for 'initial_commitment': this must be one of the values in INITIAL_COMMITMENT_ENUM." if initial_commitment && !INITIAL_COMMITMENT_ENUM.include?(initial_commitment)

      @initial_commitment = initial_commitment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        sddc_id == other.sddc_id &&
        compute_availability_domain == other.compute_availability_domain &&
        display_name == other.display_name &&
        instance_display_name_prefix == other.instance_display_name_prefix &&
        esxi_hosts_count == other.esxi_hosts_count &&
        network_configuration == other.network_configuration &&
        initial_commitment == other.initial_commitment &&
        workload_network_cidr == other.workload_network_cidr &&
        initial_host_shape_name == other.initial_host_shape_name &&
        initial_host_ocpu_count == other.initial_host_ocpu_count &&
        is_shielded_instance_enabled == other.is_shielded_instance_enabled &&
        capacity_reservation_id == other.capacity_reservation_id &&
        datastores == other.datastores &&
        vmware_software_version == other.vmware_software_version &&
        esxi_software_version == other.esxi_software_version &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [sddc_id, compute_availability_domain, display_name, instance_display_name_prefix, esxi_hosts_count, network_configuration, initial_commitment, workload_network_cidr, initial_host_shape_name, initial_host_ocpu_count, is_shielded_instance_enabled, capacity_reservation_id, datastores, vmware_software_version, esxi_software_version, freeform_tags, defined_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
