# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200630
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Information required to form and execute query on a data object.
  # This class has direct subclasses. If you are using this class as input to a service operations then you should favor using a subclass over the base class
  class Opsi::Models::DataObjectQuery
    QUERY_TYPE_ENUM = [
      QUERY_TYPE_TEMPLATIZED_QUERY = 'TEMPLATIZED_QUERY'.freeze,
      QUERY_TYPE_STANDARD_QUERY = 'STANDARD_QUERY'.freeze
    ].freeze

    # **[Required]** Type of Query
    # @return [String]
    attr_reader :query_type

    # List of bind parameters to be applied in the query.
    # @return [Array<OCI::Opsi::Models::DataObjectBindParameter>]
    attr_accessor :bind_params

    # Timeout (in seconds) to be set for the data object query execution.
    # @return [Float]
    attr_accessor :query_execution_timeout_in_seconds

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'query_type': :'queryType',
        'bind_params': :'bindParams',
        'query_execution_timeout_in_seconds': :'queryExecutionTimeoutInSeconds'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'query_type': :'String',
        'bind_params': :'Array<OCI::Opsi::Models::DataObjectBindParameter>',
        'query_execution_timeout_in_seconds': :'Float'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize


    # Given the hash representation of a subtype of this class,
    # use the info in the hash to return the class of the subtype.
    def self.get_subtype(object_hash)
      type = object_hash[:'queryType'] # rubocop:disable Style/SymbolLiteral

      return 'OCI::Opsi::Models::DataObjectStandardQuery' if type == 'STANDARD_QUERY'
      return 'OCI::Opsi::Models::DataObjectTemplatizedQuery' if type == 'TEMPLATIZED_QUERY'

      # TODO: Log a warning when the subtype is not found.
      'OCI::Opsi::Models::DataObjectQuery'
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :query_type The value to assign to the {#query_type} property
    # @option attributes [Array<OCI::Opsi::Models::DataObjectBindParameter>] :bind_params The value to assign to the {#bind_params} property
    # @option attributes [Float] :query_execution_timeout_in_seconds The value to assign to the {#query_execution_timeout_in_seconds} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.query_type = attributes[:'queryType'] if attributes[:'queryType']

      raise 'You cannot provide both :queryType and :query_type' if attributes.key?(:'queryType') && attributes.key?(:'query_type')

      self.query_type = attributes[:'query_type'] if attributes[:'query_type']

      self.bind_params = attributes[:'bindParams'] if attributes[:'bindParams']

      raise 'You cannot provide both :bindParams and :bind_params' if attributes.key?(:'bindParams') && attributes.key?(:'bind_params')

      self.bind_params = attributes[:'bind_params'] if attributes[:'bind_params']

      self.query_execution_timeout_in_seconds = attributes[:'queryExecutionTimeoutInSeconds'] if attributes[:'queryExecutionTimeoutInSeconds']

      raise 'You cannot provide both :queryExecutionTimeoutInSeconds and :query_execution_timeout_in_seconds' if attributes.key?(:'queryExecutionTimeoutInSeconds') && attributes.key?(:'query_execution_timeout_in_seconds')

      self.query_execution_timeout_in_seconds = attributes[:'query_execution_timeout_in_seconds'] if attributes[:'query_execution_timeout_in_seconds']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] query_type Object to be assigned
    def query_type=(query_type)
      raise "Invalid value for 'query_type': this must be one of the values in QUERY_TYPE_ENUM." if query_type && !QUERY_TYPE_ENUM.include?(query_type)

      @query_type = query_type
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        query_type == other.query_type &&
        bind_params == other.bind_params &&
        query_execution_timeout_in_seconds == other.query_execution_timeout_in_seconds
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [query_type, bind_params, query_execution_timeout_in_seconds].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
