# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200630
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Partial information about the VM Cluster which includes name, memory allocated etc.
  class Opsi::Models::VmClusterSummary
    # **[Required]** The name of the vm cluster.
    # @return [String]
    attr_accessor :vmcluster_name

    # The memory allocated on a vm cluster.
    # @return [Integer]
    attr_accessor :memory_allocated_in_gbs

    # The cpu allocated on a vm cluster.
    # @return [Integer]
    attr_accessor :cpu_allocated

    # The number of DB nodes on a vm cluster.
    # @return [Integer]
    attr_accessor :db_nodes_count

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'vmcluster_name': :'vmclusterName',
        'memory_allocated_in_gbs': :'memoryAllocatedInGBs',
        'cpu_allocated': :'cpuAllocated',
        'db_nodes_count': :'dbNodesCount'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'vmcluster_name': :'String',
        'memory_allocated_in_gbs': :'Integer',
        'cpu_allocated': :'Integer',
        'db_nodes_count': :'Integer'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :vmcluster_name The value to assign to the {#vmcluster_name} property
    # @option attributes [Integer] :memory_allocated_in_gbs The value to assign to the {#memory_allocated_in_gbs} property
    # @option attributes [Integer] :cpu_allocated The value to assign to the {#cpu_allocated} property
    # @option attributes [Integer] :db_nodes_count The value to assign to the {#db_nodes_count} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.vmcluster_name = attributes[:'vmclusterName'] if attributes[:'vmclusterName']

      raise 'You cannot provide both :vmclusterName and :vmcluster_name' if attributes.key?(:'vmclusterName') && attributes.key?(:'vmcluster_name')

      self.vmcluster_name = attributes[:'vmcluster_name'] if attributes[:'vmcluster_name']

      self.memory_allocated_in_gbs = attributes[:'memoryAllocatedInGBs'] if attributes[:'memoryAllocatedInGBs']

      raise 'You cannot provide both :memoryAllocatedInGBs and :memory_allocated_in_gbs' if attributes.key?(:'memoryAllocatedInGBs') && attributes.key?(:'memory_allocated_in_gbs')

      self.memory_allocated_in_gbs = attributes[:'memory_allocated_in_gbs'] if attributes[:'memory_allocated_in_gbs']

      self.cpu_allocated = attributes[:'cpuAllocated'] if attributes[:'cpuAllocated']

      raise 'You cannot provide both :cpuAllocated and :cpu_allocated' if attributes.key?(:'cpuAllocated') && attributes.key?(:'cpu_allocated')

      self.cpu_allocated = attributes[:'cpu_allocated'] if attributes[:'cpu_allocated']

      self.db_nodes_count = attributes[:'dbNodesCount'] if attributes[:'dbNodesCount']

      raise 'You cannot provide both :dbNodesCount and :db_nodes_count' if attributes.key?(:'dbNodesCount') && attributes.key?(:'db_nodes_count')

      self.db_nodes_count = attributes[:'db_nodes_count'] if attributes[:'db_nodes_count']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        vmcluster_name == other.vmcluster_name &&
        memory_allocated_in_gbs == other.memory_allocated_in_gbs &&
        cpu_allocated == other.cpu_allocated &&
        db_nodes_count == other.db_nodes_count
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [vmcluster_name, memory_allocated_in_gbs, cpu_allocated, db_nodes_count].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
