# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220901
require 'date'
require_relative 'create_software_source_details'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Provides the information used to create a versioned custom software source.
  class OsManagementHub::Models::CreateVersionedCustomSoftwareSourceDetails < OsManagementHub::Models::CreateSoftwareSourceDetails
    # **[Required]** List of vendor software sources.
    # @return [Array<OCI::OsManagementHub::Models::Id>]
    attr_accessor :vendor_software_sources

    # @return [OCI::OsManagementHub::Models::CustomSoftwareSourceFilter]
    attr_accessor :custom_software_source_filter

    # **[Required]** The version to assign to this custom software source.
    # @return [String]
    attr_accessor :software_source_version

    # Indicates whether the service should automatically resolve package dependencies when including specific packages in the software source.
    # @return [BOOLEAN]
    attr_accessor :is_auto_resolve_dependencies

    # Indicates whether the service should create the software source from a list of packages provided by the user.
    # @return [BOOLEAN]
    attr_accessor :is_created_from_package_list

    # A property used for compatibility only. It doesn't provide a complete list of packages. See {#add_packages_to_software_source_details add_packages_to_software_source_details} for providing the list of packages used to create the software source when isCreatedFromPackageList is set to true.
    # @return [Array<String>]
    attr_accessor :packages

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'compartmentId',
        'display_name': :'displayName',
        'description': :'description',
        'software_source_type': :'softwareSourceType',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'vendor_software_sources': :'vendorSoftwareSources',
        'custom_software_source_filter': :'customSoftwareSourceFilter',
        'software_source_version': :'softwareSourceVersion',
        'is_auto_resolve_dependencies': :'isAutoResolveDependencies',
        'is_created_from_package_list': :'isCreatedFromPackageList',
        'packages': :'packages'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'String',
        'display_name': :'String',
        'description': :'String',
        'software_source_type': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'vendor_software_sources': :'Array<OCI::OsManagementHub::Models::Id>',
        'custom_software_source_filter': :'OCI::OsManagementHub::Models::CustomSoftwareSourceFilter',
        'software_source_version': :'String',
        'is_auto_resolve_dependencies': :'BOOLEAN',
        'is_created_from_package_list': :'BOOLEAN',
        'packages': :'Array<String>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :compartment_id The value to assign to the {OCI::OsManagementHub::Models::CreateSoftwareSourceDetails#compartment_id #compartment_id} proprety
    # @option attributes [String] :display_name The value to assign to the {OCI::OsManagementHub::Models::CreateSoftwareSourceDetails#display_name #display_name} proprety
    # @option attributes [String] :description The value to assign to the {OCI::OsManagementHub::Models::CreateSoftwareSourceDetails#description #description} proprety
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {OCI::OsManagementHub::Models::CreateSoftwareSourceDetails#freeform_tags #freeform_tags} proprety
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {OCI::OsManagementHub::Models::CreateSoftwareSourceDetails#defined_tags #defined_tags} proprety
    # @option attributes [Array<OCI::OsManagementHub::Models::Id>] :vendor_software_sources The value to assign to the {#vendor_software_sources} property
    # @option attributes [OCI::OsManagementHub::Models::CustomSoftwareSourceFilter] :custom_software_source_filter The value to assign to the {#custom_software_source_filter} property
    # @option attributes [String] :software_source_version The value to assign to the {#software_source_version} property
    # @option attributes [BOOLEAN] :is_auto_resolve_dependencies The value to assign to the {#is_auto_resolve_dependencies} property
    # @option attributes [BOOLEAN] :is_created_from_package_list The value to assign to the {#is_created_from_package_list} property
    # @option attributes [Array<String>] :packages The value to assign to the {#packages} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['softwareSourceType'] = 'VERSIONED'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.vendor_software_sources = attributes[:'vendorSoftwareSources'] if attributes[:'vendorSoftwareSources']

      raise 'You cannot provide both :vendorSoftwareSources and :vendor_software_sources' if attributes.key?(:'vendorSoftwareSources') && attributes.key?(:'vendor_software_sources')

      self.vendor_software_sources = attributes[:'vendor_software_sources'] if attributes[:'vendor_software_sources']

      self.custom_software_source_filter = attributes[:'customSoftwareSourceFilter'] if attributes[:'customSoftwareSourceFilter']

      raise 'You cannot provide both :customSoftwareSourceFilter and :custom_software_source_filter' if attributes.key?(:'customSoftwareSourceFilter') && attributes.key?(:'custom_software_source_filter')

      self.custom_software_source_filter = attributes[:'custom_software_source_filter'] if attributes[:'custom_software_source_filter']

      self.software_source_version = attributes[:'softwareSourceVersion'] if attributes[:'softwareSourceVersion']

      raise 'You cannot provide both :softwareSourceVersion and :software_source_version' if attributes.key?(:'softwareSourceVersion') && attributes.key?(:'software_source_version')

      self.software_source_version = attributes[:'software_source_version'] if attributes[:'software_source_version']

      self.is_auto_resolve_dependencies = attributes[:'isAutoResolveDependencies'] unless attributes[:'isAutoResolveDependencies'].nil?
      self.is_auto_resolve_dependencies = false if is_auto_resolve_dependencies.nil? && !attributes.key?(:'isAutoResolveDependencies') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isAutoResolveDependencies and :is_auto_resolve_dependencies' if attributes.key?(:'isAutoResolveDependencies') && attributes.key?(:'is_auto_resolve_dependencies')

      self.is_auto_resolve_dependencies = attributes[:'is_auto_resolve_dependencies'] unless attributes[:'is_auto_resolve_dependencies'].nil?
      self.is_auto_resolve_dependencies = false if is_auto_resolve_dependencies.nil? && !attributes.key?(:'isAutoResolveDependencies') && !attributes.key?(:'is_auto_resolve_dependencies') # rubocop:disable Style/StringLiterals

      self.is_created_from_package_list = attributes[:'isCreatedFromPackageList'] unless attributes[:'isCreatedFromPackageList'].nil?
      self.is_created_from_package_list = false if is_created_from_package_list.nil? && !attributes.key?(:'isCreatedFromPackageList') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isCreatedFromPackageList and :is_created_from_package_list' if attributes.key?(:'isCreatedFromPackageList') && attributes.key?(:'is_created_from_package_list')

      self.is_created_from_package_list = attributes[:'is_created_from_package_list'] unless attributes[:'is_created_from_package_list'].nil?
      self.is_created_from_package_list = false if is_created_from_package_list.nil? && !attributes.key?(:'isCreatedFromPackageList') && !attributes.key?(:'is_created_from_package_list') # rubocop:disable Style/StringLiterals

      self.packages = attributes[:'packages'] if attributes[:'packages']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        compartment_id == other.compartment_id &&
        display_name == other.display_name &&
        description == other.description &&
        software_source_type == other.software_source_type &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        vendor_software_sources == other.vendor_software_sources &&
        custom_software_source_filter == other.custom_software_source_filter &&
        software_source_version == other.software_source_version &&
        is_auto_resolve_dependencies == other.is_auto_resolve_dependencies &&
        is_created_from_package_list == other.is_created_from_package_list &&
        packages == other.packages
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [compartment_id, display_name, description, software_source_type, freeform_tags, defined_tags, vendor_software_sources, custom_software_source_filter, software_source_version, is_auto_resolve_dependencies, is_created_from_package_list, packages].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
