# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220901
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Provides summary information for a software package.
  class OsManagementHub::Models::SoftwarePackageSummary
    ARCHITECTURE_ENUM = [
      ARCHITECTURE_I386 = 'I386'.freeze,
      ARCHITECTURE_I686 = 'I686'.freeze,
      ARCHITECTURE_AARCH64 = 'AARCH64'.freeze,
      ARCHITECTURE_X86_64 = 'X86_64'.freeze,
      ARCHITECTURE_SRC = 'SRC'.freeze,
      ARCHITECTURE_NOARCH = 'NOARCH'.freeze,
      ARCHITECTURE_OTHER = 'OTHER'.freeze,
      ARCHITECTURE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Package name.
    # @return [String]
    attr_accessor :display_name

    # **[Required]** Unique identifier for the package. Note that this is not an OCID.
    # @return [String]
    attr_accessor :name

    # **[Required]** Type of the package.
    # @return [String]
    attr_accessor :type

    # **[Required]** Version of the package.
    # @return [String]
    attr_accessor :version

    # The architecture for which this software was built.
    # @return [String]
    attr_reader :architecture

    # Checksum of the package.
    # @return [String]
    attr_accessor :checksum

    # Type of the checksum.
    # @return [String]
    attr_accessor :checksum_type

    # Indicates whether this package is the latest version.
    # @return [BOOLEAN]
    attr_accessor :is_latest

    # List of software sources that provide the software package. This property is deprecated and it will be removed in a future API release.
    # @return [Array<OCI::OsManagementHub::Models::SoftwareSourceDetails>]
    attr_accessor :software_sources

    # The OS families the package belongs to.
    # @return [Array<OCI::OsManagementHub::Models::OsFamily>]
    attr_accessor :os_families

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'display_name': :'displayName',
        'name': :'name',
        'type': :'type',
        'version': :'version',
        'architecture': :'architecture',
        'checksum': :'checksum',
        'checksum_type': :'checksumType',
        'is_latest': :'isLatest',
        'software_sources': :'softwareSources',
        'os_families': :'osFamilies'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'display_name': :'String',
        'name': :'String',
        'type': :'String',
        'version': :'String',
        'architecture': :'String',
        'checksum': :'String',
        'checksum_type': :'String',
        'is_latest': :'BOOLEAN',
        'software_sources': :'Array<OCI::OsManagementHub::Models::SoftwareSourceDetails>',
        'os_families': :'Array<OCI::OsManagementHub::Models::OsFamily>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [String] :type The value to assign to the {#type} property
    # @option attributes [String] :version The value to assign to the {#version} property
    # @option attributes [String] :architecture The value to assign to the {#architecture} property
    # @option attributes [String] :checksum The value to assign to the {#checksum} property
    # @option attributes [String] :checksum_type The value to assign to the {#checksum_type} property
    # @option attributes [BOOLEAN] :is_latest The value to assign to the {#is_latest} property
    # @option attributes [Array<OCI::OsManagementHub::Models::SoftwareSourceDetails>] :software_sources The value to assign to the {#software_sources} property
    # @option attributes [Array<OCI::OsManagementHub::Models::OsFamily>] :os_families The value to assign to the {#os_families} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.name = attributes[:'name'] if attributes[:'name']

      self.type = attributes[:'type'] if attributes[:'type']

      self.version = attributes[:'version'] if attributes[:'version']

      self.architecture = attributes[:'architecture'] if attributes[:'architecture']

      self.checksum = attributes[:'checksum'] if attributes[:'checksum']

      self.checksum_type = attributes[:'checksumType'] if attributes[:'checksumType']

      raise 'You cannot provide both :checksumType and :checksum_type' if attributes.key?(:'checksumType') && attributes.key?(:'checksum_type')

      self.checksum_type = attributes[:'checksum_type'] if attributes[:'checksum_type']

      self.is_latest = attributes[:'isLatest'] unless attributes[:'isLatest'].nil?

      raise 'You cannot provide both :isLatest and :is_latest' if attributes.key?(:'isLatest') && attributes.key?(:'is_latest')

      self.is_latest = attributes[:'is_latest'] unless attributes[:'is_latest'].nil?

      self.software_sources = attributes[:'softwareSources'] if attributes[:'softwareSources']

      raise 'You cannot provide both :softwareSources and :software_sources' if attributes.key?(:'softwareSources') && attributes.key?(:'software_sources')

      self.software_sources = attributes[:'software_sources'] if attributes[:'software_sources']

      self.os_families = attributes[:'osFamilies'] if attributes[:'osFamilies']

      raise 'You cannot provide both :osFamilies and :os_families' if attributes.key?(:'osFamilies') && attributes.key?(:'os_families')

      self.os_families = attributes[:'os_families'] if attributes[:'os_families']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] architecture Object to be assigned
    def architecture=(architecture)
      # rubocop:disable Style/ConditionalAssignment
      if architecture && !ARCHITECTURE_ENUM.include?(architecture)
        OCI.logger.debug("Unknown value for 'architecture' [" + architecture + "]. Mapping to 'ARCHITECTURE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @architecture = ARCHITECTURE_UNKNOWN_ENUM_VALUE
      else
        @architecture = architecture
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        display_name == other.display_name &&
        name == other.name &&
        type == other.type &&
        version == other.version &&
        architecture == other.architecture &&
        checksum == other.checksum &&
        checksum_type == other.checksum_type &&
        is_latest == other.is_latest &&
        software_sources == other.software_sources &&
        os_families == other.os_families
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [display_name, name, type, version, architecture, checksum, checksum_type, is_latest, software_sources, os_families].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
