# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220915
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Storage details of the database system.
  # This class has direct subclasses. If you are using this class as input to a service operations then you should favor using a subclass over the base class
  class Psql::Models::StorageDetails
    # **[Required]** Type of the database system.
    # @return [String]
    attr_accessor :system_type

    # **[Required]** Specifies if the block volume used for the database system is regional or AD-local.
    # If not specified, it will be set to false.
    # If `isRegionallyDurable` is set to true, `availabilityDomain` should not be specified.
    # If `isRegionallyDurable` is set to false, `availabilityDomain` must be specified.
    #
    # @return [BOOLEAN]
    attr_accessor :is_regionally_durable

    # Specifies the availability domain of AD-local storage.
    # If `isRegionallyDurable` is set to true, `availabilityDomain` should not be specified.
    # If `isRegionallyDurable` is set to false, `availabilityDomain` must be specified.
    #
    # @return [String]
    attr_accessor :availability_domain

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'system_type': :'systemType',
        'is_regionally_durable': :'isRegionallyDurable',
        'availability_domain': :'availabilityDomain'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'system_type': :'String',
        'is_regionally_durable': :'BOOLEAN',
        'availability_domain': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize


    # Given the hash representation of a subtype of this class,
    # use the info in the hash to return the class of the subtype.
    def self.get_subtype(object_hash)
      type = object_hash[:'systemType'] # rubocop:disable Style/SymbolLiteral

      return 'OCI::Psql::Models::OciOptimizedStorageDetails' if type == 'OCI_OPTIMIZED_STORAGE'

      # TODO: Log a warning when the subtype is not found.
      'OCI::Psql::Models::StorageDetails'
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :system_type The value to assign to the {#system_type} property
    # @option attributes [BOOLEAN] :is_regionally_durable The value to assign to the {#is_regionally_durable} property
    # @option attributes [String] :availability_domain The value to assign to the {#availability_domain} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.system_type = attributes[:'systemType'] if attributes[:'systemType']

      raise 'You cannot provide both :systemType and :system_type' if attributes.key?(:'systemType') && attributes.key?(:'system_type')

      self.system_type = attributes[:'system_type'] if attributes[:'system_type']

      self.is_regionally_durable = attributes[:'isRegionallyDurable'] unless attributes[:'isRegionallyDurable'].nil?
      self.is_regionally_durable = false if is_regionally_durable.nil? && !attributes.key?(:'isRegionallyDurable') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isRegionallyDurable and :is_regionally_durable' if attributes.key?(:'isRegionallyDurable') && attributes.key?(:'is_regionally_durable')

      self.is_regionally_durable = attributes[:'is_regionally_durable'] unless attributes[:'is_regionally_durable'].nil?
      self.is_regionally_durable = false if is_regionally_durable.nil? && !attributes.key?(:'isRegionallyDurable') && !attributes.key?(:'is_regionally_durable') # rubocop:disable Style/StringLiterals

      self.availability_domain = attributes[:'availabilityDomain'] if attributes[:'availabilityDomain']

      raise 'You cannot provide both :availabilityDomain and :availability_domain' if attributes.key?(:'availabilityDomain') && attributes.key?(:'availability_domain')

      self.availability_domain = attributes[:'availability_domain'] if attributes[:'availability_domain']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        system_type == other.system_type &&
        is_regionally_durable == other.is_regionally_durable &&
        availability_domain == other.availability_domain
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [system_type, is_regionally_durable, availability_domain].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
