# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220315
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Summary of information about a Redis cluster. A Redis cluster is a memory-based storage solution. For more information, see [OCI Caching Service with Redis](https://docs.cloud.oracle.com/iaas/Content/redis/home.htm).
  #
  class Redis::Models::RedisClusterSummary
    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm#Oracle) of the Redis cluster.
    # @return [String]
    attr_accessor :id

    # **[Required]** A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information.
    # @return [String]
    attr_accessor :display_name

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm#Oracle) of the compartment that contains the Redis cluster.
    # @return [String]
    attr_accessor :compartment_id

    # The current state of the Redis cluster.
    # @return [String]
    attr_accessor :lifecycle_state

    # A message describing the current state in more detail. For example, the message might provide actionable information for a resource in `FAILED` state.
    # @return [String]
    attr_accessor :lifecycle_details

    # **[Required]** The number of nodes in the Redis cluster.
    # @return [Integer]
    attr_accessor :node_count

    # **[Required]** The amount of memory allocated to the Redis cluster's nodes, in gigabytes.
    # @return [Float]
    attr_accessor :node_memory_in_gbs

    # **[Required]** The fully qualified domain name (FQDN) of the API endpoint for the Redis cluster's primary node.
    # @return [String]
    attr_accessor :primary_fqdn

    # **[Required]** The private IP address of the API endpoint for the Redis cluster's primary node.
    # @return [String]
    attr_accessor :primary_endpoint_ip_address

    # **[Required]** The fully qualified domain name (FQDN) of the API endpoint for the Redis cluster's replica nodes.
    # @return [String]
    attr_accessor :replicas_fqdn

    # **[Required]** The private IP address of the API endpoint for the Redis cluster's replica nodes.
    # @return [String]
    attr_accessor :replicas_endpoint_ip_address

    # **[Required]** The Redis version that the cluster is running.
    # @return [String]
    attr_accessor :software_version

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm#Oracle) of the Redis cluster's subnet.
    # @return [String]
    attr_accessor :subnet_id

    # The date and time the Redis cluster was created. An [RFC3339](https://datatracker.ietf.org/doc/html/rfc3339) formatted datetime string.
    # @return [DateTime]
    attr_accessor :time_created

    # The date and time the Redis cluster was updated. An [RFC3339](https://datatracker.ietf.org/doc/html/rfc3339) formatted datetime string.
    # @return [DateTime]
    attr_accessor :time_updated

    # OCIDs of the NSGs to control access in the customer network
    # @return [Array<String>]
    attr_accessor :nsg_ids

    # Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
    # Example: `{\"bar-key\": \"value\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Usage of system tag keys. These predefined keys are scoped to namespaces.
    # Example: `{\"orcl-cloud\": {\"free-tier-retained\": \"true\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :system_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'display_name': :'displayName',
        'compartment_id': :'compartmentId',
        'lifecycle_state': :'lifecycleState',
        'lifecycle_details': :'lifecycleDetails',
        'node_count': :'nodeCount',
        'node_memory_in_gbs': :'nodeMemoryInGBs',
        'primary_fqdn': :'primaryFqdn',
        'primary_endpoint_ip_address': :'primaryEndpointIpAddress',
        'replicas_fqdn': :'replicasFqdn',
        'replicas_endpoint_ip_address': :'replicasEndpointIpAddress',
        'software_version': :'softwareVersion',
        'subnet_id': :'subnetId',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'nsg_ids': :'nsgIds',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'system_tags': :'systemTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'display_name': :'String',
        'compartment_id': :'String',
        'lifecycle_state': :'String',
        'lifecycle_details': :'String',
        'node_count': :'Integer',
        'node_memory_in_gbs': :'Float',
        'primary_fqdn': :'String',
        'primary_endpoint_ip_address': :'String',
        'replicas_fqdn': :'String',
        'replicas_endpoint_ip_address': :'String',
        'software_version': :'String',
        'subnet_id': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'nsg_ids': :'Array<String>',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'system_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :lifecycle_details The value to assign to the {#lifecycle_details} property
    # @option attributes [Integer] :node_count The value to assign to the {#node_count} property
    # @option attributes [Float] :node_memory_in_gbs The value to assign to the {#node_memory_in_gbs} property
    # @option attributes [String] :primary_fqdn The value to assign to the {#primary_fqdn} property
    # @option attributes [String] :primary_endpoint_ip_address The value to assign to the {#primary_endpoint_ip_address} property
    # @option attributes [String] :replicas_fqdn The value to assign to the {#replicas_fqdn} property
    # @option attributes [String] :replicas_endpoint_ip_address The value to assign to the {#replicas_endpoint_ip_address} property
    # @option attributes [String] :software_version The value to assign to the {#software_version} property
    # @option attributes [String] :subnet_id The value to assign to the {#subnet_id} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [Array<String>] :nsg_ids The value to assign to the {#nsg_ids} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :system_tags The value to assign to the {#system_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.lifecycle_details = attributes[:'lifecycleDetails'] if attributes[:'lifecycleDetails']

      raise 'You cannot provide both :lifecycleDetails and :lifecycle_details' if attributes.key?(:'lifecycleDetails') && attributes.key?(:'lifecycle_details')

      self.lifecycle_details = attributes[:'lifecycle_details'] if attributes[:'lifecycle_details']

      self.node_count = attributes[:'nodeCount'] if attributes[:'nodeCount']

      raise 'You cannot provide both :nodeCount and :node_count' if attributes.key?(:'nodeCount') && attributes.key?(:'node_count')

      self.node_count = attributes[:'node_count'] if attributes[:'node_count']

      self.node_memory_in_gbs = attributes[:'nodeMemoryInGBs'] if attributes[:'nodeMemoryInGBs']

      raise 'You cannot provide both :nodeMemoryInGBs and :node_memory_in_gbs' if attributes.key?(:'nodeMemoryInGBs') && attributes.key?(:'node_memory_in_gbs')

      self.node_memory_in_gbs = attributes[:'node_memory_in_gbs'] if attributes[:'node_memory_in_gbs']

      self.primary_fqdn = attributes[:'primaryFqdn'] if attributes[:'primaryFqdn']

      raise 'You cannot provide both :primaryFqdn and :primary_fqdn' if attributes.key?(:'primaryFqdn') && attributes.key?(:'primary_fqdn')

      self.primary_fqdn = attributes[:'primary_fqdn'] if attributes[:'primary_fqdn']

      self.primary_endpoint_ip_address = attributes[:'primaryEndpointIpAddress'] if attributes[:'primaryEndpointIpAddress']

      raise 'You cannot provide both :primaryEndpointIpAddress and :primary_endpoint_ip_address' if attributes.key?(:'primaryEndpointIpAddress') && attributes.key?(:'primary_endpoint_ip_address')

      self.primary_endpoint_ip_address = attributes[:'primary_endpoint_ip_address'] if attributes[:'primary_endpoint_ip_address']

      self.replicas_fqdn = attributes[:'replicasFqdn'] if attributes[:'replicasFqdn']

      raise 'You cannot provide both :replicasFqdn and :replicas_fqdn' if attributes.key?(:'replicasFqdn') && attributes.key?(:'replicas_fqdn')

      self.replicas_fqdn = attributes[:'replicas_fqdn'] if attributes[:'replicas_fqdn']

      self.replicas_endpoint_ip_address = attributes[:'replicasEndpointIpAddress'] if attributes[:'replicasEndpointIpAddress']

      raise 'You cannot provide both :replicasEndpointIpAddress and :replicas_endpoint_ip_address' if attributes.key?(:'replicasEndpointIpAddress') && attributes.key?(:'replicas_endpoint_ip_address')

      self.replicas_endpoint_ip_address = attributes[:'replicas_endpoint_ip_address'] if attributes[:'replicas_endpoint_ip_address']

      self.software_version = attributes[:'softwareVersion'] if attributes[:'softwareVersion']

      raise 'You cannot provide both :softwareVersion and :software_version' if attributes.key?(:'softwareVersion') && attributes.key?(:'software_version')

      self.software_version = attributes[:'software_version'] if attributes[:'software_version']

      self.subnet_id = attributes[:'subnetId'] if attributes[:'subnetId']

      raise 'You cannot provide both :subnetId and :subnet_id' if attributes.key?(:'subnetId') && attributes.key?(:'subnet_id')

      self.subnet_id = attributes[:'subnet_id'] if attributes[:'subnet_id']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.nsg_ids = attributes[:'nsgIds'] if attributes[:'nsgIds']

      raise 'You cannot provide both :nsgIds and :nsg_ids' if attributes.key?(:'nsgIds') && attributes.key?(:'nsg_ids')

      self.nsg_ids = attributes[:'nsg_ids'] if attributes[:'nsg_ids']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.system_tags = attributes[:'systemTags'] if attributes[:'systemTags']

      raise 'You cannot provide both :systemTags and :system_tags' if attributes.key?(:'systemTags') && attributes.key?(:'system_tags')

      self.system_tags = attributes[:'system_tags'] if attributes[:'system_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        display_name == other.display_name &&
        compartment_id == other.compartment_id &&
        lifecycle_state == other.lifecycle_state &&
        lifecycle_details == other.lifecycle_details &&
        node_count == other.node_count &&
        node_memory_in_gbs == other.node_memory_in_gbs &&
        primary_fqdn == other.primary_fqdn &&
        primary_endpoint_ip_address == other.primary_endpoint_ip_address &&
        replicas_fqdn == other.replicas_fqdn &&
        replicas_endpoint_ip_address == other.replicas_endpoint_ip_address &&
        software_version == other.software_version &&
        subnet_id == other.subnet_id &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        nsg_ids == other.nsg_ids &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        system_tags == other.system_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, display_name, compartment_id, lifecycle_state, lifecycle_details, node_count, node_memory_in_gbs, primary_fqdn, primary_endpoint_ip_address, replicas_fqdn, replicas_endpoint_ip_address, software_version, subnet_id, time_created, time_updated, nsg_ids, freeform_tags, defined_tags, system_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
