# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200909
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # A service source or service target used to create a connector.
  # Example connector plugins include the Queue source and the Notifications target.
  # For more information about flows defined by connectors, see
  # [Overview of Connector Hub](https://docs.cloud.oracle.com/iaas/Content/connector-hub/overview.htm).
  # For configuration instructions, see
  # [Creating a Connector](https://docs.cloud.oracle.com/iaas/Content/connector-hub/create-service-connector.htm).
  #
  # This class has direct subclasses. If you are using this class as input to a service operations then you should favor using a subclass over the base class
  class Sch::Models::ConnectorPlugin
    KIND_ENUM = [
      KIND_SOURCE = 'SOURCE'.freeze,
      KIND_TARGET = 'TARGET'.freeze,
      KIND_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    ESTIMATED_THROUGHPUT_ENUM = [
      ESTIMATED_THROUGHPUT_LOW = 'LOW'.freeze,
      ESTIMATED_THROUGHPUT_MEDIUM = 'MEDIUM'.freeze,
      ESTIMATED_THROUGHPUT_HIGH = 'HIGH'.freeze,
      ESTIMATED_THROUGHPUT_UNKNOWN = 'UNKNOWN'.freeze,
      ESTIMATED_THROUGHPUT_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The plugin type discriminator.
    #
    # @return [String]
    attr_reader :kind

    # **[Required]** The service to be called by the connector plugin.
    # Example: `QueueSource`
    #
    # @return [String]
    attr_accessor :name

    # **[Required]** The date and time when this plugin became available.
    # Format is defined by [RFC3339](https://tools.ietf.org/html/rfc3339).
    # Example: `2023-09-09T21:10:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :time_created

    # The estimated throughput range (LOW, MEDIUM, HIGH).
    #
    # @return [String]
    attr_reader :estimated_throughput

    # **[Required]** The current state of the service connector.
    #
    # @return [String]
    attr_reader :lifecycle_state

    # **[Required]** A user-friendly name. It does not have to be unique, and it is changeable.
    # Avoid entering confidential information.
    #
    # @return [String]
    attr_accessor :display_name

    # Gets the specified connector plugin configuration information in OpenAPI specification format.
    #
    # @return [String]
    attr_accessor :schema

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'kind': :'kind',
        'name': :'name',
        'time_created': :'timeCreated',
        'estimated_throughput': :'estimatedThroughput',
        'lifecycle_state': :'lifecycleState',
        'display_name': :'displayName',
        'schema': :'schema'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'kind': :'String',
        'name': :'String',
        'time_created': :'DateTime',
        'estimated_throughput': :'String',
        'lifecycle_state': :'String',
        'display_name': :'String',
        'schema': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize


    # Given the hash representation of a subtype of this class,
    # use the info in the hash to return the class of the subtype.
    def self.get_subtype(object_hash)
      type = object_hash[:'kind'] # rubocop:disable Style/SymbolLiteral

      return 'OCI::Sch::Models::TargetConnectorPlugin' if type == 'TARGET'
      return 'OCI::Sch::Models::SourceConnectorPlugin' if type == 'SOURCE'

      # TODO: Log a warning when the subtype is not found.
      'OCI::Sch::Models::ConnectorPlugin'
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Layout/EmptyLines, Metrics/PerceivedComplexity, Metrics/AbcSize

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :kind The value to assign to the {#kind} property
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [String] :estimated_throughput The value to assign to the {#estimated_throughput} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :schema The value to assign to the {#schema} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.kind = attributes[:'kind'] if attributes[:'kind']

      self.name = attributes[:'name'] if attributes[:'name']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.estimated_throughput = attributes[:'estimatedThroughput'] if attributes[:'estimatedThroughput']
      self.estimated_throughput = "UNKNOWN" if estimated_throughput.nil? && !attributes.key?(:'estimatedThroughput') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :estimatedThroughput and :estimated_throughput' if attributes.key?(:'estimatedThroughput') && attributes.key?(:'estimated_throughput')

      self.estimated_throughput = attributes[:'estimated_throughput'] if attributes[:'estimated_throughput']
      self.estimated_throughput = "UNKNOWN" if estimated_throughput.nil? && !attributes.key?(:'estimatedThroughput') && !attributes.key?(:'estimated_throughput') # rubocop:disable Style/StringLiterals

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.schema = attributes[:'schema'] if attributes[:'schema']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] kind Object to be assigned
    def kind=(kind)
      # rubocop:disable Style/ConditionalAssignment
      if kind && !KIND_ENUM.include?(kind)
        OCI.logger.debug("Unknown value for 'kind' [" + kind + "]. Mapping to 'KIND_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @kind = KIND_UNKNOWN_ENUM_VALUE
      else
        @kind = kind
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] estimated_throughput Object to be assigned
    def estimated_throughput=(estimated_throughput)
      # rubocop:disable Style/ConditionalAssignment
      if estimated_throughput && !ESTIMATED_THROUGHPUT_ENUM.include?(estimated_throughput)
        OCI.logger.debug("Unknown value for 'estimated_throughput' [" + estimated_throughput + "]. Mapping to 'ESTIMATED_THROUGHPUT_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @estimated_throughput = ESTIMATED_THROUGHPUT_UNKNOWN_ENUM_VALUE
      else
        @estimated_throughput = estimated_throughput
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        kind == other.kind &&
        name == other.name &&
        time_created == other.time_created &&
        estimated_throughput == other.estimated_throughput &&
        lifecycle_state == other.lifecycle_state &&
        display_name == other.display_name &&
        schema == other.schema
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [kind, name, time_created, estimated_throughput, lifecycle_state, display_name, schema].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
