# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20210330
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The request of DiscoveryJob Resource details.
  class StackMonitoring::Models::DiscoveryDetails
    RESOURCE_TYPE_ENUM = [
      RESOURCE_TYPE_WEBLOGIC_DOMAIN = 'WEBLOGIC_DOMAIN'.freeze,
      RESOURCE_TYPE_EBS_INSTANCE = 'EBS_INSTANCE'.freeze,
      RESOURCE_TYPE_SQL_SERVER = 'SQL_SERVER'.freeze,
      RESOURCE_TYPE_APACHE_TOMCAT = 'APACHE_TOMCAT'.freeze,
      RESOURCE_TYPE_ORACLE_DATABASE = 'ORACLE_DATABASE'.freeze,
      RESOURCE_TYPE_OCI_ORACLE_DB = 'OCI_ORACLE_DB'.freeze,
      RESOURCE_TYPE_OCI_ORACLE_CDB = 'OCI_ORACLE_CDB'.freeze,
      RESOURCE_TYPE_OCI_ORACLE_PDB = 'OCI_ORACLE_PDB'.freeze,
      RESOURCE_TYPE_HOST = 'HOST'.freeze,
      RESOURCE_TYPE_ORACLE_PSFT = 'ORACLE_PSFT'.freeze,
      RESOURCE_TYPE_ORACLE_MFT = 'ORACLE_MFT'.freeze,
      RESOURCE_TYPE_APACHE_HTTP_SERVER = 'APACHE_HTTP_SERVER'.freeze,
      RESOURCE_TYPE_ORACLE_GOLDENGATE = 'ORACLE_GOLDENGATE'.freeze,
      RESOURCE_TYPE_CUSTOM_RESOURCE = 'CUSTOM_RESOURCE'.freeze,
      RESOURCE_TYPE_ORACLE_HTTP_SERVER = 'ORACLE_HTTP_SERVER'.freeze,
      RESOURCE_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LICENSE_ENUM = [
      LICENSE_STANDARD_EDITION = 'STANDARD_EDITION'.freeze,
      LICENSE_ENTERPRISE_EDITION = 'ENTERPRISE_EDITION'.freeze,
      LICENSE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The OCID of Management Agent
    # @return [String]
    attr_accessor :agent_id

    # **[Required]** Resource Type.
    # @return [String]
    attr_reader :resource_type

    # **[Required]** The Name of resource type
    # @return [String]
    attr_accessor :resource_name

    # License edition of the monitored resource.
    # @return [String]
    attr_reader :license

    # This attribute is required.
    # @return [OCI::StackMonitoring::Models::PropertyDetails]
    attr_accessor :properties

    # @return [OCI::StackMonitoring::Models::CredentialCollection]
    attr_accessor :credentials

    # @return [OCI::StackMonitoring::Models::PropertyDetails]
    attr_accessor :tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'agent_id': :'agentId',
        'resource_type': :'resourceType',
        'resource_name': :'resourceName',
        'license': :'license',
        'properties': :'properties',
        'credentials': :'credentials',
        'tags': :'tags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'agent_id': :'String',
        'resource_type': :'String',
        'resource_name': :'String',
        'license': :'String',
        'properties': :'OCI::StackMonitoring::Models::PropertyDetails',
        'credentials': :'OCI::StackMonitoring::Models::CredentialCollection',
        'tags': :'OCI::StackMonitoring::Models::PropertyDetails'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :agent_id The value to assign to the {#agent_id} property
    # @option attributes [String] :resource_type The value to assign to the {#resource_type} property
    # @option attributes [String] :resource_name The value to assign to the {#resource_name} property
    # @option attributes [String] :license The value to assign to the {#license} property
    # @option attributes [OCI::StackMonitoring::Models::PropertyDetails] :properties The value to assign to the {#properties} property
    # @option attributes [OCI::StackMonitoring::Models::CredentialCollection] :credentials The value to assign to the {#credentials} property
    # @option attributes [OCI::StackMonitoring::Models::PropertyDetails] :tags The value to assign to the {#tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.agent_id = attributes[:'agentId'] if attributes[:'agentId']

      raise 'You cannot provide both :agentId and :agent_id' if attributes.key?(:'agentId') && attributes.key?(:'agent_id')

      self.agent_id = attributes[:'agent_id'] if attributes[:'agent_id']

      self.resource_type = attributes[:'resourceType'] if attributes[:'resourceType']

      raise 'You cannot provide both :resourceType and :resource_type' if attributes.key?(:'resourceType') && attributes.key?(:'resource_type')

      self.resource_type = attributes[:'resource_type'] if attributes[:'resource_type']

      self.resource_name = attributes[:'resourceName'] if attributes[:'resourceName']

      raise 'You cannot provide both :resourceName and :resource_name' if attributes.key?(:'resourceName') && attributes.key?(:'resource_name')

      self.resource_name = attributes[:'resource_name'] if attributes[:'resource_name']

      self.license = attributes[:'license'] if attributes[:'license']

      self.properties = attributes[:'properties'] if attributes[:'properties']

      self.credentials = attributes[:'credentials'] if attributes[:'credentials']

      self.tags = attributes[:'tags'] if attributes[:'tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] resource_type Object to be assigned
    def resource_type=(resource_type)
      # rubocop:disable Style/ConditionalAssignment
      if resource_type && !RESOURCE_TYPE_ENUM.include?(resource_type)
        OCI.logger.debug("Unknown value for 'resource_type' [" + resource_type + "]. Mapping to 'RESOURCE_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @resource_type = RESOURCE_TYPE_UNKNOWN_ENUM_VALUE
      else
        @resource_type = resource_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] license Object to be assigned
    def license=(license)
      # rubocop:disable Style/ConditionalAssignment
      if license && !LICENSE_ENUM.include?(license)
        OCI.logger.debug("Unknown value for 'license' [" + license + "]. Mapping to 'LICENSE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @license = LICENSE_UNKNOWN_ENUM_VALUE
      else
        @license = license
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        agent_id == other.agent_id &&
        resource_type == other.resource_type &&
        resource_name == other.resource_name &&
        license == other.license &&
        properties == other.properties &&
        credentials == other.credentials &&
        tags == other.tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [agent_id, resource_type, resource_name, license, properties, credentials, tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
