# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20230401
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Summary of the line item in an assigned subscription.
  class TenantManagerControlPlane::Models::AssignedSubscriptionLineItemSummary
    BILLING_MODEL_ENUM = [
      BILLING_MODEL_COMMITMENT = 'COMMITMENT'.freeze,
      BILLING_MODEL_PAYGO = 'PAYGO'.freeze,
      BILLING_MODEL_PROMOTION = 'PROMOTION'.freeze,
      BILLING_MODEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Subscription line item identifier.
    # @return [String]
    attr_accessor :id

    # **[Required]** Product code.
    # @return [String]
    attr_accessor :product_code

    # **[Required]** Product number.
    # @return [Float]
    attr_accessor :quantity

    # **[Required]** Billing model supported by the associated line item.
    # @return [String]
    attr_reader :billing_model

    # **[Required]** The time the subscription item and associated products should start. An RFC 3339 formatted date and time string.
    # @return [DateTime]
    attr_accessor :time_started

    # **[Required]** The time the subscription item and associated products should end. An RFC 3339 formatted date and time string.
    # @return [DateTime]
    attr_accessor :time_ended

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'product_code': :'productCode',
        'quantity': :'quantity',
        'billing_model': :'billingModel',
        'time_started': :'timeStarted',
        'time_ended': :'timeEnded'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'product_code': :'String',
        'quantity': :'Float',
        'billing_model': :'String',
        'time_started': :'DateTime',
        'time_ended': :'DateTime'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :product_code The value to assign to the {#product_code} property
    # @option attributes [Float] :quantity The value to assign to the {#quantity} property
    # @option attributes [String] :billing_model The value to assign to the {#billing_model} property
    # @option attributes [DateTime] :time_started The value to assign to the {#time_started} property
    # @option attributes [DateTime] :time_ended The value to assign to the {#time_ended} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.product_code = attributes[:'productCode'] if attributes[:'productCode']

      raise 'You cannot provide both :productCode and :product_code' if attributes.key?(:'productCode') && attributes.key?(:'product_code')

      self.product_code = attributes[:'product_code'] if attributes[:'product_code']

      self.quantity = attributes[:'quantity'] if attributes[:'quantity']

      self.billing_model = attributes[:'billingModel'] if attributes[:'billingModel']

      raise 'You cannot provide both :billingModel and :billing_model' if attributes.key?(:'billingModel') && attributes.key?(:'billing_model')

      self.billing_model = attributes[:'billing_model'] if attributes[:'billing_model']

      self.time_started = attributes[:'timeStarted'] if attributes[:'timeStarted']

      raise 'You cannot provide both :timeStarted and :time_started' if attributes.key?(:'timeStarted') && attributes.key?(:'time_started')

      self.time_started = attributes[:'time_started'] if attributes[:'time_started']

      self.time_ended = attributes[:'timeEnded'] if attributes[:'timeEnded']

      raise 'You cannot provide both :timeEnded and :time_ended' if attributes.key?(:'timeEnded') && attributes.key?(:'time_ended')

      self.time_ended = attributes[:'time_ended'] if attributes[:'time_ended']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] billing_model Object to be assigned
    def billing_model=(billing_model)
      # rubocop:disable Style/ConditionalAssignment
      if billing_model && !BILLING_MODEL_ENUM.include?(billing_model)
        OCI.logger.debug("Unknown value for 'billing_model' [" + billing_model + "]. Mapping to 'BILLING_MODEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @billing_model = BILLING_MODEL_UNKNOWN_ENUM_VALUE
      else
        @billing_model = billing_model
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        product_code == other.product_code &&
        quantity == other.quantity &&
        billing_model == other.billing_model &&
        time_started == other.time_started &&
        time_ended == other.time_ended
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, product_code, quantity, billing_model, time_started, time_ended].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
