# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220901
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # A data signature observed on a network or host that indicates a potential security threat. Indicators can be plain text or computed (hashed) values.
  class ThreatIntelligence::Models::Indicator
    TYPE_ENUM = [
      TYPE_DOMAIN_NAME = 'DOMAIN_NAME'.freeze,
      TYPE_FILE_NAME = 'FILE_NAME'.freeze,
      TYPE_MD5_HASH = 'MD5_HASH'.freeze,
      TYPE_SHA1_HASH = 'SHA1_HASH'.freeze,
      TYPE_SHA256_HASH = 'SHA256_HASH'.freeze,
      TYPE_IP_ADDRESS = 'IP_ADDRESS'.freeze,
      TYPE_URL = 'URL'.freeze,
      TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The OCID of the indicator.
    # @return [String]
    attr_accessor :id

    # **[Required]** The type of indicator.
    # @return [String]
    attr_reader :type

    # **[Required]** The value for this indicator.
    # The value's format is dependent upon its `type`. Examples:
    #
    # DOMAIN_NAME \"evil.example.com\"
    #
    # MD5_HASH \"44d88612fea8a8f36de82e1278abb02f\"
    #
    # IP_ADDRESS \"2001:db8::1\"
    #
    # @return [String]
    attr_accessor :value

    # An integer from 0 to 100 that represents how certain we are that the indicator is malicious and a potential threat if it is detected communicating with your cloud resources. This confidence value is aggregated from the confidence in the threat types, attributes, and relationships to create an overall value for the indicator.
    # @return [Integer]
    attr_accessor :confidence

    # The OCID of the compartment that contains this indicator.
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** Characteristics of the threat indicator based on previous observations or behavior. May include related tactics, techniques, and procedures.
    # @return [Array<OCI::ThreatIntelligence::Models::ThreatType>]
    attr_accessor :threat_types

    # **[Required]** A map of attributes with additional information about the indicator.
    # Each attribute has a name (string), value (string), and attribution (supporting data).
    #
    # @return [Array<OCI::ThreatIntelligence::Models::IndicatorAttribute>]
    attr_accessor :attributes

    # **[Required]** A map of relationships between the indicator and other entities.
    # Each relationship has a name (string), related entity, and attribution (supporting data).
    #
    # @return [Array<OCI::ThreatIntelligence::Models::IndicatorRelationship>]
    attr_accessor :relationships

    # The state of the indicator. It will always be `ACTIVE`.
    # @return [String]
    attr_reader :lifecycle_state

    # **[Required]** The date and time that the indicator was first detected. An RFC3339 formatted string.
    # @return [DateTime]
    attr_accessor :time_created

    # **[Required]** The date and time that this indicator was last updated. The value is the same as `timeCreated` for a new indicator. An RFC3339 formatted string.
    # @return [DateTime]
    attr_accessor :time_updated

    # **[Required]** The date and time that this indicator was last seen. The value is the same as `timeCreated` for a new indicator. An RFC3339 formatted string.
    # @return [DateTime]
    attr_accessor :time_last_seen

    # This attribute is required.
    # @return [OCI::ThreatIntelligence::Models::GeodataDetails]
    attr_accessor :geodata

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'type': :'type',
        'value': :'value',
        'confidence': :'confidence',
        'compartment_id': :'compartmentId',
        'threat_types': :'threatTypes',
        'attributes': :'attributes',
        'relationships': :'relationships',
        'lifecycle_state': :'lifecycleState',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'time_last_seen': :'timeLastSeen',
        'geodata': :'geodata'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'type': :'String',
        'value': :'String',
        'confidence': :'Integer',
        'compartment_id': :'String',
        'threat_types': :'Array<OCI::ThreatIntelligence::Models::ThreatType>',
        'attributes': :'Array<OCI::ThreatIntelligence::Models::IndicatorAttribute>',
        'relationships': :'Array<OCI::ThreatIntelligence::Models::IndicatorRelationship>',
        'lifecycle_state': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'time_last_seen': :'DateTime',
        'geodata': :'OCI::ThreatIntelligence::Models::GeodataDetails'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :type The value to assign to the {#type} property
    # @option attributes [String] :value The value to assign to the {#value} property
    # @option attributes [Integer] :confidence The value to assign to the {#confidence} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [Array<OCI::ThreatIntelligence::Models::ThreatType>] :threat_types The value to assign to the {#threat_types} property
    # @option attributes [Array<OCI::ThreatIntelligence::Models::IndicatorAttribute>] :attributes The value to assign to the {#attributes} property
    # @option attributes [Array<OCI::ThreatIntelligence::Models::IndicatorRelationship>] :relationships The value to assign to the {#relationships} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [DateTime] :time_last_seen The value to assign to the {#time_last_seen} property
    # @option attributes [OCI::ThreatIntelligence::Models::GeodataDetails] :geodata The value to assign to the {#geodata} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.type = attributes[:'type'] if attributes[:'type']

      self.value = attributes[:'value'] if attributes[:'value']

      self.confidence = attributes[:'confidence'] if attributes[:'confidence']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.threat_types = attributes[:'threatTypes'] if attributes[:'threatTypes']

      raise 'You cannot provide both :threatTypes and :threat_types' if attributes.key?(:'threatTypes') && attributes.key?(:'threat_types')

      self.threat_types = attributes[:'threat_types'] if attributes[:'threat_types']

      self.attributes = attributes[:'attributes'] if attributes[:'attributes']

      self.relationships = attributes[:'relationships'] if attributes[:'relationships']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.time_last_seen = attributes[:'timeLastSeen'] if attributes[:'timeLastSeen']

      raise 'You cannot provide both :timeLastSeen and :time_last_seen' if attributes.key?(:'timeLastSeen') && attributes.key?(:'time_last_seen')

      self.time_last_seen = attributes[:'time_last_seen'] if attributes[:'time_last_seen']

      self.geodata = attributes[:'geodata'] if attributes[:'geodata']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] type Object to be assigned
    def type=(type)
      # rubocop:disable Style/ConditionalAssignment
      if type && !TYPE_ENUM.include?(type)
        OCI.logger.debug("Unknown value for 'type' [" + type + "]. Mapping to 'TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @type = TYPE_UNKNOWN_ENUM_VALUE
      else
        @type = type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        type == other.type &&
        value == other.value &&
        confidence == other.confidence &&
        compartment_id == other.compartment_id &&
        threat_types == other.threat_types &&
        attributes == other.attributes &&
        relationships == other.relationships &&
        lifecycle_state == other.lifecycle_state &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        time_last_seen == other.time_last_seen &&
        geodata == other.geodata
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, type, value, confidence, compartment_id, threat_types, attributes, relationships, lifecycle_state, time_created, time_updated, time_last_seen, geodata].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
