# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200107

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # This class provides a wrapper around {OCI::UsageApi::UsageapiClient} and offers convenience methods
  # for operations that would otherwise need to be chained together. For example, instead of performing an action
  # on a resource (e.g. launching an instance, creating a load balancer) and then using a waiter to wait for the resource
  # to enter a given state, you can call a single method in this class to accomplish the same functionality
  class UsageApi::UsageapiClientCompositeOperations
    # The {OCI::UsageApi::UsageapiClient} used to communicate with the service_client
    #
    # @return [OCI::UsageApi::UsageapiClient]
    attr_reader :service_client

    # Initializes a new UsageapiClientCompositeOperations
    #
    # @param [OCI::UsageApi::UsageapiClient] service_client The client used to communicate with the service.
    #   Defaults to a new service client created via {OCI::UsageApi::UsageapiClient#initialize} with no arguments
    def initialize(service_client = OCI::UsageApi::UsageapiClient.new)
      @service_client = service_client
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::UsageApi::UsageapiClient#create_email_recipients_group} and then waits for the {OCI::UsageApi::Models::EmailRecipientsGroup} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::UsageApi::Models::CreateEmailRecipientsGroupDetails] create_email_recipients_group_details New email recipients group details.
    # @param [String] subscription_id The UsageStatement Subscription unique OCID.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::UsageApi::Models::EmailRecipientsGroup#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::UsageApi::UsageapiClient#create_email_recipients_group}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::UsageApi::Models::EmailRecipientsGroup}
    def create_email_recipients_group_and_wait_for_state(create_email_recipients_group_details, subscription_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.create_email_recipients_group(create_email_recipients_group_details, subscription_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_email_recipients_group(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::UsageApi::UsageapiClient#create_schedule} and then waits for the {OCI::UsageApi::Models::Schedule} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::UsageApi::Models::CreateScheduleDetails] create_schedule_details New schedule details.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::UsageApi::Models::Schedule#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::UsageApi::UsageapiClient#create_schedule}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::UsageApi::Models::Schedule}
    def create_schedule_and_wait_for_state(create_schedule_details, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.create_schedule(create_schedule_details, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_schedule(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::UsageApi::UsageapiClient#delete_schedule} and then waits for the {OCI::UsageApi::Models::Schedule} acted upon
    # to enter the given state(s).
    #
    # @param [String] schedule_id The schedule unique OCID.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::UsageApi::Models::Schedule#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::UsageApi::UsageapiClient#delete_schedule}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type nil
    def delete_schedule_and_wait_for_state(schedule_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      initial_get_result = @service_client.get_schedule(schedule_id)
      operation_result = @service_client.delete_schedule(schedule_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)

      begin
        waiter_result = initial_get_result.wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200,
          succeed_on_not_found: true
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::UsageApi::UsageapiClient#update_email_recipients_group} and then waits for the {OCI::UsageApi::Models::EmailRecipientsGroup} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::UsageApi::Models::UpdateEmailRecipientsGroupDetails] update_email_recipients_group_details The information to be updated.
    # @param [String] email_recipients_group_id The email recipient group OCID.
    # @param [String] subscription_id The UsageStatement Subscription unique OCID.
    # @param [String] compartment_id The compartment ID in which to list resources.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::UsageApi::Models::EmailRecipientsGroup#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::UsageApi::UsageapiClient#update_email_recipients_group}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::UsageApi::Models::EmailRecipientsGroup}
    def update_email_recipients_group_and_wait_for_state(update_email_recipients_group_details, email_recipients_group_id, subscription_id, compartment_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.update_email_recipients_group(update_email_recipients_group_details, email_recipients_group_id, subscription_id, compartment_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_email_recipients_group(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:disable Layout/EmptyLines


    # Calls {OCI::UsageApi::UsageapiClient#update_schedule} and then waits for the {OCI::UsageApi::Models::Schedule} acted upon
    # to enter the given state(s).
    #
    # @param [OCI::UsageApi::Models::UpdateScheduleDetails] update_schedule_details The information to be updated. Currently supports only tags
    # @param [String] schedule_id The schedule unique OCID.
    # @param [Array<String>] wait_for_states An array of states to wait on. These should be valid values for {OCI::UsageApi::Models::Schedule#lifecycle_state}
    # @param [Hash] base_operation_opts Any optional arguments accepted by {OCI::UsageApi::UsageapiClient#update_schedule}
    # @param [Hash] waiter_opts Optional arguments for the waiter. Keys should be symbols, and the following keys are supported:
    #   * max_interval_seconds: The maximum interval between queries, in seconds.
    #   * max_wait_seconds The maximum time to wait, in seconds
    #
    # @return [OCI::Response] A {OCI::Response} object with data of type {OCI::UsageApi::Models::Schedule}
    def update_schedule_and_wait_for_state(update_schedule_details, schedule_id, wait_for_states = [], base_operation_opts = {}, waiter_opts = {})
      operation_result = @service_client.update_schedule(update_schedule_details, schedule_id, base_operation_opts)

      return operation_result if wait_for_states.empty?

      lowered_wait_for_states = wait_for_states.map(&:downcase)
      wait_for_resource_id = operation_result.data.id

      begin
        waiter_result = @service_client.get_schedule(wait_for_resource_id).wait_until(
          eval_proc: ->(response) { response.data.respond_to?(:lifecycle_state) && lowered_wait_for_states.include?(response.data.lifecycle_state.downcase) },
          max_interval_seconds: waiter_opts.key?(:max_interval_seconds) ? waiter_opts[:max_interval_seconds] : 30,
          max_wait_seconds: waiter_opts.key?(:max_wait_seconds) ? waiter_opts[:max_wait_seconds] : 1200
        )
        result_to_return = waiter_result

        return result_to_return
      rescue StandardError
        raise OCI::Errors::CompositeOperationError.new(partial_results: [operation_result])
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/ParameterLists, Metrics/PerceivedComplexity
    # rubocop:enable Layout/EmptyLines
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
