# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20180608
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Defines the frequency of the rotation and the information about the target system
  class Vault::Models::RotationConfig
    # The time interval that indicates the frequency for rotating secret data, as described in ISO 8601 format.
    # The minimum value is 1 day and maximum value is 360 days.
    # For example, if you want to set the time interval for rotating a secret data as 30 days, the duration is expressed as \"P30D.\"
    #
    # @return [String]
    attr_accessor :rotation_interval

    # This attribute is required.
    # @return [OCI::Vault::Models::TargetSystemDetails]
    attr_accessor :target_system_details

    # Enables auto rotation, when set to true rotationInterval must be set.
    #
    # @return [BOOLEAN]
    attr_accessor :is_scheduled_rotation_enabled

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'rotation_interval': :'rotationInterval',
        'target_system_details': :'targetSystemDetails',
        'is_scheduled_rotation_enabled': :'isScheduledRotationEnabled'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'rotation_interval': :'String',
        'target_system_details': :'OCI::Vault::Models::TargetSystemDetails',
        'is_scheduled_rotation_enabled': :'BOOLEAN'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :rotation_interval The value to assign to the {#rotation_interval} property
    # @option attributes [OCI::Vault::Models::TargetSystemDetails] :target_system_details The value to assign to the {#target_system_details} property
    # @option attributes [BOOLEAN] :is_scheduled_rotation_enabled The value to assign to the {#is_scheduled_rotation_enabled} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.rotation_interval = attributes[:'rotationInterval'] if attributes[:'rotationInterval']

      raise 'You cannot provide both :rotationInterval and :rotation_interval' if attributes.key?(:'rotationInterval') && attributes.key?(:'rotation_interval')

      self.rotation_interval = attributes[:'rotation_interval'] if attributes[:'rotation_interval']

      self.target_system_details = attributes[:'targetSystemDetails'] if attributes[:'targetSystemDetails']

      raise 'You cannot provide both :targetSystemDetails and :target_system_details' if attributes.key?(:'targetSystemDetails') && attributes.key?(:'target_system_details')

      self.target_system_details = attributes[:'target_system_details'] if attributes[:'target_system_details']

      self.is_scheduled_rotation_enabled = attributes[:'isScheduledRotationEnabled'] unless attributes[:'isScheduledRotationEnabled'].nil?

      raise 'You cannot provide both :isScheduledRotationEnabled and :is_scheduled_rotation_enabled' if attributes.key?(:'isScheduledRotationEnabled') && attributes.key?(:'is_scheduled_rotation_enabled')

      self.is_scheduled_rotation_enabled = attributes[:'is_scheduled_rotation_enabled'] unless attributes[:'is_scheduled_rotation_enabled'].nil?
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        rotation_interval == other.rotation_interval &&
        target_system_details == other.target_system_details &&
        is_scheduled_rotation_enabled == other.is_scheduled_rotation_enabled
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [rotation_interval, target_system_details, is_scheduled_rotation_enabled].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
