# Copyright (c) 2016, 2024, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20160918
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # For use with Oracle Cloud Infrastructure FastConnect. A cross-connect represents a
  # physical connection between an existing network and Oracle. Customers who are colocated
  # with Oracle in a FastConnect location create and use cross-connects. For more
  # information, see [FastConnect Overview](https://docs.cloud.oracle.com/iaas/Content/Network/Concepts/fastconnect.htm).
  #
  # Oracle recommends you create each cross-connect in a
  # {CrossConnectGroup} so you can use link aggregation
  # with the connection.
  #
  # **Note:** If you're a provider who is setting up a physical connection to Oracle so customers
  # can use FastConnect over the connection, be aware that your connection is modeled the
  # same way as a colocated customer's (with `CrossConnect` and `CrossConnectGroup` objects, and so on).
  #
  # To use any of the API operations, you must be authorized in an IAM policy. If you're not authorized,
  # talk to an administrator. If you're an administrator who needs to write policies to give users access, see
  # [Getting Started with Policies](https://docs.cloud.oracle.com/iaas/Content/Identity/Concepts/policygetstarted.htm).
  #
  class VnMonitoring::Models::CrossConnect
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_PENDING_CUSTOMER = 'PENDING_CUSTOMER'.freeze,
      LIFECYCLE_STATE_PROVISIONING = 'PROVISIONING'.freeze,
      LIFECYCLE_STATE_PROVISIONED = 'PROVISIONED'.freeze,
      LIFECYCLE_STATE_INACTIVE = 'INACTIVE'.freeze,
      LIFECYCLE_STATE_TERMINATING = 'TERMINATING'.freeze,
      LIFECYCLE_STATE_TERMINATED = 'TERMINATED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment containing the cross-connect group.
    # @return [String]
    attr_accessor :compartment_id

    # The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the cross-connect group this cross-connect belongs to (if any).
    #
    # @return [String]
    attr_accessor :cross_connect_group_id

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # A user-friendly name. Does not have to be unique, and it's changeable.
    # Avoid entering confidential information.
    #
    # @return [String]
    attr_accessor :display_name

    # Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
    # Example: `{\"bar-key\": \"value\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # The cross-connect's Oracle ID (OCID).
    # @return [String]
    attr_accessor :id

    # The cross-connect's current state.
    # @return [String]
    attr_reader :lifecycle_state

    # The name of the FastConnect location where this cross-connect is installed.
    #
    # @return [String]
    attr_accessor :location_name

    # A string identifying the meet-me room port for this cross-connect.
    # @return [String]
    attr_accessor :port_name

    # The port speed for this cross-connect.
    #
    # Example: `10 Gbps`
    #
    # @return [String]
    attr_accessor :port_speed_shape_name

    # A reference name or identifier for the physical fiber connection that this cross-connect
    # uses.
    #
    # @return [String]
    attr_accessor :customer_reference_name

    # The date and time the cross-connect was created, in the format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).
    #
    # Example: `2016-08-25T21:10:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :time_created

    # @return [OCI::VnMonitoring::Models::MacsecProperties]
    attr_accessor :macsec_properties

    # The FastConnect device that terminates the physical connection.
    #
    # @return [String]
    attr_accessor :oci_physical_device_name

    # The FastConnect device that terminates the logical connection.
    # This device might be different than the device that terminates the physical connection.
    #
    # @return [String]
    attr_accessor :oci_logical_device_name

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'compartmentId',
        'cross_connect_group_id': :'crossConnectGroupId',
        'defined_tags': :'definedTags',
        'display_name': :'displayName',
        'freeform_tags': :'freeformTags',
        'id': :'id',
        'lifecycle_state': :'lifecycleState',
        'location_name': :'locationName',
        'port_name': :'portName',
        'port_speed_shape_name': :'portSpeedShapeName',
        'customer_reference_name': :'customerReferenceName',
        'time_created': :'timeCreated',
        'macsec_properties': :'macsecProperties',
        'oci_physical_device_name': :'ociPhysicalDeviceName',
        'oci_logical_device_name': :'ociLogicalDeviceName'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'String',
        'cross_connect_group_id': :'String',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'display_name': :'String',
        'freeform_tags': :'Hash<String, String>',
        'id': :'String',
        'lifecycle_state': :'String',
        'location_name': :'String',
        'port_name': :'String',
        'port_speed_shape_name': :'String',
        'customer_reference_name': :'String',
        'time_created': :'DateTime',
        'macsec_properties': :'OCI::VnMonitoring::Models::MacsecProperties',
        'oci_physical_device_name': :'String',
        'oci_logical_device_name': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :cross_connect_group_id The value to assign to the {#cross_connect_group_id} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :location_name The value to assign to the {#location_name} property
    # @option attributes [String] :port_name The value to assign to the {#port_name} property
    # @option attributes [String] :port_speed_shape_name The value to assign to the {#port_speed_shape_name} property
    # @option attributes [String] :customer_reference_name The value to assign to the {#customer_reference_name} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [OCI::VnMonitoring::Models::MacsecProperties] :macsec_properties The value to assign to the {#macsec_properties} property
    # @option attributes [String] :oci_physical_device_name The value to assign to the {#oci_physical_device_name} property
    # @option attributes [String] :oci_logical_device_name The value to assign to the {#oci_logical_device_name} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.cross_connect_group_id = attributes[:'crossConnectGroupId'] if attributes[:'crossConnectGroupId']

      raise 'You cannot provide both :crossConnectGroupId and :cross_connect_group_id' if attributes.key?(:'crossConnectGroupId') && attributes.key?(:'cross_connect_group_id')

      self.cross_connect_group_id = attributes[:'cross_connect_group_id'] if attributes[:'cross_connect_group_id']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.id = attributes[:'id'] if attributes[:'id']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.location_name = attributes[:'locationName'] if attributes[:'locationName']

      raise 'You cannot provide both :locationName and :location_name' if attributes.key?(:'locationName') && attributes.key?(:'location_name')

      self.location_name = attributes[:'location_name'] if attributes[:'location_name']

      self.port_name = attributes[:'portName'] if attributes[:'portName']

      raise 'You cannot provide both :portName and :port_name' if attributes.key?(:'portName') && attributes.key?(:'port_name')

      self.port_name = attributes[:'port_name'] if attributes[:'port_name']

      self.port_speed_shape_name = attributes[:'portSpeedShapeName'] if attributes[:'portSpeedShapeName']

      raise 'You cannot provide both :portSpeedShapeName and :port_speed_shape_name' if attributes.key?(:'portSpeedShapeName') && attributes.key?(:'port_speed_shape_name')

      self.port_speed_shape_name = attributes[:'port_speed_shape_name'] if attributes[:'port_speed_shape_name']

      self.customer_reference_name = attributes[:'customerReferenceName'] if attributes[:'customerReferenceName']

      raise 'You cannot provide both :customerReferenceName and :customer_reference_name' if attributes.key?(:'customerReferenceName') && attributes.key?(:'customer_reference_name')

      self.customer_reference_name = attributes[:'customer_reference_name'] if attributes[:'customer_reference_name']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.macsec_properties = attributes[:'macsecProperties'] if attributes[:'macsecProperties']

      raise 'You cannot provide both :macsecProperties and :macsec_properties' if attributes.key?(:'macsecProperties') && attributes.key?(:'macsec_properties')

      self.macsec_properties = attributes[:'macsec_properties'] if attributes[:'macsec_properties']

      self.oci_physical_device_name = attributes[:'ociPhysicalDeviceName'] if attributes[:'ociPhysicalDeviceName']

      raise 'You cannot provide both :ociPhysicalDeviceName and :oci_physical_device_name' if attributes.key?(:'ociPhysicalDeviceName') && attributes.key?(:'oci_physical_device_name')

      self.oci_physical_device_name = attributes[:'oci_physical_device_name'] if attributes[:'oci_physical_device_name']

      self.oci_logical_device_name = attributes[:'ociLogicalDeviceName'] if attributes[:'ociLogicalDeviceName']

      raise 'You cannot provide both :ociLogicalDeviceName and :oci_logical_device_name' if attributes.key?(:'ociLogicalDeviceName') && attributes.key?(:'oci_logical_device_name')

      self.oci_logical_device_name = attributes[:'oci_logical_device_name'] if attributes[:'oci_logical_device_name']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        compartment_id == other.compartment_id &&
        cross_connect_group_id == other.cross_connect_group_id &&
        defined_tags == other.defined_tags &&
        display_name == other.display_name &&
        freeform_tags == other.freeform_tags &&
        id == other.id &&
        lifecycle_state == other.lifecycle_state &&
        location_name == other.location_name &&
        port_name == other.port_name &&
        port_speed_shape_name == other.port_speed_shape_name &&
        customer_reference_name == other.customer_reference_name &&
        time_created == other.time_created &&
        macsec_properties == other.macsec_properties &&
        oci_physical_device_name == other.oci_physical_device_name &&
        oci_logical_device_name == other.oci_logical_device_name
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [compartment_id, cross_connect_group_id, defined_tags, display_name, freeform_tags, id, lifecycle_state, location_name, port_name, port_speed_shape_name, customer_reference_name, time_created, macsec_properties, oci_physical_device_name, oci_logical_device_name].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
